<?php
namespace application\personal_calendar;

use common\libraries\Configuration;
use common\libraries\InCondition;
use common\libraries\Utilities;
use common\libraries\Filesystem;
use common\libraries\EqualityCondition;
use common\libraries\OrCondition;
use common\libraries\Request;
use common\libraries\WebApplication;
use common\libraries\NotCondition;
use common\libraries\AndCondition;

/**
 * $Id: personal_calendar_data_manager.class.php 127 2009-11-09 13:11:56Z vanpouckesven $
 * 
 * @package application.personal_calendar
 */

/**
 * This abstract class provides the necessary functionality to connect a personal calendar to a storage system.
 */
class PersonalCalendarDataManager
{

    /**
     * Instance of the class, for the singleton pattern.
     */
    private static $instance;

    /**
     * Constructor. Initializes the data manager.
     */
    public function __construct()
    {
        $this->initialize();
    }

    /**
     * - Creates the shared instance of the configured data manager if necessary and returns it. Uses a factory pattern.
     * 
     * @return PersonalCalendarDataManager The instance.
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'PersonalCalendarDataManager';
            self :: $instance = new $class();
        }
        return self :: $instance;
    }

    /**
     * Gets the events
     * 
     * @param $from_date int
     * @param $to_date int
     */
    public static function get_events($parent, $from_date, $to_date)
    {
        $events = self :: get_user_events($parent, $from_date, $to_date);
        $events = array_merge($events, self :: get_connector_events($parent, $from_date, $to_date));
        $events = array_merge($events, self :: get_user_shared_events($parent, $from_date, $to_date));
        return $events;
    }

    public static function get_user_events($parent, $from_date, $to_date)
    {
        $dm = PersonalCalendarDatamanager :: get_instance();
        $condition = new EqualityCondition(PersonalCalendarPublication :: PROPERTY_PUBLISHER, 
                $parent->get_user()->get_id());
        $publications = $dm->retrieve_personal_calendar_publications($condition);
        
        return self :: render_personal_calendar_events($parent, $publications, $from_date, $to_date);
    }

    public static function get_connector_events($parent, $from_date, $to_date)
    {
        $events = array();
        
        $path = dirname(__FILE__) . '/connector/';
        $files = Filesystem :: get_directory_content($path, Filesystem :: LIST_FILES, false);
        foreach ($files as $file)
        {
//            $application = str_replace('_connector.class.php', '', $file);
//            $application = str_replace(PersonalCalendarManager :: APPLICATION_NAME . '_', '', $application);
//            $application = Utilities :: camelcase_to_underscores($application);
//            
//            if (WebApplication :: is_active(WebApplication :: get_application_namespace($application)))
//            {
//                $file_class = split('.class.php', $file);
//                $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($file_class[0]);
//                
//                $connector = new $class();
//                $events = array_merge($events, $connector->get_events($parent->get_user(), $from_date, $to_date));
//            }
//            
                $file_class = split('.class.php', $file);
                require_once $path . $file;
                $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($file_class[0]);

                $connector = new $class();

                //ask connector of the connected application is active instead of getting application from connector file name -> e.g. different connectors for different tools of the weblcms,

                if($connector->is_active())
                {
                     $events = array_merge($events, $connector->get_events($parent->get_user(), $from_date, $to_date));
                }
        }
        
        return $events;
    }

    public static function get_user_shared_events($parent, $from_date, $to_date)
    {
        $events = array();
        $user_groups = $parent->get_user()->get_groups(true);
        
        $pcdm = PersonalCalendarDatamanager :: get_instance();
        $conditions = array();
        $conditions[] = new EqualityCondition('user_id', $parent->get_user()->get_id(), 'publication_user');
        if (count($user_groups) > 0)
        {
            $conditions[] = new InCondition('group_id', $user_groups, 'publication_group');
        }
        $share_condition = new OrCondition($conditions);
        
        $publisher_condition = new NotCondition(
                new EqualityCondition(PersonalCalendarPublication :: PROPERTY_PUBLISHER, $parent->get_user()->get_id()));
        
        $condition = new AndCondition($share_condition, $publisher_condition);
        $publications = $pcdm->retrieve_shared_personal_calendar_publications($condition);
        
        return self :: render_personal_calendar_events($parent, $publications, $from_date, $to_date, 'SharedEvents');
    }

    public function render_personal_calendar_events($parent, $publications, $from_date, $to_date, 
            $source = PersonalCalendarManager :: APPLICATION_NAME)
    {
        $events = array();
        $query = Request :: post('query');
        
        while ($publication = $publications->next_result())
        {
            $parser = PersonalCalendarEventParser :: factory($parent, $publication, $from_date, $to_date);
            $events = array_merge($events, $parser->get_events());
        }
        return $events;
    }
}
