<?php
namespace application\personal_messenger\integration\core\home;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\Redirect;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * Abstract class representing a personal messenger block to use on the home page.
 *
 * @version 1.0.0
 * @author Stijn Van Hoecke
 */
abstract class Block extends \core\home\Block
{
    // **************************************************************************
    // GENERAL CONSTANTS
    // **************************************************************************
    /**
     * The separator used to separate fields in the display.
     */
    const SEPARATOR = '-';

    /**
     * The number of messages to be displayed.
     */
    const NUMBER_OF_MESSAGES = 5;

    // **************************************************************************
    // GENERAL FUNCTIONS
    // **************************************************************************
    /**
     * Retrieves and displays the messages depending on the return values of the abstract functions.
     *
     * @return String HTML representing the messages
     */
    public function display_messages()
    {
        $html = array();

        // get the basic conditions for the inbox
        $condition = \application\personal_messenger\PersonalMessengerFolder :: get_conditions($this->get_folder(), - 1);

        // add additional conditions
        if ($this->new_only())
        {
            $conditions = array();
            $conditions[] = $condition;
            $conditions[] = new EqualityCondition(
                    \application\personal_messenger\UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                    \application\personal_messenger\UserMessage :: STATUS_UNREAD,
                    \application\personal_messenger\PersonalMessengerDataManager :: get_instance()->get_alias(
                            \application\personal_messenger\UserMessage :: get_table_name()), true);
            $condition = new AndCondition($conditions);
        }

        // construct the order
        $order = array(
                new ObjectTableOrder(\application\personal_messenger\Message :: PROPERTY_MESSAGE_MODIFICATION_TIME,
                        SORT_DESC,
                        \application\personal_messenger\PersonalMessengerDataManager :: get_instance()->get_alias(
                                \application\personal_messenger\Message :: get_table_name())));

        // get the messages
        $dm = \application\personal_messenger\PersonalMessengerDataManager :: get_instance();
        $messages = $dm->retrieve_user_messages($condition, $order, 0, self :: NUMBER_OF_MESSAGES);

        // display each message
        if ($messages->size() > 0)
        {
            while ($message_with_optional_properties = $messages->next_result())
            {
                $html[] = $this->get_message_html($message_with_optional_properties, $this->show_content());
            }
        }
        else
        {
            $html[] = Translation :: get('NoMessages');
        }

        return implode("\n", $html);
    }

    /**
     * Displays a single given message.
     *
     * @param Message $message_with_optional_properties The message
     * @param boolean $show_content Indicates if the content of the message should be displayed
     * @return String HTML representing the message
     */
    public function get_message_html($message_with_optional_properties, $show_content)
    {
        $html = array();

        // image
        $html[] = '<img width="15" height="15" src="';
        $html[] = \application\personal_messenger\UserMessage :: get_status_icon_path(
                $message_with_optional_properties->get_optional_property(
                        \application\personal_messenger\UserMessage :: PROPERTY_USER_MESSAGE_STATUS));
        $html[] = '" alt="';
        $html[] = Translation :: get(
                Utilities :: underscores_to_camelcase(
                        \application\personal_messenger\UserMessage :: get_status_name(
                                $message_with_optional_properties->get_optional_property(
                                        \application\personal_messenger\UserMessage :: PROPERTY_USER_MESSAGE_STATUS))));
        $html[] = '" />';

        // link
        $target = $this->get_link_target();
        $target = $target ? ' target="' . $target . '" ' : '';
        $html[] = '<a href="' . $this->get_message_viewing_link($message_with_optional_properties) . '"' . $target . '>';

        // sender
        $html[] = Utilities :: truncate_string($message_with_optional_properties->get_author_name(), 25);
        $html[] = self :: SEPARATOR;

        // title
        $html[] = Utilities :: truncate_string($message_with_optional_properties->get_title(), 25);
        $html[] = '</a><br />';

        if ($show_content)
        {
            $html[] = '<i>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
            $html[] = Utilities :: truncate_string($message_with_optional_properties->get_content(), 50);
            $html[] = '</i>';
            $html[] = '<br />	';
        }

        return implode("\n", $html);
    }

    /**
     * Returns a link to the viewer of the message.
     *
     * @param Message $message
     * @return String The link to the viewer of the message
     */
    public function get_message_viewing_link($message)
    {
        // construct viewing parameters
        $params = array();
        $params[\application\personal_messenger\PersonalMessengerManager :: PARAM_FOLDER] = $this->get_folder();
        $params[\application\personal_messenger\PersonalMessengerManager :: PARAM_MESSAGE_ID] = $message->get_id();

        if ($message->is_discussion())
        {
            $params[\application\personal_messenger\PersonalMessengerManager :: PARAM_ACTION] = \application\personal_messenger\PersonalMessengerManager :: ACTION_VIEW_DISCUSSION;
        }
        else
        {
            $params[\application\personal_messenger\PersonalMessengerManager :: PARAM_ACTION] = \application\personal_messenger\PersonalMessengerManager :: ACTION_VIEW_MESSAGE;
        }

        // return a link for the Personal Messenger application, with the given parameters
        return Redirect :: get_link(\application\personal_messenger\PersonalMessengerManager :: APPLICATION_NAME,
                $params, array(), false, Redirect :: TYPE_APPLICATION);
    }

    // **************************************************************************
    // ABSTRACT FUNCTIONS
    // **************************************************************************
    /**
     * Indicates which folder should be displayed.
     */
    abstract public function get_folder();

    /**
     * Indicates if the block should display new messages only.
     */
    abstract public function new_only();

    /**
     * Indicates if the block should display the content of the message.
     */
    abstract public function show_content();
}
