<?php

namespace application\personal_messenger;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Mdb2Database;
use group\Group;
use user\User;
use user\UserDataManager;

/** * ***************************************************************************
 * This class is the actual implementation for the data provider: mdb2 database.
 *
 * @name Chamilo Personal Messenger > Mdb2PersonalMessengerDataManager
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class Mdb2PersonalMessengerDataManager extends Mdb2Database implements PersonalMessengerDataManagerInterface
{

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /*
     * Inherited from PersonalMessengerDataManagerInterface:
     * Initializes the data manager.
     */
    public function initialize()
    {
        parent :: initialize();

        // set the prefix for all database tables
        $this->set_prefix('personal_messenger_');
    }

    //**************************************************************************
    // MESSAGE FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Count the messages based on a given condition.
     * @param Condition $condition
     * @return int The number of messages
     */
    public function count_message($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(Message :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the message.
     * @param Message $message The message
     * @return boolean
     */
    public function create_message($message)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object and automatically
        // assign a number as primary key ID to it.
        return $this->create($message);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single message.
     * @param Condition $condition The condition
     * @return Message
     */
    public function retrieve_message($condition = null)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(Message :: get_table_name(), $condition, array(), Message :: CLASS_NAME);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of messages.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageResultSet
     */
    public function retrieve_all_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                Message :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                Message :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Updates the given message.
     * @param Message $message The message
     * @return boolean
     */
    public function update_message($message)
    {
        // create an EqualityCondition identifying the message (ID)
        $condition = new EqualityCondition(
                        Message :: PROPERTY_ID,
                        $message->get_id()
        );

        // call to the method 'update(object, condition)' in superclass 'database'
        return $this->update($message, $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given message.
     * @param Message $message The message
     * @return boolean
     */
    public function delete_message($message)
    {
        // create an EQUALITY identifying the message (ID)
        $condition = new EqualityCondition(
                        Message :: PROPERTY_ID,
                        $message->get_id()
        );

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(Message :: get_table_name(), $condition);
    }

    //**************************************************************************
    // MESSAGE ATTACHMENT FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Count the message attachments based on a given condition.
     * @param Condition $condition
     * @return int The number of message attachments
     */
    public function count_message_attachment($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(MessageAttachment :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the message attachment
     * @param MessageAttachment $message_attachment The message attachment
     * @return boolean
     */
    public function create_message_attachment($message_attachment)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object but restricts generating
        // a numeric key because it is a linking table where ID's are known
        return $this->create($message_attachment, false);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single message attachment.
     * @param Condition $condition The condition
     * @return MessageAttachment
     */
    public function retrieve_message_attachment($condition = null)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(
                MessageAttachment :: get_table_name(),
                $condition,
                array(),
                MessageAttachment :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of message attachments.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageAttachmentResultSet
     */
    public function retrieve_all_message_attachments($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                MessageAttachment :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                MessageAttachment :: CLASS_NAME
        );
    }

    // !!! FEATURE DISABLED IN INTERFACE -> message_attachment is a linking table with only primary key, no update possible !!!

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given message attachment.
     * @param MessageAttachment $message_attachment The message attachment
     * @return boolean
     */
    public function delete_message_attachment($message_attachment)
    {
        // create an EQUALITY identifying the message attachment using message_id AND content_object_id
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID,
                        $message_attachment->get_message_id()
        );
        $conditions[] = new EqualityCondition(
                        MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_CONTENT_OBJECT_ID,
                        $message_attachment->get_content_object_id()
        );
        $condition = new AndCondition($conditions);

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(MessageAttachment :: get_table_name(), $condition);
    }

    //**************************************************************************
    // MESSAGE_GROUP_RECIPIENT FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Count the group relations based on a given condition.
     * @param Condition $condition
     * @return int The number of groups related to the message
     */
    public function count_message_group_recipient($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(MessageGroupRecipient :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the group relation
     * @param MessageGroupRecipient $message_group_recipient The group relation
     * @return boolean
     */
    public function create_message_group_recipient($message_group_recipient)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object but restricts generating
        // a numeric key because it is a linking table where ID's are known
        return $this->create($message_group_recipient, false);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single group relation.
     * @param Condition $condition The condition
     * @return MessageGroupRecipient
     */
    public function retrieve_message_group_recipient($condition = null)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(
                MessageGroupRecipient :: get_table_name(),
                $condition,
                array(),
                MessageGroupRecipient :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of group relations.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageGroupRecipientResultSet
     */
    public function retrieve_all_message_group_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                MessageGroupRecipient :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                MessageGroupRecipient :: CLASS_NAME
        );
    }

    // !!! FEATURE DISABLED IN INTERFACE -> message_group_recipient is a linking table with only primary key, no update possible !!!

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given group relation.
     * @param MessageGroupRecipient $message_group_recipient The group relation
     * @return boolean
     */
    public function delete_message_group_recipient($message_group_recipient)
    {
        // create an EQUALITY identifying the group relation using message_id AND group_id
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        MessageGroupRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID,
                        $message_group_recipient->get_message_id()
        );
        $conditions[] = new EqualityCondition(
                        MessageGroupRecipient :: PROPERTY_MESSAGE_GROUP_RECIPIENT_GROUP_ID,
                        $message_group_recipient->get_group_id()
        );
        $condition = new AndCondition($conditions);

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(MessageGroupRecipient :: get_table_name(), $condition);
    }

    //**************************************************************************
    // MESSAGE_REL_USER FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Counts the user relations based on a given condition.
     * @param Condition $condition
     * @return int The number of users related to the message
     */
    public function count_message_user_recipient($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(MessageUserRecipient :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the user relation
     * @param MessageUserRecipient $message_user_recipient The user relation
     * @return boolean
     */
    public function create_message_user_recipient($message_user_recipient)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object but restricts generating
        // a numeric key because it is a linking table where ID's are known
        return $this->create($message_user_recipient, false);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single user relation.
     * @param Condition $condition The condition
     * @return MessageUserRecipient
     */
    public function retrieve_message_user_recipient($condition = null)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(
                MessageUserRecipient :: get_table_name(),
                $condition,
                array(),
                MessageUserRecipient :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of user relations.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageUserRecipientResultSet
     */
    public function retrieve_all_message_user_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                MessageUserRecipient :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                MessageUserRecipient :: CLASS_NAME
        );
    }

    // !!! FEATURE DISABLED IN INTERFACE -> message_user_recipient is a linking table with only primary key, no update

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given user relation.
     * @param MessageUserRecipient $message_user_recipient The user relation
     * @return boolean
     */
    public function delete_message_user_recipient($message_user_recipient)
    {
        // create an EQUALITY identifying the user relation using message_id AND user_id
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        MessageUserRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID,
                        $message_user_recipient->get_message_id()
        );
        $conditions[] = new EqualityCondition(
                        MessageUserRecipient :: PROPERTY_MESSAGE_USER_RECIPIENT_USER_ID,
                        $message_user_recipient->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(MessageUserRecipient :: get_table_name(), $condition);
    }

    //**************************************************************************
    // USER_MESSAGE FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Counts the user messages based on a given condition.
     * @param Condition $condition
     * @return int The number of user message records
     */
    public function count_user_message($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(UserMessage :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the user message
     * @param UserMessage $user_message The user message
     * @return boolean
     */
    public function create_user_message($user_message)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object but restricts generating
        // a numeric key because it is a linking table where ID's are known
        return $this->create($user_message, false);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single user message.
     * @param Condition $condition The condition
     * @return UserMessage
     */
    public function retrieve_user_message($condition)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(
                UserMessage :: get_table_name(),
                $condition,
                array(),
                UserMessage :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of user messages.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return UserMessageResultSet
     */
    public function retrieve_all_user_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                UserMessage :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                UserMessage :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Updates the given user message.
     * @param UserMessage $user_message The user message
     * @return boolean
     */
    public function update_user_message($user_message)
    {
        // create an EQUALITY identifying the user relation folder using message_id AND user_id
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $user_message->get_message_id()
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $user_message->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // call to the method 'update(object, condition)' in superclass 'database'
        return $this->update($user_message, $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given user message.
     * @param UserMessage $user_message The user message
     * @return boolean
     */
    public function delete_user_message($user_message)
    {
        // create an EQUALITY identifying the user relation category using message_id AND user_id
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $user_message->get_message_id()
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $user_message->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(UserMessage :: get_table_name(), $condition);
    }

    //**************************************************************************
    // USER_CATEGORY FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Counts the user categories based on a given condition.
     * @param Condition $condition
     * @return int The number of user_categories
     */
    public function count_user_category($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(UserCategory :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the user category
     * @param UserCategory $user_category The user category
     * @return boolean
     */
    public function create_user_category($user_category)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object and automatically
        // assign a number as primary key ID to it.
        return $this->create($user_category);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single user category.
     * @param Condition $condition The condition
     * @return UserCategory
     */
    public function retrieve_user_category($condition = null)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(
                UserCategory :: get_table_name(),
                $condition,
                array(),
                UserCategory :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of user categories.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return UserCategoryResultSet
     */
    public function retrieve_all_user_categories($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                UserCategory :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                UserCategory :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Updates the given user category.
     * @param UserCategory $user_category The user category
     * @return boolean
     */
    public function update_user_category($user_category)
    {
        // create an EqualityCondition identifying the message (ID)
        $condition = new EqualityCondition(
                        UserCategory :: PROPERTY_ID,
                        $user_category->get_id()
        );

        // call to the method 'update(object, condition)' in superclass 'database'
        return $this->update($user_category, $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given user category.
     * @param UserCategory $user_category The user category
     * @return boolean
     */
    public function delete_user_category($user_category)
    {
        // create an EQUALITY identifying the message (ID)
        $condition = new EqualityCondition(
                        UserCategory :: PROPERTY_ID,
                        $user_category->get_id()
        );

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(UserCategory :: get_table_name(), $condition);
    }

    //**************************************************************************
    // USER_IGNORE FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Counts the ignored users based on a given condition.
     * @param Condition $condition
     * @return int The number of ignored users
     */
    public function count_user_ignore($condition = null)
    {
        // call to the method 'count_objects(table_name, condition)' in superclass 'database'
        return $this->count_objects(UserIgnore :: get_table_name(), $condition);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Creates the ignored user
     * @param UserIgnore $user_ignore The ignored user
     * @return boolean
     */
    public function create_user_ignore($user_ignore)
    {
        // call to the method 'create(object)' in superclass 'database'
        // this method will create the object but restricts generating
        // a numeric key because it is a linking table where ID's are known
        return $this->create($user_ignore, false);
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a single ignored user.
     * @param Condition $condition The condition
     * @return UserIgnore
     */
    public function retrieve_user_ignore($condition = null)
    {
        // call to the method 'retrieve_object(table_name, condition, order_by, class_name)' in superclass 'database'
        return $this->retrieve_object(
                UserIgnore :: get_table_name(),
                $condition,
                array(),
                UserIgnore :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves a series of ignored users.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return UserIgnoreResultSet
     */
    public function retrieve_all_user_ignores($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        // call to the method 'retrieve_objects(table_name, ...)' in superclass 'database'
        return $this->retrieve_objects(
                UserIgnore :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                UserIgnore :: CLASS_NAME
        );
    }

    // !!! FEATURE DISABLED IN INTERFACE -> message_attachment is a linking table with only primary key, no update possible !!!

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Deletes the given ignored user.
     * @param UserIgnore $user_ignore The ignored user
     * @return boolean
     */
    public function delete_user_ignore($user_ignore)
    {
        // create an EQUALITY identifying the user ignore using user_id AND ignored_user_id
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserIgnore :: PROPERTY_USER_IGNORE_USER_ID,
                        $user_ignore->get_user_id()
        );
        $conditions[] = new EqualityCondition(
                        UserIgnore :: PROPERTY_USER_IGNORE_IGNORED_USER_ID,
                        $user_ignore->get_ignored_user_id()
        );
        $condition = new AndCondition($conditions);

        // call the method 'delete(table_name, condition)' in superclass 'database'
        return $this->delete(UserIgnore :: get_table_name(), $condition);
    }

    //**************************************************************************
    // ADVANCED FUNCTIONS
    //**************************************************************************
    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves all messages for a given user, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageResultSet
     */
    public function retrieve_user_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $user_dm = UserDatamanager :: get_instance();
        $query = 'SELECT ' . $this->get_alias(Message :: get_table_name()) . '.*';
        $query .= ', ' . $this->get_alias(UserMessage :: get_table_name()) . '.*';
        $query .= ', ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_FIRSTNAME;
        $query .= ', ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_LASTNAME;

        $query .= ' FROM ' . $this->escape_table_name(Message :: get_table_name());
        $query .= ' AS ' . $this->get_alias(Message :: get_table_name());

        $query .= ' RIGHT JOIN ' . $this->escape_table_name(UserMessage :: get_table_name());
        $query .= ' AS ' . $this->get_alias(UserMessage :: get_table_name());
        $query .= ' ON ' . $this->get_alias(Message :: get_table_name()) . '.' . Message :: PROPERTY_ID;
        $query .= ' = ' . $this->get_alias(UserMessage :: get_table_name()) . '.' . UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID;

        $query .= ' LEFT JOIN ' . $user_dm->escape_table_name(User :: get_table_name());
        $query .= ' AS ' . $user_dm->get_alias(User :: get_table_name());
        $query .= ' ON ' . $this->get_alias(Message :: get_table_name()) . '.' . Message :: PROPERTY_MESSAGE_AUTHOR_ID;
        $query .= ' = ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_ID;

        return $this->retrieve_object_set(
                $query,
                Message :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                Message :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Counts all messages for a given user, depending on the given conditions.
     * @param Condition $condition
     * @return int
     */
    public function count_user_messages($condition = null)
    {
        $user_dm = UserDatamanager :: get_instance();
        $query = 'SELECT COUNT(*)';

        $query .= ' FROM ' . $this->escape_table_name(Message :: get_table_name());
        $query .= ' AS ' . $this->get_alias(Message :: get_table_name());

        $query .= ' RIGHT JOIN ' . $this->escape_table_name(UserMessage :: get_table_name());
        $query .= ' AS ' . $this->get_alias(UserMessage :: get_table_name());
        $query .= ' ON ' . $this->get_alias(Message :: get_table_name()) . '.' . Message :: PROPERTY_ID;
        $query .= ' = ' . $this->get_alias(UserMessage :: get_table_name()) . '.' . UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID;

        return $this->count_result_set(
                $query,
                Message :: get_table_name(),
                $condition
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves all user recipients, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageUserRecipientResultSet
     */
    public function retrieve_message_user_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $user_dm = UserDatamanager :: get_instance();
        $query = 'SELECT ' . $this->get_alias(MessageUserRecipient :: get_table_name()) . '.*';
        $query .= ', ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_FIRSTNAME;
        $query .= ', ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_LASTNAME;

        $query .= ' FROM ' . $this->escape_table_name(MessageUserRecipient :: get_table_name());
        $query .= ' AS ' . $this->get_alias(MessageUserRecipient :: get_table_name());

        $query .= ' LEFT JOIN ' . $user_dm->escape_table_name(User :: get_table_name());
        $query .= ' AS ' . $user_dm->get_alias(User :: get_table_name());
        $query .= ' ON ' . $this->get_alias(MessageUserRecipient :: get_table_name()) . '.' . MessageUserRecipient :: PROPERTY_MESSAGE_USER_RECIPIENT_USER_ID;
        $query .= ' = ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_ID;

        return $this->retrieve_object_set(
                $query,
                MessageUserRecipient :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                MessageUserRecipient :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves all group recipients, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageGroupRecipientResultSet
     */
    public function retrieve_message_group_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $group_dm = \group\DataManager :: get_instance();
        $query = 'SELECT ' . $this->get_alias(MessageGroupRecipient :: get_table_name()) . '.*';
        $query .= ', ' . $group_dm->get_alias(Group :: get_table_name()) . '.' . Group :: PROPERTY_NAME;

        $query .= ' FROM ' . $this->escape_table_name(MessageGroupRecipient :: get_table_name());
        $query .= ' AS ' . $this->get_alias(MessageGroupRecipient :: get_table_name());

        $query .= ' LEFT JOIN ' . $group_dm->escape_table_name(Group :: get_table_name());
        $query .= ' AS ' . $group_dm->get_alias(Group :: get_table_name());
        $query .= ' ON ' . $this->get_alias(MessageGroupRecipient :: get_table_name()) . '.' . MessageGroupRecipient :: PROPERTY_MESSAGE_GROUP_RECIPIENT_GROUP_ID;
        $query .= ' = ' . $group_dm->get_alias(Group :: get_table_name()) . '.' . Group :: PROPERTY_ID;

        return $this->retrieve_object_set(
                $query,
                MessageGroupRecipient :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                MessageGroupRecipient :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves all ignored users, depending on the given conditions.
     * @param Condition $condition
     * @param array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return UserResultSet
     */
    public function retrieve_ignored_user_users($condition, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $user_dm = UserDatamanager :: get_instance();
        $query = 'SELECT ' . $user_dm->get_alias(User :: get_table_name()) . '.*';

        $query .= ' FROM ' . $user_dm->escape_table_name(User :: get_table_name());
        $query .= ' AS ' . $user_dm->get_alias(User :: get_table_name());

        $query .= ' RIGHT JOIN ' . $this->escape_table_name(UserIgnore :: get_table_name());
        $query .= ' AS ' . $this->get_alias(UserIgnore :: get_table_name());
        $query .= ' ON ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_ID;
        $query .= ' = ' . $this->get_alias(UserIgnore :: get_table_name()) . '.' . UserIgnore :: PROPERTY_USER_IGNORE_IGNORED_USER_ID;

        return $this->retrieve_object_set(
                $query,
                User :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                User :: CLASS_NAME
        );
    }

    /**
     * Inherited from PersonalMessengerDataManagerInterface:
     * Retrieves all users currently active in a message, depending on the given conditions.
     * @param Condition $condition
     * @param array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return UserResultSet
     */
    public function retrieve_user_message_users($condition, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $user_dm = UserDatamanager :: get_instance();
        $query = 'SELECT ' . $user_dm->get_alias(User :: get_table_name()) . '.*';

        $query .= ' FROM ' . $user_dm->escape_table_name(User :: get_table_name());
        $query .= ' AS ' . $user_dm->get_alias(User :: get_table_name());

        $query .= ' RIGHT JOIN ' . $this->escape_table_name(UserMessage :: get_table_name());
        $query .= ' AS ' . $this->get_alias(UserMessage :: get_table_name());
        $query .= ' ON ' . $user_dm->get_alias(User :: get_table_name()) . '.' . User :: PROPERTY_ID;
        $query .= ' = ' . $this->get_alias(UserMessage :: get_table_name()) . '.' . UserMessage :: PROPERTY_USER_MESSAGE_USER_ID;

        return $this->retrieve_object_set(
                $query,
                User :: get_table_name(),
                $condition,
                $offset,
                $max_objects,
                $order_by,
                User :: CLASS_NAME
        );
    }

}
