<?php

namespace application\personal_messenger;

use common\libraries\AndCondition;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\ContentObjectDisplay;

/** * ***************************************************************************
 * This component offers the functionality to view an attachment of a message.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerAttachmentViewerComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerManagerAttachmentViewerComponent extends PersonalMessengerManager
{

    /**
     * Indicates if the viewer is openend in a popup, or fullscreen.
     * @var <type> 
     */
    private $popup;
    /**
     * The message to return to.
     * @var Message
     */
    private $message;

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        // get attachment
        $attachment = $this->get_attachment();

        $this->popup = Request :: get(self :: PARAM_POPUP);

        // display all
        if (!$this->popup)
        {
            $this->display_header();
        }
        else
        {
            Display :: small_header();
        }

        echo $this->get_attachment_as_html($attachment);

        if (!$this->popup)
        {
            $this->display_footer();
        }
        else
        {
            Display :: small_footer();
        }
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the message attachment as a ContentObject after performing some checks.
     * @return ContentObject
     */
    public function get_attachment()
    {
        // get message id
        $message_id = Request :: get(self :: PARAM_MESSAGE_ID);
        if (!$message_id)
        {
            if (!$this->popup)
            {
                $this->display_header();
            }
            Display :: error_message(Translation :: get('NoMessageSelected'));
            if (!$this->popup)
            {
                $this->display_footer();
            }
            exit();
        }
        else
        {
            $this->message = $this->retrieve_message_by_id($message_id);
            if (!$this->message)
            {
                if (!$this->popup)
                {
                    $this->display_header();
                }
                Display :: error_message(Translation :: get('NoMessageSelected'));
                if (!$this->popup)
                {
                    $this->display_footer();
                }
                exit();
            }

            // if we have a discussion, compare to the root
            if ($this->message->is_discussion())
            {
                $this->message = $this->retrieve_message_by_id($this->message->get_parent_id());
            }

            // create user message conditions
            $conditions = array();
            $conditions[] = new EqualityCondition(
                            UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                            $this->message->get_id()
            );
            $conditions[] = new EqualityCondition(
                            UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                            $this->get_user_id()
            );
            $user_message_condition = new AndCondition($conditions);

            // retrieve the user message
            $user_message = $this->retrieve_user_message($user_message_condition);

            // check if there is a user message
            if (!$user_message)
            {
                if (!$this->popup)
                {
                    $this->display_header();
                }
                Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                if (!$this->popup)
                {
                    $this->display_footer();
                }
                exit();
            }
            else
            {
                $attachment_id = Request :: get(self :: PARAM_ATTACHMENT_ID);

                // create attachment conditions
                $conditions = array();
                $conditions[] = new EqualityCondition(
                                MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID,
                                $message_id
                );
                $conditions[] = new EqualityCondition(
                                MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_CONTENT_OBJECT_ID,
                                $attachment_id
                );
                $attachment_condition = new AndCondition($conditions);

                // retrieve the attachment
                $attachment = $this->retrieve_attachment($attachment_condition);

                if (!$attachment)
                {
                    if (!$this->popup)
                    {
                        $this->display_header();
                    }
                    Display :: error_message(Translation :: get('NoAttachmentToDisplay'));
                    if (!$this->popup)
                    {
                        $this->display_footer();
                    }
                    exit();
                }
                elseif (!$attachment->ensure())
                {
                    if (!$this->popup)
                    {
                        $this->display_header();
                    }
                    Display :: error_message(Translation :: get('AttachmentDeleted'));
                    if (!$this->popup)
                    {
                        $this->display_footer();
                    }
                    exit();
                }
                else
                {
                    return $attachment->retieve_attachment_content_object();
                }
            }
        }
    }

    /**
     * Renders the given content object (attachment) and returns its HTML.
     * @param ContentObject $attachment
     * @return String HTML
     */
    public function get_attachment_as_html($attachment)
    {
        $html = array();
        $display = ContentObjectDisplay :: factory($attachment);
        $html[] = $display->get_full_html();

        return implode("\n", $html);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_MESSAGE_ID, self :: PARAM_ATTACHMENT_ID, self :: PARAM_POPUP);
    }

    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ))));

        // message
        $message = $this->retrieve_message_by_id(Request :: get(self :: PARAM_MESSAGE_ID));
        if ($message)
        {
            $message_url = $this->get_message_view_url($message);
            $breadcrumbtrail->add(new Breadcrumb(
                            $message_url,
                            Translation :: get(
                                    $message->is_discussion() ? 'PersonalMessengerManagerViewerDiscussionComponent' : 'PersonalMessengerManagerViewerMessageComponent'
                            )
            ));
        }
    }

}
