<?php

namespace application\personal_messenger;

use common\libraries\Application;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DatetimeUtilities;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This component offers the functionality to reply on a message.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerReplyComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @uses MessageForm
 * **************************************************************************** */
class PersonalMessengerManagerReplyComponent extends PersonalMessengerManager
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    /**
     * String put in front of the parent title to indicate it's a reply.
     */
    const FIXED_REPLY_TITLE_PREFIX = 'RE: ';

    /**
     * String put in front of the original message making it indent etc.
     */
    const FIXED_REPLY_MESSAGE_PREFIX = '<p></p><hr/><div>';
    const FIXED_REPLY_MESSAGE_PREFIX_INDENT = '<p></p><hr/><div STYLE="padding-left: 2em;">';

    /**
     * Close string depending on the FIXED_REPLY_MESSAGE_PREFIX.
     */
    const FIXED_REPLY_MESSAGE_SUFFIX = '</div>';

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * The message we send.
     * @var Message
     */
    private $message;
    /**
     * The message on which we reply.
     * @var Message
     */
    private $original_message;
    /**
     * Indicates if the original message should be included.
     * @var boolean
     */
    private $include_original_message;
    /**
     * Indicates if the original message should be indenten.
     * @var boolean
     */
    private $indent_original_message;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    public function __construct($user)
    {
        parent :: __construct($user);
        if (Request :: get(PersonalMessengerManager :: PARAM_INCLUDE_ORIGINAL_MESSAGE))
        {
            $this->include_original_message = Request :: get(PersonalMessengerManager :: PARAM_INCLUDE_ORIGINAL_MESSAGE);
        }
        else
        {
            $this->include_original_message = true;
        }

        if (Request :: get(PersonalMessengerManager :: PARAM_INDENT_ORIGINAL_MESSAGE))
        {
            $this->indent_original_message = Request :: get(PersonalMessengerManager :: PARAM_INDENT_ORIGINAL_MESSAGE);
        }
        else
        {
            $this->indent_original_message = true;
        }
    }

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component.
     */
    public function run()
    {
        // create a new message
        $this->message = new Message();
        $this->original_message = $this->get_message_by_id(Request :: get(self :: PARAM_REPLY_ID));

        // if there is no message
        if (!$this->original_message)
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NoMessageSelected'));
            $this->display_footer();
            exit();
        }
        else
        {
            // preset the title of the message
            $this->message->set_title($this->get_reply_title());

            // if the message is not a discussion, and we want to include the original message
            if (!$this->original_message->is_discussion() && $this->include_original_message)
            {
                $this->message->set_content($this->get_reply_message());
            }

            // preset IDs
            $this->message->set_reply_id(Request :: get(self :: PARAM_REPLY_ID));
            $this->message->set_parent_id(Request :: get(self :: PARAM_PARENT_ID));

            // preset the recipients of the message
            if (Request :: get(self :: PARAM_REPLY_TYPE) == MessageForm :: TYPE_REPLY)
            {
                $this->message->set_target_users(array($this->original_message->get_author_id()));
            }
            elseif (Request :: get(self :: PARAM_REPLY_TYPE) == MessageForm :: TYPE_REPLY_ALL)
            {
                $this->message->set_target_users($this->original_message->get_target_users());
                $this->message->set_target_groups($this->original_message->get_target_groups());
            }

            // create the form
            $form = new MessageForm(
                            Request :: get(self :: PARAM_REPLY_TYPE),
                            $this->message,
                            $this->get_url(),
                            $this->get_user(),
                            $this,
                            !$this->original_message->is_discussion(),
                            !$this->original_message->is_discussion()
            );

            if ($form->validate())
            {
                $success = $form->create_message();

                // go back
                $go_back_parameters = array();
                if (Request :: get(self :: PARAM_REFERER))
                {
                    $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
                    // in case of reply, if you have a referer it is the (viewer) of the original message
                    // -> return to the original message
                    $go_back_parameters[self :: PARAM_MESSAGE_ID] = $this->message->get_reply_id();
                }
                else
                {
                    $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
                }

                $this->redirect(
                        Translation :: get(
                                $success ? 'ObjectCreated' : 'ObjectNotCreated',
                                array('OBJECT' => Translation :: get('Reply')),
                                Utilities :: COMMON_LIBRARIES
                        ),
                        ($success ? false : true),
                        $go_back_parameters
                );
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Constructs a reply title based on the parent title.
     * @param Message $parent
     * @return String
     */
    public function get_reply_title()
    {
        if (!$this->original_message->is_discussion())
        {
            $reply_title = $this->original_message->get_title();
            if (substr($reply_title, 0, strlen(self :: FIXED_REPLY_TITLE_PREFIX)) != self :: FIXED_REPLY_TITLE_PREFIX)
            {
                $reply_title = self :: FIXED_REPLY_TITLE_PREFIX . $reply_title;
            }
        }
        else
        {
            $reply_title = '';
        }

        return $reply_title;
    }

    /**
     * Constructs a reply message based on the parent message.
     * @param Message $parent
     * @return String
     */
    public function get_reply_message()
    {
        if ($this->indent_original_message)
        {
            $reply_message .= self :: FIXED_REPLY_MESSAGE_PREFIX_INDENT;
        }
        else
        {
            $reply_message .= self :: FIXED_REPLY_MESSAGE_PREFIX;
        }

        $reply_message .= '<b>' . Translation :: get('OriginalMessage') . ': </b><i>';
        $reply_message .= $this->original_message->get_title() . '</i><br/>';
        $reply_message .= '<b>' . Translation :: get('MessageFrom') . ': </b><i>';
        $reply_message .= $this->original_message->get_author_name() . '</i><br/>';
        $reply_message .= '<b>' . Translation :: get('MessageTo') . ': </b><i>';
        $reply_message .= implode(', ', $this->original_message->get_recipient_names()) . '</i><br/>';
        $reply_message .= '<b>' . Translation :: get('MessageDate') . ': </b><i>';
        $reply_message .= DatetimeUtilities :: format_locale_date(
                        Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) .
                        ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $this->original_message->get_modification_time()
        );
        $reply_message .= '</i><br/>';
        $reply_message .= $this->original_message->get_content();
        $reply_message .= self :: FIXED_REPLY_MESSAGE_SUFFIX;

        return $reply_message;
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER,
            self :: PARAM_PARENT_ID,
            self :: PARAM_REPLY_ID,
            self :: PARAM_REPLY_TYPE,
            self :: PARAM_INCLUDE_ORIGINAL_MESSAGE,
            self :: PARAM_INDENT_ORIGINAL_MESSAGE
        );
    }

    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ))));

        // message
        $message = $this->retrieve_message_by_id(Request :: get(self :: PARAM_REPLY_ID));
        if ($message)
        {
            $message_url = $this->get_message_view_url($message);
            $breadcrumbtrail->add(new Breadcrumb(
                            $message_url,
                            Translation :: get(
                                    $message->is_discussion() ? 'PersonalMessengerManagerViewerDiscussionComponent' : 'PersonalMessengerManagerViewerMessageComponent'
                            )
            ));
        }
    }

}
