<?php

namespace application\personal_messenger;

use common\libraries\AndCondition;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use repository\ContentObject;
use repository\content_object\personal_message\PersonalMessage;
use common\libraries\package\PackageList;
use common\libraries\Theme;
use common\libraries\CommonDataManager;

/** * ***************************************************************************
 * This class is the hart of the personal messenger. It delegates all actions
 * to the proper components.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManager
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerManager extends WebApplication
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const APPLICATION_NAME = 'personal_messenger';

    //**************************************************************************
    // PARAMETER CONSTANTS
    //**************************************************************************
    const PARAM_ATTACHMENT_ID = 'attachment_id';
    const PARAM_CATEGORY_ID = 'cat_id';
    const PARAM_DELETE_SELECTED = 'delete_selected';
    const PARAM_FOLDER = 'folder';
    const PARAM_GROUP_ID = 'group_id';
    const PARAM_INCLUDE_ORIGINAL_MESSAGE = 'incl_orig';
    const PARAM_INDENT_ORIGINAL_MESSAGE = 'ind_orig';
    const PARAM_MARK_STATUS = 'status';
    const PARAM_MESSAGE_ID = 'message_id';
    const PARAM_MOVE_SELECTED = 'move_selected';
    const PARAM_PARENT_ID = 'parent_id';
    const PARAM_POPUP = 'pop';
    const PARAM_REFERER = 'referer';
    const PARAM_REPLY_ID = 'reply_id';
    const PARAM_REPLY_TYPE = 'reply_type';
    const PARAM_SWITCH_CURRENT_CATEGORY = 'switch_cat';
    const PARAM_SWITCH_CURRENT_FOLDER = 'switch_fol';
    const PARAM_USER_ID = 'user_id';

    //**************************************************************************
    // ACTION CONSTANTS
    //**************************************************************************
    const ACTION_ADD_USER = 'user_adder';
    const ACTION_BROWSE_MESSAGES = 'browser';
    const ACTION_CREATE_MESSAGE = 'creator';
    const ACTION_CREATE_MESSAGE_FROM_TEMPLATE = 'creator_from_template';
    const ACTION_CREATE_TEMPLATE = 'templator';
    const ACTION_DELETE_MESSAGE = 'deleter_message';
    const ACTION_DELETE_USER = 'user_deleter';
    const ACTION_DELETE_USER_MESSAGE = 'deleter_user_message';
    const ACTION_FORWARD = 'forwarder';
    const ACTION_IGNORE_USER = 'user_ignore';
    const ACTION_MANAGE_CATEGORIES = 'category_manager';
    const ACTION_MANAGE_USERS_DISCUSSION = 'user_manager_discussion';
    const ACTION_MANAGE_USERS_IGNORED = 'user_manager_ignored';
    const ACTION_MARK_MESSAGE_IGNORE = 'marker_ignore';
    const ACTION_MARK_MESSAGE_READ = 'marker_read';
    const ACTION_MARK_MESSAGE_TRASH = 'marker_trash';
    const ACTION_MARK_MESSAGE_UNIGNORE = 'marker_unignore';
    const ACTION_MARK_MESSAGE_UNREAD = 'marker_unread';
    const ACTION_MARK_MESSAGE_UNTRASH = 'marker_untrash';
    const ACTION_MOVE_MESSAGE = 'category_mover';
    const ACTION_REPLY = 'reply';
    const ACTION_UPDATE_MESSAGE = 'updater';
    const ACTION_UNIGNORE_USER = 'user_unignore';
    const ACTION_VIEW_MESSAGE = 'viewer_message';
    const ACTION_VIEW_DISCUSSION = 'viewer_discussion';
    const ACTION_VIEW_ATTACHMENT = 'attachment_viewer';

    //**************************************************************************
    // DEFAULTS
    //**************************************************************************
    const DEFAULT_ACTION = self :: ACTION_BROWSE_MESSAGES;

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * Associative array that stores the requested messages by ID. In one request
     * the messages can be cached here to avoid database overhead.
     * Because messages are stored by reference, they will always be up to date.
     * @var array<message_id, message>
     */
    private $messages;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     * @param User $user The current user
     */
    public function __construct($user = null)
    {
        parent :: __construct($user);

        // always give the folder as parameter
        $this->set_folder($this->get_folder());
        $this->set_category_id($this->get_category_id());
        $this->messages = array();
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Gets the folder.
     * If no folder is stored, the default folder is returned.
     * @return int
     */
    public function get_folder()
    {
        $folder = Request :: get(self :: PARAM_FOLDER);
        if ($folder)
        {
            return $folder;
        }
        else
        {
            return PersonalMessengerFolder :: DEFAULT_FOLDER;
        }
    }

    /**
     * Sets the folder.
     * If an invalid folder is specified the default folder is stored.
     * @param int $folder
     */
    public function set_folder($folder)
    {
        // check for valid folder, else just use default
        if (!PersonalMessengerFolder :: is_valid_folder($folder))
        {
            $folder = PersonalMessengerFolder :: DEFAULT_FOLDER;
        }

        // set the parameter
        $this->set_parameter(self :: PARAM_FOLDER, $folder);
    }

    /**
     * Gets the category_id.
     * If no category is stored, the default category (0) is returned.
     * @return int
     */
    public function get_category_id()
    {
        $category_id = Request :: get(self :: PARAM_CATEGORY_ID);
        if ($category_id)
        {
            return $category_id;
        }
        else
        {
            return 0;
        }
    }

    /**
     * Sets the category_id.
     * If no category is specified the default category (0) is stored.
     * @param int $category_id
     */
    public function set_category_id($category_id)
    {
        // check for category, else just use default
        if (!$category_id)
        {
            $category_id = 0;
        }

        // set the parameter
        $this->set_parameter(self :: PARAM_CATEGORY_ID, $category_id);
    }

    //**************************************************************************
    // SHARED FUNCTIONS
    //**************************************************************************
    /**
     * Converts a given message to a content object, registers the content object in the database
     * and returns the newly created content object.
     * @param Message $message
     * @return PersonalMessageContentObject $content_object
     */
    public function create_template($message)
    {
        $template_content_object = new PersonalMessage(array(
                    ContentObject :: PROPERTY_TYPE => PersonalMessage :: get_type_name(),
                    ContentObject :: PROPERTY_OWNER_ID => $this->get_user_id(),
                    ContentObject :: PROPERTY_TITLE => $message->get_title(),
                    ContentObject :: PROPERTY_DESCRIPTION => $message->get_content()
                ));
        return $template_content_object->create();
    }

    /**
     * Retrieves a single user message, depending on the given condition.
     * @param Condition $condition
     * @return UserMessage
     */
    public function retrieve_user_message($condition)
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        return $dm->retrieve_user_message($condition);
    }

    /**
     * Retrieves a single message, depending on the given message ID.
     * This method does not perform any checks, use get_message_by_id() instead
     * to check if the user is allowed to get the message.
     * @param int $message_id
     * @return Message
     * @see get_message_by_id()
     */
    public function retrieve_message_by_id($message_id)
    {
        if (!array_key_exists($message_id, $this->messages))
        {
            $message = PersonalMessengerDataManager :: retrieve_message_by_id($message_id);
            if ($message)
            {
                $this->messages[$message->get_id()] = $message;
            }
        }
        return $this->messages[$message_id];
    }

    /**
     * Retrieves a single message, depending on the given condition.
     * @param Condition $condition
     * @return Message
     */
    public function retrieve_message($condition)
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        return $dm->retrieve_message($condition);
    }

    /**
     * Retrieves all messages for a given user, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageResultSet
     */
    public function retrieve_user_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        return $dm->retrieve_user_messages($condition, $order_by, $offset, $max_objects);
    }

    /**
     * Count the message user recipients.
     * @param Condition $condition
     * @return int The number of messages
     */
    public function count_message_user_recipient($condition = null)
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        $count = $dm->count_message_user_recipient($condition);
        return $count;
    }

    /**
     * Retrieves a single message attachment, based on the given condition.
     * @param Condition $condition
     * @return MessageAttachment
     */
    public function retrieve_attachment($condition)
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        return $dm->retrieve_message_attachment($condition);
    }

    /**
     * Returns the message (!) after performing some checks (!).
     * @return Message
     * @uses retrieve_message_by_id()
     */
    public function get_message_by_id($message_id)
    {
        if (!$message_id)
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NoMessageSelected'));
            $this->display_footer();
            exit();
        }
        else
        {
            // retrieve the message
            $message = $this->retrieve_message_by_id($message_id);

            if ($message)
            {
                // create user message conditions
                $conditions = array();
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                                $message->get_id()
                );
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                                $this->get_user_id()
                );
                $user_message_condition = new AndCondition($conditions);

                // retrieve the user message
                $user_message = $this->retrieve_user_message($user_message_condition);

                // check if there is a user message
                if (!$user_message)
                {
                    $this->display_header();
                    Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                    $this->display_footer();
                    exit();
                }
                else
                {
                    return $message;
                }
            }
            else
            {
                $this->display_header();
                Display :: error_message(Translation :: get('NoMessageSelected'));
                $this->display_footer();
                exit();
            }
        }
    }

    public function add_users($message, $user_ids = array(), $group_ids = array())
    {
        $failures = 0;

        // add the users
        if (!empty($user_ids))
        {
            // construct array if id is single element
            if (!is_array($user_ids))
            {
                $user_ids = array($user_ids);
            }

            // for each user, create the link
            foreach ($user_ids as $user_id)
            {
                if (!$message->add_user_recipient($user_id, true))
                {
                    $failures++;
                }
            }
        }

        // add the groups
        if (!empty($group_ids))
        {
            // construct array if id is single element
            if (!is_array($group_ids))
            {
                $group_ids = array($group_ids);
            }

            // for each group, create the link
            foreach ($group_ids as $group_id)
            {
                if (!$message->add_group_recipient($group_id, true))
                {
                    $failures++;
                }
            }
        }

        return $failures;
    }

    public function ignore_users($user_ids)
    {
        $failures = 0;

        // checks
        if (!empty($user_ids))
        {
            // construct array if id is single element
            if (!is_array($user_ids))
            {
                $user_ids = array($user_ids);
            }

            // for each user, create the block
            foreach ($user_ids as $user_id)
            {
                // create the user ignore
                $user_ignore = new UserIgnore();
                $user_ignore->set_user_id($this->get_user_id());
                $user_ignore->set_ignored_user_id($user_id);

                // save the user ignore
                if (!$user_ignore->create())
                {
                    $failures++;
                }
            }
        }

        return $failures;
    }

    public function delete_users($message, $user_ids)
    {
        $failures = 0;

        // checks
        if (!empty($user_ids))
        {
            // construct array if id is single element
            if (!is_array($user_ids))
            {
                $user_ids = array($user_ids);
            }

            // for each user, create the block
            foreach ($user_ids as $user_id)
            {
                // we cannot remove ourselve, we have to delete the message in the inbox...
                if ($user_id != $this->get_user_id())
                {
                    // delete the UserMessage
                    if (!$message->delete_user_message_relation($user_id))
                    {
                        $failures++;
                    }
                }
            }
        }

        return $failures;
    }

    public function unignore_users($user_ids)
    {
        $failures = 0;

        // checks
        if (!empty($user_ids))
        {
            // construct array if id is single element
            if (!is_array($user_ids))
            {
                $user_ids = array($user_ids);
            }

            // for each user, remove the block
            foreach ($user_ids as $user_id)
            {
                // construct the condition
                $conditions = array();
                $conditions[] = new EqualityCondition(
                                UserIgnore :: PROPERTY_USER_IGNORE_USER_ID,
                                $this->get_user_id()
                );
                $conditions[] = new EqualityCondition(
                                UserIgnore :: PROPERTY_USER_IGNORE_IGNORED_USER_ID,
                                $user_id
                );
                $condition = new AndCondition($conditions);

                // get the UserMessage
                $dm = PersonalMessengerDataManager :: get_instance();
                $user_ignore = $dm->retrieve_user_ignore($condition);

                // update the status
                if (!$user_ignore->delete())
                {
                    $failures++;
                }
            }
        }

        return $failures;
    }

    //**************************************************************************
    // GET_URL FUNCTIONS
    //**************************************************************************
    public function get_browser_url($folder, $category_id = 0, $referer = null)
    {
        // parameter folder is needed to create URLs with a different folder than the current one
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_BROWSE_MESSAGES,
            self :: PARAM_FOLDER => $folder,
            self :: PARAM_CATEGORY_ID => $category_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_create_url($referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_CREATE_MESSAGE,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_create_from_template_url($referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_CREATE_MESSAGE_FROM_TEMPLATE,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_reply_url($message, $include_original_message = true, $indent_original_message = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_REPLY,
            self :: PARAM_REPLY_ID => $message->get_id(),
            self :: PARAM_PARENT_ID => $message->get_parent_id(),
            self :: PARAM_REPLY_TYPE => MessageForm :: TYPE_REPLY,
            self :: PARAM_INCLUDE_ORIGINAL_MESSAGE => $include_original_message,
            self :: PARAM_INDENT_ORIGINAL_MESSAGE => $indent_original_message,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_reply_all_url($message, $include_original_message = true, $indent_original_message = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_REPLY,
            self :: PARAM_REPLY_ID => $message->get_id(),
            self :: PARAM_PARENT_ID => $message->get_parent_id(),
            self :: PARAM_REPLY_TYPE => MessageForm :: TYPE_REPLY_ALL,
            self :: PARAM_INCLUDE_ORIGINAL_MESSAGE => $include_original_message,
            self :: PARAM_INDENT_ORIGINAL_MESSAGE => $indent_original_message,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_forward_url($message, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_FORWARD,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_move_url($message, $switch_category = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MOVE_MESSAGE,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_CATEGORY => $switch_category,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_mark_read_url($message, $switch_folder = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MARK_MESSAGE_READ,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_FOLDER => $switch_folder,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_mark_unread_url($message, $switch_folder = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MARK_MESSAGE_UNREAD,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_FOLDER => $switch_folder,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_create_template_url($message, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_CREATE_TEMPLATE,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_ignore_url($message, $switch_folder = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MARK_MESSAGE_IGNORE,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_FOLDER => $switch_folder,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_unignore_url($message, $switch_folder = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MARK_MESSAGE_UNIGNORE,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_FOLDER => $switch_folder,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_trash_url($message, $switch_folder = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MARK_MESSAGE_TRASH,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_FOLDER => $switch_folder,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_untrash_url($message, $switch_folder = true, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MARK_MESSAGE_UNTRASH,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_SWITCH_CURRENT_FOLDER => $switch_folder,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_delete_url($message, $referer = null)
    {
        // construct viewing parameters
        $params = array();
        $params[self :: PARAM_MESSAGE_ID] = $message->get_id();
        $params[self :: PARAM_REFERER] = $referer;
        $params[self :: PARAM_REPLY_ID] = $message->get_reply_id();

        if ($message->is_discussion() && (!$message->is_discussion_root()))
        {
            $params[self :: PARAM_ACTION] = self :: ACTION_DELETE_MESSAGE;
        }
        else
        {
            $params[self :: PARAM_ACTION] = self :: ACTION_DELETE_USER_MESSAGE;
        }

        return $this->get_url($params);
    }

    public function get_message_update_url($message, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_UPDATE_MESSAGE,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_add_user_url($message, $user_id, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_ADD_USER,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_USER_ID => $user_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_delete_user_url($message, $user_id, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_DELETE_USER,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_USER_ID => $user_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_ignore_user_url($message, $user_id, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_IGNORE_USER,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_USER_ID => $user_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_unignore_user_url($message, $user_id, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_UNIGNORE_USER,
            self :: PARAM_MESSAGE_ID => $message->get_id(),
            self :: PARAM_USER_ID => $user_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_message_view_url($message, $referer = null)
    {
        // construct viewing parameters
        $params = array();
        $params[self :: PARAM_MESSAGE_ID] = $message->get_id();
        $params[self :: PARAM_REFERER] = $referer;

        if ($message->is_discussion())
        {
            $params[self :: PARAM_ACTION] = self :: ACTION_VIEW_DISCUSSION;
        }
        else
        {
            $params[self :: PARAM_ACTION] = self :: ACTION_VIEW_MESSAGE;
        }

        return $this->get_url($params);
    }

    public function get_attachment_viewer_url($message_id, $attachment_id, $popup = false, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_VIEW_ATTACHMENT,
            self :: PARAM_MESSAGE_ID => $message_id,
            self :: PARAM_ATTACHMENT_ID => $attachment_id,
            self :: PARAM_POPUP => $popup,
            self :: PARAM_REFERER => $referer));
    }

    public function get_user_ignore_url($user_id, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_IGNORE_USER,
            self :: PARAM_USER_ID => $user_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_user_unignore_url($user_id, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_UNIGNORE_USER,
            self :: PARAM_USER_ID => $user_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_discussion_user_manager_url($message_id = null, $referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MANAGE_USERS_DISCUSSION,
            self :: PARAM_MESSAGE_ID => $message_id,
            self :: PARAM_REFERER => $referer));
    }

    public function get_ignored_user_manager_url($referer = null)
    {
        return $this->get_url(array(
            self :: PARAM_ACTION => self :: ACTION_MANAGE_USERS_IGNORED,
            self :: PARAM_MESSAGE_ID => $message_id,
            self :: PARAM_REFERER => $referer));
    }

    /**
     * Helper function for the Application class,
     * pending access to class constants via variables in PHP 5.3
     * e.g. $name = $class :: DEFAULT_ACTION
     *
     * DO NOT USE IN THIS APPLICATION'S CONTEXT
     * Instead use:
     * - self :: DEFAULT_ACTION in the context of this class
     * - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function get_installable_application_packages($include_installed = false)
    {
        $package_list = new PackageList(
            self :: context(),
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        $integration_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration',
            Translation :: get('Integrations', null, __NAMESPACE__));

        $core_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration\core',
            Translation :: get('Core', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\core\home') || $include_installed)
        {
            $core_list->add_package(self :: context() . '\integration\core\home');
        }

        $integration_list->add_child($core_list);
        $package_list->add_child($integration_list);

        return $package_list;
    }

}
