<?php
namespace application\weblcms\course;

use common\libraries\DoctrineDatabase;
use common\libraries\InequalityCondition;
use common\libraries\DoctrineRecordResultSet;
use common\libraries\DataClassNoResultException;
use common\libraries\CaseElementConditionVariable;
use common\libraries\CaseConditionVariable;
use common\libraries\GroupBy;
use user\User;
use group\GroupRelUser;
use common\libraries\OrCondition;
use common\libraries\AndCondition;
use common\libraries\StaticConditionVariable;
use common\libraries\FunctionConditionVariable;
use common\libraries\FixedPropertyConditionVariable;
use common\libraries\RecordRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\DataClassJoin;
use common\libraries\Joins;
use application\weblcms\course\CourseGroupRelation;
use group\Group;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassProperties;

/**
 * Doctrine implementation of the datamanager
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class DoctrineDataManager extends DoctrineDatabase
{
    /**
     * Initializes and sets the prefix for this implementation
     */
    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('weblcms_');
    }

    const PARAM_SUBSCRIPTION_STATUS = 'record_subscription_status';
    const PARAM_SUBSCRIPTION_TYPE = 'record_subscription_type';
    const PARAM_COUNT = 'count';

    /****************************************************************************************************************
     * Implemented Functionality                                                                                    *
     ****************************************************************************************************************/

    /**
     * Retrieves all course users of a given course with status and subscription type
     *
     * @param int $course_id
     * @param Condition $condition
     * @param int $offset
     * @param int $count
     * @param ObjectTableOrder[] $order_property
     *
     * @throws \common\libraries\DataClassNoResultException
     *
     * @return RecordResultSet
     */
    public function retrieve_all_course_users(
        $course_id, $condition = null, $offset = null, $count = null, $order_property = null
    )
    {
        $properties = $this->get_user_properties_for_select();

        $properties->add(
            new FunctionConditionVariable(
                FunctionConditionVariable :: MIN,
                new StaticConditionVariable(self :: PARAM_SUBSCRIPTION_STATUS, false),
                'subscription_status'
            )
        );

        $properties->add(
            new FunctionConditionVariable(
                FunctionConditionVariable :: SUM,
                new StaticConditionVariable(self :: PARAM_SUBSCRIPTION_TYPE, false),
                'subscription_type'
            )
        );

        $sql = $this->build_basic_sql_all_course_users($properties, $course_id, $condition);

        $query_builder = $this->get_connection()->createQueryBuilder();

        $group_by = new GroupBy();
        $group_by->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_USERNAME));

        $query_builder = $this->process_group_by($query_builder, $group_by);
        $query_builder = $this->process_order_by($query_builder, User :: class_name(), $order_property);
        $query_builder = $this->process_limit($query_builder, $count, $offset);

        $sql .= substr($query_builder->getSQL(), 14);

        try
        {
            return new DoctrineRecordResultSet($this->get_connection()->query($sql));
        }
        catch(\PDOException $exception)
        {
            $this->error_handling($exception);
            throw new DataClassNoResultException(User :: class_name(), null);
        }
    }

    /**
     * Counts all course users of a given course with status and subscription type
     *
     * @param int $course_id
     * @param Condition $condition
     *
     * @throws \common\libraries\DataClassNoResultException
     *
     * @return RecordResultSet
     */
    public function count_all_course_users($course_id, $condition = null)
    {
        $properties = new DataClassProperties();

        $properties->add(
            new FunctionConditionVariable(
                FunctionConditionVariable::COUNT,
                new FunctionConditionVariable(
                    FunctionConditionVariable :: DISTINCT,
                    new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_USERNAME)
                ),
                self :: PARAM_COUNT
            )

        );

        $sql = $this->build_basic_sql_all_course_users($properties, $course_id, $condition);

        try
        {
            $result = $this->get_connection()->query($sql);
            $row = $result->fetch(\PDO :: FETCH_ASSOC);

            return $row[self :: PARAM_COUNT];
        }
        catch(\PDOException $exception)
        {
            $this->error_handling($exception);
            throw new DataClassNoResultException(User :: class_name(), null);
        }
    }

    /****************************************************************************************************************
     * All Course Users Helper Functionality                                                                        *
     ****************************************************************************************************************/

    /**
     * Builds the basic sql for the "all course users" funtions
     *
     * @param DataClassProperties $properties
     * @param int $course_id
     * @param Condition $condition
     *
     * @return string
     */
    protected function build_basic_sql_all_course_users($properties, $course_id, $condition)
    {
        $query_builder = $this->get_connection()->createQueryBuilder();
        $query_builder = $this->process_data_class_properties($query_builder, User :: class_name(), $properties);

        $sql = $query_builder->getSQL();

        $sql_subscribed_users = $this->build_sql_for_subscribed_users($course_id, $condition);
        $sql_subscribed_groups = $this->build_sql_for_subscribed_group_users($course_id, $condition);

        $sql .= '('  . ($sql_subscribed_groups ? $sql_subscribed_users . ' UNION ' . $sql_subscribed_groups :
            $sql_subscribed_users) . ') AS ' .
            \user\DataManager :: get_instance()->get_alias(User :: get_table_name()) . ' ';

        return $sql;
    }

    /**
     * Retrieves the direct subscribed groups joined with the group table to support the tree values
     *
     * @param $course_id
     *
     * @return \common\libraries\DoctrineRecordResultSet
     */
    protected function retrieve_direct_subscribed_groups_with_tree_values($course_id)
    {
        $properties = new DataClassProperties();

        $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE));
        $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE));

        $properties->add(
            new PropertyConditionVariable(CourseGroupRelation :: class_name(), CourseGroupRelation :: PROPERTY_STATUS)
        );

        $joins = new Joins();

        $joins->add(
            new DataClassJoin(
                Group :: class_name(),
                new EqualityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID),
                    new PropertyConditionVariable(
                        CourseGroupRelation :: class_name(), CourseGroupRelation :: PROPERTY_GROUP_ID
                    )
                )
            )
        );

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                CourseGroupRelation :: class_name(), CourseGroupRelation :: PROPERTY_COURSE_ID
            ),
            new StaticConditionVariable($course_id)
        );

        $parameters = new RecordRetrievesParameters($properties, $condition, null, null, null, $joins);

        return self :: records(CourseGroupRelation :: class_name(), $parameters);
    }

    /**
     * Returns the properties for the user that are used in the select statement
     *
     * @return \common\libraries\DataClassProperties
     */
    protected function get_user_properties_for_select()
    {
        $properties = new DataClassProperties();

        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_ID));
        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_OFFICIAL_CODE));
        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_LASTNAME));
        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_FIRSTNAME));
        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_USERNAME));
        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_EMAIL));
        $properties->add(new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_STATUS));

        return $properties;
    }

    /**
     * Builds the sql for the directly subscribed users
     *
     * @param int $course_id
     * @param Condition $condition
     *
     * @return string
     */
    protected function build_sql_for_subscribed_users($course_id, $condition)
    {
        $properties = $this->get_user_properties_for_select();

        $properties->add(
            new FixedPropertyConditionVariable(
                CourseUserRelation :: class_name(), CourseUserRelation :: PROPERTY_STATUS,
                self :: PARAM_SUBSCRIPTION_STATUS
            )
        );

        $properties->add(new StaticConditionVariable('1 AS ' . self :: PARAM_SUBSCRIPTION_TYPE, false));

        $joins = new Joins();

        $joins->add(
            new DataClassJoin(
                User :: class_name(),
                new EqualityCondition(
                    new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_ID),
                    new PropertyConditionVariable(
                        CourseUserRelation :: class_name(), CourseUserRelation :: PROPERTY_USER_ID
                    )
                )
            )
        );

        $conditions = array();

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                CourseUserRelation :: class_name(), CourseUserRelation :: PROPERTY_COURSE_ID
            ),
            new StaticConditionVariable($course_id)
        );

        if (isset($condition))
        {
            $conditions[] = $condition;
        }

        $condition = new AndCondition($conditions);

        $parameters = new RecordRetrievesParameters($properties, $condition, null, null, null, $joins);

        return self :: build_records_sql(CourseUserRelation :: class_name(), $parameters);
    }

    /**
     * Builds the sql for the users subscribed through (sub) platform groups
     *
     * @param int $course_id
     * @param Condition $condition
     *
     * @return string
     */
    protected function build_sql_for_subscribed_group_users($course_id, $condition)
    {
        $direct_groups_with_tree_values = self :: retrieve_direct_subscribed_groups_with_tree_values($course_id);

        if ($direct_groups_with_tree_values->size() > 0)
        {
            $properties = $this->get_user_properties_for_select();

            $case_condition_variable = new CaseConditionVariable(array(), self :: PARAM_SUBSCRIPTION_STATUS);

            $direct_group_conditions = array();
            while ($group = $direct_groups_with_tree_values->next_result())
            {
                $and_conditions = array();

                $and_conditions[] = new InequalityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE),
                    InequalityCondition :: GREATER_THAN_OR_EQUAL,
                    new StaticConditionVariable($group[Group :: PROPERTY_LEFT_VALUE])
                );

                $and_conditions[] = new InequalityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE),
                    InequalityCondition :: LESS_THAN_OR_EQUAL,
                    new StaticConditionVariable($group[Group :: PROPERTY_RIGHT_VALUE])
                );

                $direct_group_condition = new AndCondition($and_conditions);
                $direct_group_conditions[] = $direct_group_condition;

                $case_condition_variable->add(
                    new CaseElementConditionVariable(
                        $group[CourseGroupRelation :: PROPERTY_STATUS],
                        $direct_group_condition
                    )
                );

            }

            $properties->add($case_condition_variable);
            $properties->add(new StaticConditionVariable('2 AS ' . self :: PARAM_SUBSCRIPTION_TYPE, false));

            $joins = new Joins();

            $joins->add(
                new DataClassJoin(
                    GroupRelUser :: class_name(),
                    new EqualityCondition(
                        new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID),
                        new PropertyConditionVariable(
                            GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID
                        )
                    )
                )
            );

            $joins->add(
                new DataClassJoin(
                    User :: class_name(),
                    new EqualityCondition(
                        new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_ID),
                        new PropertyConditionVariable(
                            GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID
                        )
                    )
                )
            );

            $conditions = array();

            $conditions[] = new OrCondition($direct_group_conditions);

            if (isset($condition))
            {
                $conditions[] = $condition;
            }

            $condition = new AndCondition($conditions);

            $parameters = new RecordRetrievesParameters($properties, $condition, null, null, null, $joins);

            return \group\DataManager :: get_instance()->build_records_sql(Group :: class_name(), $parameters);
        }
    }
}
