<?php

namespace application\weblcms\integration\reporting;

use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\integration\tracking\AssessmentAttempt;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;
use application\weblcms\integration\tracking\QuestionAttempt;
use application\weblcms\tool\assessment\AssessmentTool;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\ArrayResultSet;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\Redirect;
use common\libraries\Request;
use common\libraries\StaticConditionVariable;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assessment\Assessment;

/**
 * Abstract class that defines common functionality for blocks of the assessment
 *
 * @package application\weblcms\integration\reporting
 */
abstract class AssessmentBlock extends ToolBlock
{
    const PROPERTY_ASSESSMENT_ATTEMPT = 'assessment_attempt';

    /**
     * Returns the assessment attempts data by a given publication id, optionally limited by user
     *
     * @param int $publication_id
     * @param int $user_id
     *
     * @return \common\libraries\ResultSet
     */
    protected function get_assessment_attempts($publication_id, $user_id = null)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_ASSESSMENT_ID
            ),
            new StaticConditionVariable($publication_id)
        );

        if($user_id)
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_USER_ID
                ),
                new StaticConditionVariable($user_id)
            );
        }

        $condition = new AndCondition($conditions);

        return WeblcmsTrackingDataManager :: retrieves(AssessmentAttempt :: class_name(), $condition);
    }

    /**
     * Returns the assessment question attempts data by a question complex content object item id and/or
     * a given set of assessment attempt id's
     *
     * @param int $question_cid
     * @param int[] $assessment_attempt_ids
     *
     * @return \common\libraries\ResultSet
     */
    protected function get_question_attempts($question_cid = null, $assessment_attempt_ids = array())
    {
        $conditions = array();

        if($question_cid)
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    QuestionAttempt :: class_name(), QuestionAttempt :: PROPERTY_QUESTION_CID
                ),
                new StaticConditionVariable($question_cid)
            );
        }

        if($assessment_attempt_ids)
        {
            if(!is_array($assessment_attempt_ids))
            {
                $assessment_attempt_ids = array($assessment_attempt_ids);
            }

            $conditions[] = new InCondition(
                new PropertyConditionVariable(
                    QuestionAttempt :: class_name(), QuestionAttempt :: PROPERTY_ASSESSMENT_ATTEMPT_ID
                ),
                $assessment_attempt_ids
            );
        }

        $condition = new AndCondition($conditions);

        return WeblcmsTrackingDataManager :: retrieves(QuestionAttempt :: class_name(), $condition);
    }

    /**
     * Returns the question attempts from a given publication and question (optionally limited by user)
     *
     * @param int $publication_id
     * @param int $question_cid
     * @param int $user_id
     *
     * @return \common\libraries\ResultSet
     */
    protected function get_question_attempts_from_publication_and_question(
        $publication_id, $question_cid, $user_id = null
    )
    {
        $assessment_attempt_ids = array();
        $assessment_attempts_by_id = array();

        $assessment_attempts = $this->get_assessment_attempts($publication_id, $user_id);
        while($assessment_attempt = $assessment_attempts->next_result())
        {
            $assessment_attempt_ids[] = $assessment_attempt->get_id();
            $assessment_attempts_by_id[$assessment_attempt->get_id()] = $assessment_attempt;
        }

        if(count($assessment_attempt_ids) == 0)
        {
            return new ArrayResultSet(array());
        }

        $question_attempts = $this->get_question_attempts($question_cid, $assessment_attempt_ids);

        while($question_attempt = $question_attempts->next_result())
        {
            $question_attempt->set_optional_property(
                self :: PROPERTY_ASSESSMENT_ATTEMPT,
                $assessment_attempts_by_id[$question_attempt->get_assessment_attempt_id()]
            );
        }

        $question_attempts->reset();

        return $question_attempts;
    }

    /**
     * Returns the selected question id from the url
     *
     * @return int
     */
    protected function get_question_id()
    {
        return Request :: get(ReportingTool :: PARAM_QUESTION);
    }

    /**
     * Renders the difficulty index
     *
     * @param int $correct_answers
     * @param int $total_answers
     *
     * @return string
     */
    protected function render_difficulty_index($correct_answers, $total_answers)
    {
        $difficulty_index = round($correct_answers / $total_answers, 2);
        return $this->get_score_bar($difficulty_index * 100);

        /*$color = $difficulty_index > 0.75 ? 'orange' : ($difficulty_index < 0.25 ? 'red' : 'green');

        return '<span style="color: ' . $color . '; font-weight: bold;">' .
            $difficulty_index . '</span>';*/
    }

    /**
     * Returns the assessment title with a link
     *
     * @param $assessment_publication
     *
     * @return string
     */
    protected function get_assessment_title_with_link($assessment_publication)
    {
        $assessment = $assessment_publication->get_content_object();

        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $assessment_publication->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = $assessment_publication->get_tool();
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $assessment_publication->get_id();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;

        $url = Redirect :: get_url($params);

        return '<a href="' . $url . '">' . $assessment->get_title() . '</a>';
    }

    /**
     * Returns the assessment information headers
     *
     * @return string[]
     */
    protected function get_assessment_information_headers()
    {
        return array(
            Translation :: get('AssessmentTitle'),
            Translation :: get('AssessmentDescription'),
            Translation :: get('Published'),
            Translation :: get('LastModified'),
        );
    }

    /**
     * Returns the assessment information for a given publication
     *
     * @param ContentObjectPublication $assessment_publication
     *
     * @return string[string]
     */
    protected function get_assessment_information($assessment_publication)
    {
        $assessment_information = array();

        $assessment = $assessment_publication->get_content_object();

        $assessment_information[Translation :: get('AssessmentTitle')] =
            $this->get_assessment_title_with_link($assessment_publication);

        $assessment_information[Translation :: get('AssessmentDescription')] = $assessment->get_description();

        $assessment_information[Translation :: get('Published')] =
            DatetimeUtilities :: format_locale_date(null, $assessment_publication->get_publication_date());

        $assessment_information[Translation :: get('LastModified')] =
            DatetimeUtilities :: format_locale_date(null, $assessment_publication->get_modified_date());

        return $assessment_information;
    }

    /**
     * Returns the question information headers
     *
     * @return string[]
     */
    protected function get_question_information_headers()
    {
        return array(
            Translation :: get('QuestionTitle'),
            Translation :: get('QuestionDescription'),
            Translation :: get('QuestionType')
        );
    }

    /**
     * Returns the assessment information for a given publication
     *
     * @param ContentObject $question
     *
     * @return string[string]
     */
    protected function get_question_information($question)
    {
        $question_information = array();

        $question_information[Translation :: get('QuestionTitle')] = $question->get_title();
        $question_information[Translation :: get('QuestionDescription')] = $question->get_description();
        $question_information[Translation :: get('QuestionType')] = Translation :: get(
            'TypeName',
            array(),
            Utilities :: get_namespace_from_classname($question->get_type())
        );

        return $question_information;
    }

    /**
     * Adds a row to the reporting data from the given array data
     *
     * @param int $row_count
     * @param mixed[] $array
     * @param ReportingData $reporting_data
     */
    protected function add_row_from_array($row_count, $array, $reporting_data)
    {
        foreach($array as $header => $value)
        {
            $reporting_data->add_data_category_row($row_count, $header, $value);
        }
    }

    /**
     * Adds a category to the reporting data from the given array data
     *
     * @param string $category_name
     * @param mixed[] $array
     * @param ReportingData $reporting_data
     */
    protected function add_category_from_array($category_name, $array, $reporting_data)
    {
        foreach($array as $header => $value)
        {
            $reporting_data->add_data_category_row($header, $category_name, $value);
        }
    }

    /**
     * Returns the link to the assessment result viewer for an assessment or an individual question
     *
     * @param int $assessment_attempt_id
     * @param int $question_id
     *
     * @return string
     */
    protected function get_assessment_result_viewer_link($assessment_attempt_id, $question_id = null)
    {
        $details_image = Theme :: get_common_image('action_browser');

        $params = array();

        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = Assessment :: get_type_name();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssessmentTool :: ACTION_ATTEMPT_RESULT_VIEWER;
        $params[AssessmentTool :: PARAM_USER_ASSESSMENT] = $assessment_attempt_id;

        if($question_id)
        {
            $params[AssessmentTool :: PARAM_QUESTION_ATTEMPT] = $question_id;
        }

        $params[AssessmentTool :: PARAM_ASSESSMENT] = $this->get_publication_id();

        $link = Redirect :: get_url($params);

        return '<a href="#" onclick="javascript:openPopup(\'' . $link . '\')">' . $details_image . '</a>';
    }
}