<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublicationListRenderer;
use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use application\weblcms\tool\assignment\AssignmentTool;
use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\tool\user\UserTool;
use application\weblcms\Tool;
use common\libraries\Application;
use common\libraries\DatetimeUtilities;
use common\libraries\PlatformSetting;
use common\libraries\Redirect;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\DataManager as GroupDataManager;
use group\Group;
use repository\content_object\assignment\Assignment;
use repository\ContentObjectRenderer;
use user\User;
use user\UserDataManager;
use user\UserManager;
use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

/**
 * This manager creates a new level between the reporting responsible for the Assignment Tool and the Weblcms reporting
 * system. It groups common code between the reporting blocks in one place so that each block is no longer responsible
 * for code that should be shared.
 *
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Bert De Clercq (Hogeschool Gent)
 */
abstract class AssignmentReportingManager extends ToolBlock
{

    /**
     * Formats the colour of the score with reference to the platform setting passing percentage.
     *
     * @param $score int The score to be formatted.
     * @return string The score in coloured HTML format.
     */
    protected function format_score_html($score)
    {
        if ($score !== null)
        {
            $colour = null;

            if ($score < PlatformSetting :: get('passing_percentage'))
            {
                $colour = 'red';
            }
            else
            {
                $colour = 'green';
            }

            return '<span style="color:' . $colour . '">' . $score . '%</span>';
        }
        else
        {
            return '-';
        }
    }

    /**
     * Formats a date and colours it red when it is later than the critical date.
     *
     * @param $date type The date to be formatted.
     * @param $critical_date type The date that is used to decide whether $date is later.
     * @return string The date in coloured HTML format.
     */
    protected function format_date_html($date, $critical_date)
    {
        $formatted_date = DatetimeUtilities :: format_locale_date(
            Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES),
            $date);

        if ($date > $critical_date)
        {
            return '<span style="color:red">' . $formatted_date . '</span>';
        }

        return $formatted_date;
    }

    /**
     * Retrieves the course id from the url.
     *
     * @return type the course id.
     */
    public function get_course_id()
    {
        return Request :: get(WeblcmsManager :: PARAM_COURSE);
    }

    /**
     * Retrieves the publication id from the url.
     *
     * @return type the publication id.
     */
    public function get_publication_id()
    {
        return Request :: get(Tool :: PARAM_PUBLICATION_ID);
    }

    /**
     * Retrieves the submitter type from the url.
     *
     * @return int the submitter type.
     */
    public function get_submitter_type()
    {
        return Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);
    }

    /**
     * Retrieves the target id from the url.
     *
     * @return int the target id.
     */
    public function get_target_id()
    {
        return Request :: get(AssignmentTool :: PARAM_TARGET_ID);
    }

    /**
     * Returns the given type of score from the given score trackers. This will return the average score by default.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers
     * @param $score_type int The type of score you want
     * @return int The score from the given type
     */
    protected function get_score_by_type($score_trackers, $score_type)
    {
        switch ($score_type)
        {
            case self :: SCORE_TYPE_AVG :
                return $this->get_avg_score($score_trackers);

            case self :: SCORE_TYPE_MIN :
                return $this->get_min_score($score_trackers);

            case self :: SCORE_TYPE_MAX :
                return $this->get_max_score($score_trackers);

            case self :: SCORE_TYPE_FIRST :
                return $this->get_first_score($score_trackers);

            case self :: SCORE_TYPE_LAST :
                return $this->get_last_score($score_trackers);

            default :
                return $this->get_avg_score($score_trackers);
        }
    }

    /**
     * Returns the average score from the given score trackers.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers used to the
     *            average score
     * @return int The average score
     */
    protected function get_avg_score($score_trackers)
    {
        if (! $score_trackers || count($score_trackers) < 1)
        {
            return null;
        }
        $score = null;

        foreach ($score_trackers as $score_tracker)
        {
            $score += $score_tracker->get_score();
        }

        return round($score / count($score_trackers), 2);
    }

    /**
     * Returns the minimum score from the given score trackers.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers used to
     *            determine the minimum score
     * @return int The minimum score
     */
    protected function get_min_score($score_trackers)
    {
        if (! $score_trackers || count($score_trackers) < 1)
        {
            return null;
        }
        $score = null;

        foreach ($score_trackers as $score_tracker)
        {
            if (is_null($score) || $score_tracker->get_score() < $score)
            {
                $score = $score_tracker->get_score();
            }
        }

        return round($score, 2);
    }

    /**
     * Returns the maximum score from the given score trackers.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers used to
     *            determine the maximum score
     * @return int The maximum score
     */
    protected function get_max_score($score_trackers)
    {
        if (! $score_trackers || count($score_trackers) < 1)
        {
            return null;
        }
        $score = null;

        foreach ($score_trackers as $score_tracker)
        {
            if (is_null($score) || $score_tracker->get_score() > $score)
            {
                $score = $score_tracker->get_score();
            }
        }

        return round($score, 2);
    }

    /**
     * Returns the first score from the given score trackers.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers used to
     *            determine the first score.
     * @return int The first score
     */
    protected function get_first_score($score_trackers)
    {
        if (! $score_trackers || count($score_trackers) < 1)
        {
            return null;
        }
        $score = null;
        $score_modified = null;

        foreach ($score_trackers as $score_tracker)
        {
            if (is_null($score) || $score_tracker->get_modified() < $score_modified)
            {
                $score_modified = $score_tracker->get_modified();
                $score = $score_tracker->get_score();
            }
        }

        return round($score, 2);
    }

    /**
     * Returns the last score from the given score trackers.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers used to
     *            determine the last score.
     * @return int The last score.
     */
    protected function get_last_score($score_trackers)
    {
        if (! $score_trackers || count($score_trackers) < 1)
        {
            return null;
        }
        $score = null;
        $score_modified = null;

        foreach ($score_trackers as $score_tracker)
        {
            if (is_null($score) || $score_tracker->get_modified() > $score_modified)
            {
                $score_modified = $score_tracker->get_modified();
                $score = $score_tracker->get_score();
            }
        }

        return round($score, 2);
    }

    /**
     * Generates an HTML link to the assignment tool using the name of the publication passed.
     *
     * @param $publication_id int The id of the publication.
     * @return string The HTML representation of the link.
     */
    protected function generate_assignment_name_link($publication_id)
    {
        $assignment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id)->get_content_object();
        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = Utilities :: get_classname_from_namespace(
            Assignment :: class_name(),
            true);
        $params[Tool :: PARAM_ACTION] = AssignmentTool :: ACTION_BROWSE_SUBMITTERS;
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $publication_id;
        $url_title = Redirect :: get_url($params);
        return '<a href="' . $url_title . '">' . $assignment->get_title() . '</a>';
    }

    /**
     * Generates an HTML link to the assignment tool using the name of the submitter passed.
     *
     * @param $submitter_type int The type of the submitter
     *            (\application\weblcms\integration\tracking\AssignmentSubmission::SUBMITTER_TYPE_...).
     * @param $submitter_id int The id of the submitter.
     * @return string The HTML representation of the link.
     */
    protected function generate_submitter_name_link($submitter_type, $submitter_id)
    {
        $submitter_name = $this->get_submitter_name_by_id($submitter_type, $submitter_id);
        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = Utilities :: get_classname_from_namespace(
            Assignment :: class_name(),
            true);
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->get_publication_id();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssignmentTool :: ACTION_BROWSE_SUBMISSIONS;
        $params[Tool :: PARAM_BROWSER_TYPE] = ContentObjectRenderer :: TYPE_TABLE;
        $params[Tool :: PARAM_PUBLICATION_ID] = $this->get_publication_id();
        $params[AssignmentTool :: PARAM_TARGET_ID] = $submitter_id;
        $params[AssignmentTool :: PARAM_SUBMITTER_TYPE] = $submitter_type;
        return ' <a href="' . Redirect :: get_url($params) . '">' . $submitter_name . '</a>';
    }

    /**
     * Generates an HTML link to the assignment tool using the title of the submission in the submissions tracker
     * passed.
     *
     * @param $submission_tracker \application\weblcms\integration\tracking\AssignmentSubmission The submission tracker.
     * @return string The HTML representation of the link.
     */
    protected function generate_submission_title_link($submission_tracker)
    {
        $submission_title = \repository\DataManager :: retrieve_content_object(
            $submission_tracker->get_content_object_id())->get_title();
        $params = array();
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[WeblcmsManager :: PARAM_TOOL] = Utilities :: get_classname_from_namespace(
            Assignment :: class_name(),
            true);
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssignmentTool :: ACTION_VIEW_SUBMISSION;
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->get_publication_id();
        $params[AssignmentTool :: PARAM_TARGET_ID] = $submission_tracker->get_submitter_id();
        $params[AssignmentTool :: PARAM_SUBMITTER_TYPE] = $submission_tracker->get_submitter_type();
        $params[AssignmentTool :: PARAM_SUBMISSION] = $submission_tracker->get_id();
        return '<a href="' . Redirect :: get_url($params) . '">' . $submission_title . '</a>';
    }

    /**
     * Generates an HTML link to the user tool using tha name of the user passed.
     *
     * @param $user_id int The id of the user.
     * @return string The HTML representation of the link.
     */
    protected function generate_user_name_link($user_id)
    {
        $user_name = $this->get_submitter_name_by_id(
            \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER,
            $user_id);
        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = UserManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = UserTool :: ACTION_USER_DETAILS;
        $params[Tool :: PARAM_BROWSER_TYPE] = ContentObjectPublicationListRenderer :: TYPE_LIST;
        $params[WeblcmsManager :: PARAM_USERS] = $user_id;
        $url_user_name = Redirect :: get_url($params);
        return '<a href="' . $url_user_name . '">' . $user_name . '</a>';
    }

    /**
     * Obtains the name of the submitter passed.
     *
     * @param $submitter_type int The type of the submitter
     *            (\application\weblcms\integration\tracking\AssignmentSubmission::SUBMITTER_TYPE_...).
     * @param $submitter_id int The id of the submitter.
     * @return string The name of the submitter.
     */
    protected function get_submitter_name_by_id($submitter_type, $submitter_id)
    {
        switch ($submitter_type)
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                return CourseGroupDataManager :: retrieve_by_id(CourseGroup :: class_name(), $submitter_id)->get_name();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return GroupDataManager :: retrieve_by_id(Group :: class_name(), $submitter_id)->get_name();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                return UserDataManager :: get_instance()->retrieve_user($submitter_id)->get_fullname();
        }
    }
}
