<?php
namespace application\weblcms;

use common\libraries\StaticConditionVariable;
use HTML_Menu_ArrayRenderer;
use common\libraries\PropertyConditionVariable;
use common\libraries\TreeMenuRenderer;
use common\libraries\SubselectCondition;
use HTML_Menu;
use common\libraries\OrCondition;
use common\libraries\InCondition;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use repository\ContentObject;
use common\libraries\OptionsMenuRenderer;

/**
 * $Id: content_object_publication_category_tree.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.browser
 */
/**
 * A tree menu to display categories in a tool
 */
class ContentObjectPublicationCategoryTree extends HTML_Menu
{
    const TREE_NAME = __CLASS__;

    /**
     * The browser to which this category tree is associated
     */
    private $browser;

    /**
     * An id for this tree
     */
    private $tree_id;

    private $url_params;

    /**
     * Create a new category tree
     *
     * @param $browser PublicationBrowser The browser to associate this category tree with.
     * @param $tree_id string An id for the tree
     */
    public function __construct($browser, $tree_id, $url_params = array())
    {
        $this->browser = $browser;
        $this->tree_id = $tree_id;
        $this->url_params = $url_params;
        $menu = $this->get_menu_items();
        parent :: __construct($menu);
        $this->forceCurrentUrl($this->get_category_url($tree_id));
    }

    public function as_html()
    {
        $renderer = new TreeMenuRenderer($this->get_tree_name());
        $this->render($renderer, 'sitemap');
        return $renderer->toHTML();
    }

    public static function get_tree_name()
    {
        return Utilities :: get_classname_from_namespace(self :: TREE_NAME, true);
    }

    /**
     * Gets the current selected category id.
     *
     * @return int The current category id
     */
    public function get_current_category_id()
    {
        return intval(Request :: get(WeblcmsManager :: PARAM_CATEGORY));
    }

    private function get_menu_items($extra_items = null)
    {
        $menu = array();
        $menu_item = array();
        $menu_item['title'] = Translation :: get(
            Utilities :: underscores_to_camelcase($this->browser->get_tool_id()) . 'Title') . $this->get_category_count(
            0);
        $menu_item['url'] = $this->get_category_url(0);
        $sub_menu_items = $this->get_sub_menu_items(0);
        if (count($sub_menu_items) > 0)
        {
            $menu_item['sub'] = $sub_menu_items;
        }
        $menu_item['class'] = 'type_category';
        $menu_item[OptionsMenuRenderer :: KEY_ID] = 0;
        $menu[0] = $menu_item;
        if (count($extra_items))
        {
            $menu = array_merge($menu, $extra_items);
        }

        return $menu;
    }

    private function get_sub_menu_items($parent)
    {
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $parent);
        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE,
            $this->browser->get_parent()->get_course_id());
        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_TOOL,
            $this->browser->get_parent()->get_tool_id());
        $condition = new AndCondition($conditions);

        $objects = DataManager :: retrieves(ContentObjectPublicationCategory :: class_name(), $condition);

        $categories = array();
        while ($category = $objects->next_result())
        {
            $menu_item = array();
            $menu_item['title'] = $category->get_name() . $this->get_category_count($category->get_id());
            $menu_item['url'] = $this->get_category_url($category->get_id());
            $sub_menu_items = $this->get_sub_menu_items($category->get_id());
            if (count($sub_menu_items) > 0)
            {
                $menu_item['sub'] = $sub_menu_items;
            }
            $menu_item['class'] = 'type_category';
            $menu_item[OptionsMenuRenderer :: KEY_ID] = $category->get_id();
            $categories[$category->get_id()] = $menu_item;
        }
        return $categories;
    }

    private function get_category_count($category_id)
    {
        $count = $this->get_publication_count($category_id);
        return ($count > 0) ? ' (' . $count . ')' : '';
    }

    private function get_publication_count($category)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_COURSE_ID,
            $this->browser->get_parent()->get_course_id());
        $conditions[] = $this->get_condition($category);

        $course_groups = $this->browser->get_course_groups();

        $course_group_ids = array();

        foreach ($course_groups as $course_group)
        {
            $course_group_ids[] = $course_group->get_id();
        }

        $subselect_condition = new InCondition('type', $this->browser->get_allowed_types());

        $conditions[] = new SubselectCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(),
                ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID),
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
            ContentObject :: get_table_name(),
            $subselect_condition);

        $condition = new AndCondition($conditions);

        return DataManager :: count_content_object_publications_with_view_right_granted_in_category_location(
            $this->browser->get_location(),
            $this->browser->get_entities(),
            $condition,
            $this->browser->get_user_id());
    }

    private function get_condition($category = null)
    {
        if (is_null($category))
        {
            $category = $this->get_current_category_id();
        }
        $tool_cond = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL,
            $this->browser->get_parent()->get_tool_id());
        $category_cond = new EqualityCondition(ContentObjectPublication :: PROPERTY_CATEGORY_ID, $category);
        return new AndCondition($tool_cond, $category_cond);
    }

    /**
     * Gets the URL of a category
     *
     * @param $category_id int The id of the category of which the URL is requested
     * @return string The URL
     */
    private function get_category_url($category_id)
    {
        $this->url_params[WeblcmsManager :: PARAM_CATEGORY] = $category_id;
        return $this->browser->get_url($this->url_params);
    }

    public function get_breadcrumbs()
    {
        $array_renderer = new HTML_Menu_ArrayRenderer();
        $this->render($array_renderer, 'urhere');
        $breadcrumbs = $array_renderer->toArray();
        foreach ($breadcrumbs as &$crumb)
        {
            $split = explode('(', $crumb['title']);
            $crumb['title'] = $split[0];
        }
        return $breadcrumbs;
    }
}
