<?php
namespace application\weblcms;

use common\libraries\DisplayOrderDataClassListener;
use core\admin\AdminDataManager;
use user\UserDataManager;
use common\libraries\Utilities;
use common\libraries\DataClass;
use repository\ContentObject;
use common\libraries\DisplayOrderDataClassListenerSupport;
use common\libraries\PropertyConditionVariable;
use \ErrorException;
/**
 * $Id: content_object_publication.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms
 */

/**
 * This class represents a learning object publication. When publishing a learning object from the repository in the
 * weblcms application, a new object of this type is created.
 */
class ContentObjectPublication extends DataClass implements DisplayOrderDataClassListenerSupport
{
    const CLASS_NAME = __CLASS__;

    /*
     * #@+ Constant defining a property of the publication
     */
    const PROPERTY_CONTENT_OBJECT_ID = 'content_object_id';
    const PROPERTY_COURSE_ID = 'course_id';
    const PROPERTY_TOOL = 'tool';
    const PROPERTY_PARENT_ID = 'parent_id';
    const PROPERTY_CATEGORY_ID = 'category_id';
    const PROPERTY_FROM_DATE = 'from_date';
    const PROPERTY_TO_DATE = 'to_date';
    const PROPERTY_HIDDEN = 'hidden';
    const PROPERTY_PUBLISHER_ID = 'publisher_id';
    const PROPERTY_PUBLICATION_DATE = 'published';
    const PROPERTY_MODIFIED_DATE = 'modified';
    const PROPERTY_DISPLAY_ORDER_INDEX = 'display_order';
    const PROPERTY_EMAIL_SENT = 'email_sent';
    const PROPERTY_SHOW_ON_HOMEPAGE = 'show_on_homepage';

    private $target_course_groups;

    private $target_users;

    private $target_groups;

    private $content_object;

    private $publisher;

    public function __construct($default_properties = array(), $optional_properties = array())
    {
        parent :: __construct($default_properties = $optional_properties);
        $this->add_listener(new DisplayOrderDataClassListener($this));
    }

    public static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(
            array(
                self :: PROPERTY_CONTENT_OBJECT_ID,
                self :: PROPERTY_COURSE_ID,
                self :: PROPERTY_TOOL,
                self :: PROPERTY_PARENT_ID,
                self :: PROPERTY_CATEGORY_ID,
                self :: PROPERTY_FROM_DATE,
                self :: PROPERTY_TO_DATE,
                self :: PROPERTY_HIDDEN,
                self :: PROPERTY_PUBLISHER_ID,
                self :: PROPERTY_PUBLICATION_DATE,
                self :: PROPERTY_MODIFIED_DATE,
                self :: PROPERTY_DISPLAY_ORDER_INDEX,
                self :: PROPERTY_EMAIL_SENT,
                self :: PROPERTY_SHOW_ON_HOMEPAGE));
    }

    /**
     * Gets the learning object.
     *
     * @return ContentObject
     */
    public function get_content_object_id()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_OBJECT_ID);
    }

    /**
     * Gets the course code of the course in which this publication was made.
     *
     * @return string The course code
     */
    public function get_course_id()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_ID);
    }

    /**
     * Gets the tool in which this publication was made.
     *
     * @return string
     */
    public function get_tool()
    {
        return $this->get_default_property(self :: PROPERTY_TOOL);
    }

    /**
     * Gets the parent_id of the learning object publication
     *
     * @return int
     */
    public function get_parent_id()
    {
        return $this->get_default_property(self :: PROPERTY_PARENT_ID);
    }

    /**
     * Gets the id of the learning object publication category in which this publication was made
     *
     * @return int
     */
    public function get_category_id()
    {
        return $this->get_default_property(self :: PROPERTY_CATEGORY_ID);
    }

    /**
     * Gets the list of target users of this publication
     *
     * @return array An array of user ids.
     * @see is_for_everybody()
     */
    public function get_target_users()
    {
        if (! isset($this->target_users))
        {
            $this->target_users = DataManager :: retrieve_publication_target_user_ids($this->get_id());
        }

        return $this->target_users;
    }

    /**
     * Gets the list of target course_groups of this publication
     *
     * @return array An array of course_group ids.
     * @see is_for_everybody()
     */
    public function get_target_course_groups()
    {
        if (! isset($this->target_course_groups))
        {
            $this->target_course_groups = DataManager :: retrieve_publication_target_course_group_ids($this->get_id());
        }

        return $this->target_course_groups;
    }

    /**
     * Gets the list of target groups of this publication
     *
     * @return array An array of group ids.
     * @see is_for_everybody()
     */
    public function get_target_groups()
    {
        if (! isset($this->target_groups))
        {
            $this->target_groups = DataManager :: retrieve_publication_target_platform_group_ids($this->get_id());
        }

        return $this->target_groups;
    }

    /**
     * Gets the date on which this publication becomes available
     *
     * @return int
     * @see is_forever()
     */
    public function get_from_date()
    {
        return $this->get_default_property(self :: PROPERTY_FROM_DATE);
    }

    /**
     * Gets the date on which this publication becomes unavailable
     *
     * @return int
     * @see is_forever()
     */
    public function get_to_date()
    {
        return $this->get_default_property(self :: PROPERTY_TO_DATE);
    }

    /**
     * Gets the user id of the user who made this publication
     *
     * @return int
     */
    public function get_publisher_id()
    {
        return $this->get_default_property(self :: PROPERTY_PUBLISHER_ID);
    }

    /**
     * Retrieves the content object associated with this publication.
     *
     * @param bool $full Whether the content object must have its full complement of properties.
     */
    public function get_content_object($full = false)
    {
        if (is_null($this->content_object) || $full)
        {
            if (! is_null($this->get_optional_property(ContentObject :: PROPERTY_TITLE)) && ! $full)
            {
                $class = $this->get_optional_property(ContentObject :: PROPERTY_TYPE);
                $this->content_object = new $class($this->get_optional_properties());
                $this->content_object->set_id($this->get_content_object_id());
            }
            else
            {
                // todo: refactoring
                $this->content_object = \repository\DataManager :: retrieve_content_object(
                    $this->get_content_object_id());
            }
        }

        return $this->content_object;
    }

    public function set_content_object($content_object)
    {
        $this->content_object = $content_object;
    }

    public function get_publication_publisher()
    {
        if (! isset($this->publisher))
        {
            $udm = UserDataManager :: get_instance();
            $this->publisher = $udm->retrieve_user($this->get_publisher_id());
        }

        return $this->publisher;
    }

    /**
     * Sets the publication publisher for caching
     *
     * @param $user; User
     */
    public function set_publication_publisher(User $user)
    {
        $this->publisher = $user;
    }

    /**
     * Gets the date on which this publication was made
     *
     * @return int
     */
    public function get_publication_date()
    {
        return $this->get_default_property(self :: PROPERTY_PUBLICATION_DATE);
    }

    /**
     * Gets the date on which this publication was made
     *
     * @return int
     */
    public function get_modified_date()
    {
        return $this->get_default_property(self :: PROPERTY_MODIFIED_DATE);
    }

    /**
     * Determines whether this publication was sent by email to the users and course_groups for which this publication
     * was made
     *
     * @return boolean True if an email was sent
     */
    public function is_email_sent()
    {
        return $this->get_default_property(self :: PROPERTY_EMAIL_SENT);
    }

    /**
     * Determines whether this publication is hidden or not
     *
     * @return boolean True if the publication is hidden.
     */
    public function is_hidden()
    {
        return $this->get_default_property(self :: PROPERTY_HIDDEN);
    }

    /**
     * Determines whether this publication is available forever
     *
     * @return boolean True if the publication is available forever
     * @see get_from_date()
     * @see get_to_date()
     */
    public function is_forever()
    {
        return $this->get_from_date() == 0 && $this->get_to_date() == 0;
    }

    public function is_for_everybody()
    {
        return (count($this->get_target_users()) == 0 && count($this->get_target_course_groups()) == 0 &&
             count($this->get_target_groups()) == 0);
    }

    public function is_visible_for_target_users()
    {
        return (! $this->is_hidden()) &&
             ($this->is_forever() || ($this->get_from_date() <= time() && time() <= $this->get_to_date()));
    }

    public function get_display_order_index()
    {
        return $this->get_default_property(self :: PROPERTY_DISPLAY_ORDER_INDEX);
    }

    public function set_content_object_id($content_object_id)
    {
        $this->set_default_property(self :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
    }

    public function set_course_id($course)
    {
        $this->set_default_property(self :: PROPERTY_COURSE_ID, $course);
    }

    public function set_tool($tool)
    {
        $this->set_default_property(self :: PROPERTY_TOOL, $tool);
    }

    public function set_parent_id($parent_id)
    {
        $this->set_default_property(self :: PROPERTY_PARENT_ID, $parent_id);
    }

    public function set_category_id($category)
    {
        $this->set_default_property(self :: PROPERTY_CATEGORY_ID, $category);
    }

    public function set_target_users($target_users)
    {
        $this->target_users = $target_users;
    }

    public function set_target_course_groups($target_course_groups)
    {
        $this->target_course_groups = $target_course_groups;
    }

    public function set_target_groups($target_groups)
    {
        $this->target_groups = $target_groups;
    }

    public function set_from_date($from_date)
    {
        $this->set_default_property(self :: PROPERTY_FROM_DATE, $from_date);
    }

    public function set_to_date($to_date)
    {
        $this->set_default_property(self :: PROPERTY_TO_DATE, $to_date);
    }

    public function set_publisher_id($publisher)
    {
        $this->set_default_property(self :: PROPERTY_PUBLISHER_ID, $publisher);
    }

    public function set_publication_date($publication_date)
    {
        $this->set_default_property(self :: PROPERTY_PUBLICATION_DATE, $publication_date);
    }

    public function set_modified_date($modified_date)
    {
        $this->set_default_property(self :: PROPERTY_MODIFIED_DATE, $modified_date);
    }

    public function set_hidden($hidden)
    {
        $this->set_default_property(self :: PROPERTY_HIDDEN, $hidden);
    }

    public function set_display_order_index($display_order)
    {
        $this->set_default_property(self :: PROPERTY_DISPLAY_ORDER_INDEX, $display_order);
    }

    public function set_email_sent($email_sent)
    {
        $this->set_default_property(self :: PROPERTY_EMAIL_SENT, $email_sent);
    }

    /*
     * #@-
     */

    /**
     * Toggles the visibility of this publication.
     */
    public function toggle_visibility()
    {
        $this->set_hidden(! $this->is_hidden());
    }

    public function get_show_on_homepage()
    {
        return $this->get_default_property(self :: PROPERTY_SHOW_ON_HOMEPAGE);
    }

    public function set_show_on_homepage($show_on_homepage)
    {
        $this->set_default_property(self :: PROPERTY_SHOW_ON_HOMEPAGE, $show_on_homepage);
    }

    /**
     * Creates this publication in persistent storage
     */
    public function create($create_in_batch = false)
    {
        if (is_null($this->get_category_id()))
        {
            $this->set_category_id(0);
        }

        if (! parent :: create())
        {
            return false;
        }

        if ($this->get_category_id())
        {
            $parent = WeblcmsRights :: get_instance()->get_weblcms_location_id_by_identifier_from_courses_subtree(
                WeblcmsRights :: TYPE_COURSE_CATEGORY,
                $this->get_category_id(),
                $this->get_course_id());
        }
        else
        {
            if ($this->get_tool() == 'home')
            {
                $parent_id = WeblcmsRights :: get_instance()->get_courses_subtree_root_id($this->get_course_id());
                return WeblcmsRights :: get_instance()->create_location_in_courses_subtree(
                    WeblcmsRights :: TYPE_PUBLICATION,
                    $this->get_id(),
                    $parent_id,
                    $this->get_course_id(),
                    $create_in_batch
                );
            }
            else
            {
                $course_tool = DataManager :: retrieve_course_tool_by_name($this->get_tool());
                $course_tool_id = $course_tool->get_id();

                $parent = WeblcmsRights :: get_instance()->get_weblcms_location_id_by_identifier_from_courses_subtree(
                    WeblcmsRights :: TYPE_COURSE_MODULE,
                    $course_tool_id,
                    $this->get_course_id());
            }
        }

        return WeblcmsRights :: get_instance()->create_location_in_courses_subtree(
            WeblcmsRights :: TYPE_PUBLICATION,
            $this->get_id(),
            $parent,
            $this->get_course_id(),
            $create_in_batch);
    }

    /**
     * Moves the publication up or down in the list.
     *
     * @param $places The number of places to move the publication down. A negative number moves it up.
     * @return int The number of places that the publication was moved down.
     */
    public function move($places)
    {
        $this->set_display_order_index($this->get_display_order_index() + $places);
        $success = $this->update();
        return $success ? $places : 0;
    }

    public function delete()
    {
        $location = WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(
            WeblcmsRights :: TYPE_PUBLICATION,
            $this->get_id(),
            $this->get_course_id());
        if ($location)
        {
            if (! $location->delete())
            {
                return false;
            }
        }

        if (! parent :: delete())
        {
            return false;
        }

        return AdminDataManager :: get_instance()->delete_feedback_from_publication(
            WeblcmsManager :: APPLICATION_NAME,
            $this->get_id());
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public function get_target_entities()
    {
        try
        {
        return WeblcmsRights :: get_instance()->get_target_entities(
            WeblcmsRights :: VIEW_RIGHT,
            WeblcmsManager :: context(),
            $this->get_id(),
            WeblcmsRights :: TYPE_PUBLICATION,
            $this->get_course_id(),
            WeblcmsRights :: TREE_TYPE_COURSE);
        }
        catch(ErrorException $exception)
        {
            error_log($exception->getMessage());
            return false;
        }
    }

    public function render_target_entities_as_string()
    {
        return WeblcmsRights :: get_instance()->render_target_entities_as_string($this->get_target_entities());
    }

    /**
     * Returns the property for the display order
     *
     * @return string
     */
    public function get_display_order_property()
    {
        return new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_DISPLAY_ORDER_INDEX);
    }

    /**
     * Returns the properties that define the context for the display order (the properties on which has to be limited)
     *
     * @return Condition
     */
    public function get_display_order_context_properties()
    {
        return array(
            new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_COURSE_ID),
            new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_TOOL),
            new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_CATEGORY_ID));
    }

    /**
     * Ignores the display order for this specific object
     */
    public function ignore_display_order()
    {
        $this->remove_listener(0);
    }
}