<?php

namespace application\weblcms;

use common\libraries\Theme;
use common\libraries\DataClassRetrievesParameters;
use application\weblcms\tool\course_group\CourseGroup;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\AdvancedElementFinderElementType;
use common\libraries\AdvancedElementFinderElement;

use rights\NestedRightsEntity;

use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

/**
 * Class that describes the course groups for the rights editor
 *
 * @author Sven Vanpoucke
 */
class CourseGroupEntity implements NestedRightsEntity
{
    const ENTITY_NAME = 'course_group';
    const ENTITY_TYPE = 4;

    private $course_group_cache;
    private $course_id;

    private static $instance;

    public function __construct($course_id)
    {
        $this->course_id = $course_id;
    }

    public static function get_instance($course_id)
    {
        if (!isset(self :: $instance))
        {
            self :: $instance = new self($course_id);
        }
        return self :: $instance;
    }

    /**
     * Retrieves the items for this entity
     *
     * @param $condition Condition
     * @param $offset int
     * @param $count int
     * @param $order_property Array
     * @return ObjectResultSet
     */
    public function retrieve_entity_items($condition = null, $offset = null, $count = null, $order_property = null)
    {
        $condition = $this->get_condition($condition);

        return CourseGroupDataManager :: retrieves(
            CourseGroup :: class_name(),
            new DataClassRetrievesParameters($condition, $count, $offset, $order_property)
        );
    }

    /**
     * Retrieves the entity item ids relevant for a given user
     *
     * @param $user_id integer
     * @return array
     */
    public function retrieve_entity_item_ids_linked_to_user($user_id)
    {
        if (is_null($this->course_group_cache[$user_id]))
        {
            $course_groups = CourseGroupDataManager :: get_user_course_groups($user_id, $this->course_id); // todo:
            foreach ($course_groups as $course_group)
            {
                $this->course_group_cache[$user_id][] = $course_group->get_id();
            }
        }

        return $this->course_group_cache[$user_id];
    }

    /**
     * Counts the items for this entity
     *
     * @param $condition Condition
     * @return int
     */
    public function count_entity_items($condition = null)
    {
        $condition = $this->get_condition($condition);
        return DataManager :: count(CourseGroup :: class_name(), $condition);
    }

    /**
     * Returns the name of this entity
     *
     * @return String
     */
    public function get_entity_name()
    {
        return self :: ENTITY_NAME;
    }

    public function get_course_id()
    {
        return $this->course_id;
    }

    /**
     * Returns the translated name of this entiry for displaying purposes only!
     *
     * @return String Translated name of the entity
     */
    public function get_entity_translated_name()
    {
        return Translation :: get(Utilities :: underscores_to_camelcase(self :: ENTITY_NAME));
    }

    /**
     * Returns the type of this entity
     *
     * @return int
     */
    public function get_entity_type()
    {
        return self :: ENTITY_TYPE;
    }

    /**
     * Returns the path to the icon of the entity
     *
     * @return String
     */
    public function get_entity_icon()
    {
        return Theme :: get_image_path('common\extensions\rights_editor_manager') . 'place_group.png';
    }

    /**
     * Returns the cell renderer of this entity
     *
     * @param $browser Application
     * @return LocationCourseGroupBrowserTableCellRenderer
     */
    public function get_entity_cell_renderer($browser)
    {
        return new LocationCourseGroupBrowserTableCellRenderer($browser);
    }

    /**
     * Returns the column model of this entity
     *
     * @param $browser Application
     * @return LocationCourseGroupBrowserTableColumnModel
     */
    public function get_entity_column_model($browser)
    {
        return new LocationCourseGroupBrowserTableColumnModel($browser);
    }

    /**
     * Returns the property for the ID column of this entity
     *
     * @return String
     */
    public function get_id_property()
    {
        return CourseGroup :: PROPERTY_ID;
    }

    /**
     * Returns the property for the PARENT column of this entity
     *
     * @return String
     */
    public function get_parent_property()
    {
        return CourseGroup :: PROPERTY_PARENT_ID;
    }

    /**
     * Returns the property for the TITLE column of this entity
     *
     * @return String
     */
    public function get_title_property()
    {
        return CourseGroup :: PROPERTY_NAME;
    }

    /**
     * Returns the root ids for this entity
     *
     * @return Array<int>
     */
    public function get_root_ids()
    {
        return array(CourseGroupDataManager :: retrieve_course_group_root($this->course_id)->get_id());
    }

    /**
     * Returns the properties on which can be searched
     *
     * @return Array
     */
    public function get_search_properties()
    {
        return array(CourseGroup :: PROPERTY_NAME, CourseGroup :: PROPERTY_DESCRIPTION);
    }

    /**
     * Returns the xml feed which will be used for the element finder and for
     * the ajax tree menu
     */
    public function get_xml_feed()
    {
        return '';
    }

    // Helper functionality

    /**
     * Adds the course condition to the given condition
     *
     * @return Condition
     */
    private function get_condition($condition)
    {
        $course_condition = new EqualityCondition(CourseGroup :: PROPERTY_COURSE_CODE, $this->course_id);
        if ($condition)
        {
            $conditions = array();

            $conditions[] = $condition;
            $conditions[] = $course_condition;

            return new AndCondition($conditions);
        }

        return $course_condition;
    }

    /**
     * Retrieves the type for the advanced element finder for the simple rights
     * editor
     */
    public function get_element_finder_type()
    {
        return new AdvancedElementFinderElementType(
            'course_groups', Translation :: get(
                'CourseGroups'
            ), __NAMESPACE__, 'course_groups_feed', array(
                'course_id' => $this->course_id
            )
        );
    }

    /**
     * Retrieves an element for the advanced element finder for the simple
     * rights editor with the given id
     */
    public function get_element_finder_element($id)
    {
        $group = CourseGroupDataManager :: retrieve_by_id(CourseGroup :: class_name(), $id);
        if (!$group)
        {
            return null;
        }

        return new AdvancedElementFinderElement(
            self :: ENTITY_TYPE . '_' . $id, 'type type_group', $group->get_name(), $group->get_description()
        );
    }

    /**
     * Returns the class name of the data class that is used for this entity
     *
     * @return string
     */
    public static function data_class_class_name()
    {
        return CourseGroup :: class_name();
    }

}
