<?php
namespace application\weblcms;

use common\libraries\CommonDataManager;
use application\weblcms\course\Course;
use common\libraries\PlatformSetting;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use user\User;

/**
 * Simple connector class to facilitate rendering course settings
 *
 * @package application\weblcms;
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class CourseSettingsConnector
{
    /**
     * **************************************************************************************************************
     * Definition of settings *
     * **************************************************************************************************************
     */
    const VISIBILITY = 'visibility';
    const CATEGORY = 'category';
    const TITULAR = 'titular';
    const LANGUAGE = 'language';
    const COURSE_ACCESS = 'course_access';
    const OPEN_COURSE_ACCESS_TYPE = 'open_course_access_type';
    const THEME = 'theme';
    const TOOL_LAYOUT = 'tool_layout';
    const MENU_LAYOUT = 'menu_layout';
    const BREADCRUMB_LAYOUT = 'breadcrumb_layout';
    const TOOL_SHORTCUT_MENU = 'tool_shortcut_menu';
    const ALLOW_FEEDBACK = 'allow_feedback';
    const ALLOW_INTRODUCTION_TEXT = 'allow_introduction_text';
    const SHOW_COURSE_CODE = 'show_course_code';
    const SHOW_COURSE_TITULAR = 'show_course_titular';
    const SHOW_COURSE_LANGUAGE = 'show_course_language';

    /**
     * **************************************************************************************************************
     * Definition of setting options *
     * **************************************************************************************************************
     */
    const COURSE_ACCESS_OPEN = 1;
    const COURSE_ACCESS_CLOSED = 2;
    const OPEN_COURSE_ACCESS_REGISTERED_USERS = 1;
    const OPEN_COURSE_ACCESS_PLATFORM = 2;
    const OPEN_COURSE_ACCESS_WORLD = 3;
    const TOOL_LAYOUT_TWO_COLUMNS = 1;
    const TOOL_LAYOUT_THREE_COLUMNS = 2;
    const TOOL_LAYOUT_TWO_COLUMNS_GROUP_INACTIVE = 3;
    const TOOL_LAYOUT_THREE_COLUMNS_GROUP_INACTIVE = 4;
    const MENU_LAYOUT_OFF = 1;
    const MENU_LAYOUT_LEFT_WITH_ICONS = 2;
    const MENU_LAYOUT_LEFT_WITH_TEXT = 3;
    const MENU_LAYOUT_LEFT_BOTH = 4;
    const MENU_LAYOUT_RIGHT_WITH_ICONS = 5;
    const MENU_LAYOUT_RIGHT_WITH_TEXT = 6;
    const MENU_LAYOUT_RIGHT_BOTH = 7;
    const BREADCRUMB_LAYOUT_TITLE = 1;
    const BREADCRUMB_LAYOUT_VISUAL_CODE = 2;
    const BREADCRUMB_LAYOUT_COURSE_HOME = 3;

    /**
     * **************************************************************************************************************
     * Connector Functions *
     * **************************************************************************************************************
     */

    /**
     * Returns the available platform languages
     *
     * @return string[]
     */
    public static function get_languages()
    {
        return array_merge(
            CommonDataManager :: get_languages(),
            array('platform_language' => Translation :: get('PlatformLanguage', null, 'core\admin')
            )
        );
    }

    /**
     * Returns the available themes
     *
     * @return string[]
     */
    public static function get_themes()
    {
        return Theme :: get_themes();
    }

    /**
     * Returns the available course categories
     *
     * @return string[]
     */
    public static function get_categories()
    {
        $categories = array();

        $categories_result_set = DataManager :: retrieve_course_categories_ordered_by_name();
        while ($category = $categories_result_set->next_result())
        {
            $categories[$category->get_id()] = $category->get_name();
        }

        return $categories;
    }

    /**
     * Returns the users with status teacher
     *
     * @return string[]
     */
    public static function get_titulars()
    {
        $users = array();

        $condition = new EqualityCondition(User :: PROPERTY_STATUS, User :: STATUS_TEACHER);

        $order = array(
            new ObjectTableOrder(User :: PROPERTY_FIRSTNAME),
            new ObjectTableOrder(User :: PROPERTY_LASTNAME)
        );

        $format = PlatformSetting::get_instance()->get_value('fullname_format', User :: CONTEXT);
        if($format == User :: NAME_FORMAT_LAST)
        {
            $order = array_reverse($order);
        }

        $users_result_set = \user\DataManager :: retrieve_active_users($condition, null, null, $order);

        while ($user = $users_result_set->next_result())
        {
            $users[$user->get_id()] = $user->get_fullname() . ' (' . $user->get_official_code() . ')';
        }

        return $users;
    }

    /**
     * **************************************************************************************************************
     * Setting Helper Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns the title for the course breadcrumb depending on the settings
     *
     * @param $course Course
     *
     * @return String
     */
    public static function get_breadcrumb_title_for_course(Course $course)
    {
        $course_settings_controller = CourseSettingsController :: get_instance();
        $breadcrumb_setting = $course_settings_controller->get_course_setting(
            $course->get_id(),
            self :: BREADCRUMB_LAYOUT);

        switch ($breadcrumb_setting)
        {
            case self :: BREADCRUMB_LAYOUT_COURSE_HOME :
                return Translation :: get('CourseHome');
            case self :: BREADCRUMB_LAYOUT_VISUAL_CODE :
                return $course->get_visual_code();
            default :
                return $course->get_title();
        }
    }

    /**
     * **************************************************************************************************************
     * Special Settings Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns the settings that need to be copied to the course object
     *
     * @return string[]
     */
    public static function get_copied_settings_for_course()
    {
        return array(
            self :: LANGUAGE => Course :: PROPERTY_LANGUAGE,
            self :: CATEGORY => Course :: PROPERTY_CATEGORY_ID,
            self :: TITULAR => Course :: PROPERTY_TITULAR_ID);
    }

    /**
     * Returns the course property (if any) belonging to the given course setting
     *
     * @param $course_setting CourseSetting
     *
     * @return String | null
     */
    public static function get_course_property_for_setting(CourseSetting $course_setting)
    {
        $copied_settings = self :: get_copied_settings_for_course();

        if (array_key_exists($course_setting->get_name(), $copied_settings))
        {
            return $copied_settings[$course_setting->get_name()];
        }
    }
}
