<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\Tool;
use common\libraries\Display;
use application\weblcms\integration\tracking\QuestionAttempt;
use application\weblcms\integration\tracking\AssessmentAttempt;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;
use common\libraries\DataClassRetrievesParameters;
use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\Filesystem;
use common\libraries\InCondition;
use common\libraries\NotAllowedException;
use common\libraries\ObjectTableOrder;
use common\libraries\PropertyConditionVariable;
use common\libraries\Request;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assessment\AssessmentResult;
use repository\content_object\assessment\AssessmentResultsExportController;
use repository\content_object\assessment\QuestionResult;
use repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion;
use repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestion;
use repository\content_object\assessment_matching_question\AssessmentMatchingQuestion;
use repository\content_object\assessment_matrix_question\AssessmentMatrixQuestion;
use repository\content_object\assessment_multiple_choice_question\AssessmentMultipleChoiceQuestion;
use repository\content_object\assessment_open_question\AssessmentOpenQuestion;
use repository\content_object\assessment_rating_question\AssessmentRatingQuestion;
use repository\content_object\assessment_select_question\AssessmentSelectQuestion;
use repository\content_object\fill_in_blanks_question\FillInBlanksQuestion;
use repository\content_object\hotspot_question\HotspotQuestion;
use repository\content_object\match_question\MatchQuestion;
use repository\content_object\ordering_question\OrderingQuestion;
use repository\ComplexContentObjectItem;
use repository\ContentObject;

use user\User;

/**
 * Description of raw_export_results
 * 
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Sven Vanpoucke - Hogeschool Gent (Refactoring to abstract exporter in assessment content object)
 */
class AssessmentToolRawExportResultsComponent extends AssessmentTool
{
    const COLUMN_COURSE_GROUP_ID = 'course_group_id';

    /**
     * Runs this component
     *
     * @throws \common\libraries\NotAllowedException
     */
    public function run()
    {
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            throw new NotAllowedException();
        }

        $additional_information_columns = array(
            self :: COLUMN_COURSE_GROUP_ID => Translation :: get(
                'CourseGroups', null, Tool :: get_tool_type_namespace('course_group')
            )
        );

        $controller = new AssessmentResultsExportController(
            $this->get_assessments(), $this->get_assessment_results(), $additional_information_columns
        );

        $path = $controller->run();

        Filesystem :: file_send_for_download($path, true);
        Filesystem :: remove($path);
    }

    /**
     * Retrieves the assessment objects and put them in an array
     *
     * @return Assessment[]
     */
    protected function get_assessments()
    {
        $publication = \application\weblcms\DataManager :: retrieve_by_id(
            ContentObjectPublication :: class_name(), $this->get_publication_id()
        );

        return array($publication->get_content_object());
    }

    /**
     * Returns the publication id
     *
     * @return int
     */
    protected function get_publication_id()
    {
        return Request :: get(self :: PARAM_ASSESSMENT);
    }

    /**
     * Retrieves the assessment attempts and places them in AssessmentResult classes
     *
     * @return AssessmentResult[]
     */
    protected function get_assessment_results()
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(AssessmentAttempt :: class_name(), AssessmentAttempt::PROPERTY_ASSESSMENT_ID),
            new StaticConditionVariable($this->get_publication_id())
        );

        $assessment_results = array();

        $assessment_attempts = WeblcmsTrackingDataManager :: retrieves(AssessmentAttempt :: class_name(), $condition);
        while($assessment_attempt = $assessment_attempts->next_result())
        {
            $assessment_result = new AssessmentResult(
                $assessment_attempt->get_id(), $assessment_attempt->get_assessment_id(),
                $assessment_attempt->get_end_time(), array(), $assessment_attempt->get_start_time(),
                $assessment_attempt->get_total_score(), $assessment_attempt->get_total_time(),
                $assessment_attempt->get_user_id()
            );

            $assessment_result->set_question_results(
                $this->get_question_results_from_assessment_attempt($assessment_attempt, $assessment_result)
            );

            $assessment_results[] = $assessment_result;
        }

        return $assessment_results;
    }

    /**
     * Retrieves the question attempts and places them in the QuestionResult classes
     *
     * @param AssessmentAttempt $assessment_attempt
     * @param AssessmentResult $assessment_result
     *
     * @return QuestionAttempt[]
     */
    protected function get_question_results_from_assessment_attempt($assessment_attempt, $assessment_result)
    {
        $course_groups = \application\weblcms\tool\course_group\DataManager :: get_course_groups_from_user_as_string(
            $assessment_attempt->get_user_id(), $this->get_course_id()
        );

        $additional_information = array(self :: COLUMN_COURSE_GROUP_ID => $course_groups);

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                QuestionAttempt :: class_name(), QuestionAttempt::PROPERTY_ASSESSMENT_ATTEMPT_ID
            ),
            new StaticConditionVariable($assessment_attempt->get_id())
        );

        $question_results = array();

        $question_attempts = WeblcmsTrackingDataManager :: retrieves(QuestionAttempt :: class_name(), $condition);
        while($question_attempt = $question_attempts->next_result())
        {
            $question_results[] = new QuestionResult(
                unserialize($question_attempt->get_answer()), $assessment_result,
                $question_attempt->get_question_cid(), $question_attempt->get_score(),
                $additional_information
            );
        }

        return $question_results;
    }
}
