<?php
namespace application\weblcms\tool\assessment\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use application\weblcms\ContentObjectPublication;
use common\libraries\StaticConditionVariable;
use application\weblcms\tool\assessment\AssessmentTool;
use common\libraries\RecordRetrievesParameters;
use common\libraries\DataClassProperties;
use application\weblcms\tool\assessment\Publication;

/**
 *
 * @author Pieterjan Broekaert
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\application\weblcms\tool\assessment\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use ($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES)
                    );
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES)
                    );
                }

                if (! $success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES)
                    );
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES)
                    );
                }

                if (! $success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES)
                    );
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES)
                    );
                }

                return $success;
            }
        );

        if ($success)
        {
            $this->add_message(
                self :: TYPE_NORMAL,
                Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES)
            );
        }
        else
        {
            $this->add_message(
                self :: TYPE_ERROR,
                Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES)
            );
        }

        return $success;
    }

    /**
     * Upgrade the table structures
     *
     * @param \application\weblcms\tool\assessment\DataManager $data_manager
     *
     * @return bool
     */
    public function upgrade_db($data_manager)
    {
        // Create assessment publication table
        $path = Path :: namespace_to_full_path($data_manager :: context()) . 'php/package/install/publication.xml';
        $storage_unit_info = self :: parse_xml_file($path);

        if (! $data_manager->storage_unit_exist($storage_unit_info['name']))
        {
            if (! $data_manager->create_storage_unit(
                $storage_unit_info['name'],
                $storage_unit_info['properties'],
                $storage_unit_info['indexes']
            )
            )
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                        'CreateStorageUnitFailed', array('UNIT' => $storage_unit_info['name']),
                        'core\lynx'
                    )
                );

                return false;
            }
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(),
                ContentObjectPublication :: PROPERTY_TOOL
            ),
            new StaticConditionVariable('assessment')
        );

        $parameters = new RecordRetrievesParameters(
            new DataClassProperties(
                array(
                    new PropertyConditionVariable(
                        ContentObjectPublication :: class_name(),
                        ContentObjectPublication :: PROPERTY_ID
                    )
                )
            ),
            $condition
        );

        $publications = \application\weblcms\DataManager :: records(
            ContentObjectPublication :: class_name(), $parameters
        );

        while ($publication = $publications->next_result())
        {
            $assessment_publication = new Publication();
            $assessment_publication->set_publication_id($publication[ContentObjectPublication :: PROPERTY_ID]);
            $assessment_publication->set_hints_allowed(1);
            $assessment_publication->set_feedback_summary(1);
            $assessment_publication->set_feedback_per_page(0);
            $assessment_publication->set_feedback_type(0);

            if (! $assessment_publication->create())
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                        'CreateAssessmentPublicationFailed',
                        array('ID' => $publication[ContentObjectPublication :: PROPERTY_ID])
                    )
                );

                return false;
            }
        }

        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     *
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration($data_manager)
    {
        $package = Package :: get($data_manager :: context());
        $registration = CommonDataManager :: get_registration($data_manager :: context());
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }

        return true;
    }
}
