<?php

namespace application\weblcms\tool\assignment;

use common\libraries\FormValidator;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use tracking\Event;

/**
 * This form allows a user to edit and save notes with a submission.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class SubmissionDetailNotesSection extends FormValidator
{

    /**
     * Caching variable for the note tracker.
     *
     * @var \application\weblcms\integration\tracking\SubmissionNote The note tracker
     */
    private $note_tracker;

    /**
     * Fills the form with all the data.
     *
     * @param $note_tracker \application\weblcms\integration\tracking\SubmissionNote
     * @param $url string
     *            The url of the main page
     */
    public function __construct($note_tracker, $url = '')
    {
        parent :: __construct('submission_notes', 'post', $url);
        $this->note_tracker = $note_tracker;

        $html_editor_options = array();
        $html_editor_options['collapse_toolbar'] = true;
        $html_editor_options['show_tags'] = false;
        $html_editor_options['toolbar'] = 'RepositoryQuestion';

        $this->add_html_editor(
            \application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_NOTE, Translation :: get('NoteLong'),
            false, $html_editor_options
        );

        $save_button = $this->createElement(
            'style_submit_button', 'submit', Translation :: get('Save', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive publish'
            )
        );
        $this->addElement($save_button);

        $this->set_initial_note();
    }

    /**
     * If a note already exists, set this note on the form.
     */
    private function set_initial_note()
    {
        if ($this->note_tracker)
        {
            $defaults = array();
            $defaults[\application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_NOTE] =
                $this->note_tracker->get_note();
            parent :: setDefaults($defaults);
        }
    }

    /**
     * Updates or creates the note tracker.
     */
    public function set_note()
    {
        if ($this->note_tracker)
        {
            $this->update_tracker();
        }
        else
        {
            $this->create_tracker();
        }
    }

    /**
     * Creates a note tracker.
     *
     * @return boolean
     */
    public function create_tracker()
    {
        $values = $this->exportValues();
        $note = $values[\application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_NOTE];

        if ($note == "" || $note == "<br />")
        {
            return false;
        }

        $arguments = array(
            \application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_SUBMISSION_ID => Request :: get(
                AssignmentTool :: PARAM_SUBMISSION
            ),
            \application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_CREATED => time(),
            \application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_MODIFIED => time(),
            \application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_USER_ID => Request :: get(
                AssignmentTool :: PARAM_TARGET_ID
            ),
            \application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_NOTE => $note
        );
        Event :: trigger('note_submission', WeblcmsManager :: context(), $arguments);

        return true;
    }

    /**
     * Updates the score tracker if the note is modified.
     *
     * @return boolean
     */
    public function update_tracker()
    {
        $values = $this->exportValues();
        $note = $values[\application\weblcms\integration\tracking\SubmissionNote :: PROPERTY_NOTE];

        if ($note == "" || $note == "<br />")
        {
            return $this->note_tracker->delete();
        }

        $this->note_tracker->set_note($note);
        $this->note_tracker->set_modified(time());

        return $this->note_tracker->update();
    }
}
