<?php
namespace application\weblcms\tool\assignment;

use application\weblcms\Tool;
use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\reporting\ReportingTool;


use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\ActionBarRenderer;
use common\libraries\AndCondition;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\PatternMatchCondition;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\DataManager as GroupDataManager;
use group\Group;
use reporting\Reporting;
use reporting\ReportingManager;
use user\User;
use user\UserDataManager;

use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

/**
 *
 * @package application.weblcms.tool.assignment.php.component Viewer for the submissions
 *
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolSubmissionsBrowserComponent extends SubmissionsManager
{

    /**
     * The assignment content object
     *
     * @var Assignment
     */
    private $assignment;

    /**
     * The id of the assignment publication
     *
     * @var int
     */
    private $publication_id;

    /**
     * ID of the submitter (user or group)
     *
     * @var int
     */
    private $submitter_id;

    /**
     * The type of the submitter (user = 0, course group = 1, platform group = 2)
     *
     * @var int
     */
    private $submitter_type;

    /**
     * Submissions of the submitter for an assignment
     *
     * @var array \application\weblcms\integration\tracking\AssignmentSubmission
     */
    private $submissions;

    public function run()
    {
        $this->test_view_rights();
        $this->submitter_id = Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        $this->submitter_type = Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);

        switch ($this->submitter_type)
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                $breadcrumb_title = \user\DataManager :: get_fullname_from_user($this->submitter_id);
                break;
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                $breadcrumb_title = \application\weblcms\tool\course_group\DataManager :: retrieve(
                    CourseGroup :: class_name(),
                    $this->submitter_id
                )->get_name();
                break;
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $breadcrumb_title = GroupDataManager :: retrieve_by_id(Group :: class_name(), $this->submitter_id)->get_name();
                break;
        }

        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();
        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb(
            $this->get_url(
                array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                    Tool :: PARAM_PUBLICATION_ID => $this->publication_id,
                    AssignmentTool :: PARAM_TARGET_ID => $this->submitter_id,
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => $this->submitter_type
                )
            ), $breadcrumb_title
        );
        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
        $this->get_submissions_from_target();
        $this->display_assignment_submissions();
    }

    /**
     * This function checks whether or not a user has permission to access this page.
     */
    private function test_view_rights()
    {
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id()
        );

        if (
            !$this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication) ||
            !$this->is_allowed(WeblcmsRights :: VIEW_RIGHT)
        )
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID
                )
            );
        }
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT) || $this->assignment->get_visibility_submissions())
        {
            return;
        }
        $is_member = false;
        switch ($this->get_submitter_type())
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                $is_member = $this->is_course_group_member($this->get_target_id(), $this->get_user_id());
                break;
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $is_member = $this->is_platform_group_member($this->get_target_id(), $this->get_user_id());
                break;
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                $is_member = $this->is_current_user($this->get_target_id());
                break;
        }
        if (!$is_member)
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID
                )
            );
        }
    }

    private function is_course_group_member($group_id, $user_id)
    {
        return \application\weblcms\tool\course_group\DataManager :: is_course_group_member($group_id, $user_id);
    }

    private function is_platform_group_member($group_id, $user_id)
    {
        $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $group_id);
        if (GroupDataManager :: is_group_member($group_id, $user_id))
        {
            return true;
        }
        if ($group->has_children())
        {
            foreach ($group->get_subgroups() as $subgroup)
            {
                if ($this->is_platform_group_member($subgroup->get_id(), $user_id))
                {
                    return true;
                }
            }
        }
        return false;
    }

    private function is_current_user($user_id)
    {
        return $this->get_user_id() == $user_id;
    }

    public function get_assignment()
    {
        return $this->assignment;
    }

    /**
     * Gets all submissions from a target user or group for the publication
     */
    private function get_submissions_from_target()
    {
        $order_by[] = new ObjectTableOrder(\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_DATE_SUBMITTED);
        $this->submissions = \application\weblcms\integration\tracking\AssignmentSubmission :: get_data(
            \application\weblcms\integration\tracking\AssignmentSubmission :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME,
            $this->get_table_conditions(), null, null, $order_by
        );
    }

    /**
     * Constructs the conditions needed by the database manager to retrieve the necessary submissions.
     *
     * @return \common\libraries\AndCondition the aggregate of all the conditions applicable.
     */
    private function get_table_conditions()
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID, $this->publication_id
        );
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID, $this->submitter_id
        );
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE, $this->submitter_type
        );
        return new AndCondition($conditions);
    }

    /**
     * Displays the assignment, submitter and submissions
     *
     * @return null
     */
    // FIXME too much different stuff for this function
    private function display_assignment_submissions()
    {
        if ($this->submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            //$name = UserDataManager :: get_instance()->retrieve_user($this->submitter_id)->get_fullname();
        }
        elseif ($this->submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP)
        {
            $group = \application\weblcms\DataManager :: retrieve(
                CourseGroup :: class_name(),
                $this->submitter_id
            );

            //$name = $group->get_name();
        }
        else
        {
            $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $this->submitter_id);
            //$name = $group->get_name();
        }

        // Check if toolbar should be shown
        if ($this->submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER &&
            $this->get_user_id() == $this->submitter_id
        )
        {
            $display_add = true;
        }
        elseif ($this->submitter_type != \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            if ($this->submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP)
            {
                $display_add = \application\weblcms\tool\course_group\DataManager :: is_course_group_member(
                    $group->get_id(), $this->get_user_id()
                );
            }
            else
            {
                $display_add = $this->is_group_member($group);
            }
        }
        $this->display_header($display_add);

        $html = array();

        $html[] = $this->generate_navigation_bar_html();
        $html[] =
            '<div class="announcements level_1" style="background-image: url(' . Theme :: get_common_image_path() .
                'content_object/introduction.png)">';
        $html[] = $this->generate_assignment_details_html();

        // Display group members
        if ($this->submitter_type != \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            $html[] = '<div class="description" style="font-weight:bold;float:left">';
            $html[] = Translation :: get('GroupMembers') . ':&nbsp;';
            $html[] = '</div>';
            $html[] = '<div style="float:left">';
            $html[] = $this->display_group_members();
            $html[] = '<br/><br/></div>';
        }
        $html[] = $this->get_reporting_as_html();
        $html[] = '</div>';

        echo implode("\n", $html);

        if ($this->submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            $table = new SubmitterUserSubmissionsBrowserTable(
                $this, $this->get_search_condition(), $this->get_table_conditions()
            );
            echo $table->as_html();
        }
        else
        {
            $table = new SubmitterGroupSubmissionsBrowserTable(
                $this, $this->get_search_condition(), $this->get_table_conditions()
            );
            echo $table->as_html();
        }

        $this->display_footer();
    }

    private function generate_navigation_bar_html()
    {
        $html = array();
        $html[] = '<div class="announcements level_2" style="background-image:url(' . Theme :: get_common_image_path() .
            'content_object/introduction.png;width=100%">';

        if ($this->assignment->get_visibility_submissions() || $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $html[] = $this->generate_submitters_navigator();
        }
        $html[] = '</div>';
        $html[] = '<div class="clear">&nbsp;</div><br/>';
        return implode("\n", $html);
    }

    private function generate_submitters_navigator()
    {
        $html = array();
        $html[] = '<div style="text-align:center">';
        $previous_submitter_url = $this->get_previous_submitter_url();
        if ($previous_submitter_url)
        {
            $html[] = '<a href="' . $previous_submitter_url . '">';
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_prev.png"/>';
            $html[] = Translation :: get('PreviousSubmitter');
            $html[] = '</a>';
        }
        else
        {
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_prev_na.png"/>';
            $html[] = Translation :: get('PreviousSubmitter');
        }
        $html[] = ' [' . $this->get_position_submitter($this->get_submitter_type(), $this->get_target_id()) . '/' .
            $this->get_count_submitters($this->get_submitter_type()) . '] ';
        $next_submitter_url = $this->get_next_submitter_url();
        if ($next_submitter_url)
        {
            $html[] = '<a href="' . $next_submitter_url . '">';
            $html[] = Translation :: get('NextSubmitter');
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_next.png"/>';
            $html[] = '</a>';
        }
        else
        {
            $html[] = Translation :: get('NextSubmitter');
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_next_na.png"/>';
        }
        $html[] = '</div>';
        return implode("\n", $html);
    }

    private function get_previous_submitter_url()
    {
        $previous_submitter_information = $this->get_previous_submitter_information(
            $this->get_submitter_type(), $this->get_target_id()
        );
        if (!$previous_submitter_information)
        {
            return null;
        }
        return $this->get_url(
            array(
                AssignmentTool :: PARAM_TARGET_ID =>
                $previous_submitter_information[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID],
                AssignmentTool :: PARAM_SUBMITTER_TYPE =>
                $previous_submitter_information[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE],
                AssignmentTool :: PARAM_SUBMISSION =>
                $previous_submitter_information[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_ID]
            )
        );
    }

    private function get_next_submitter_url()
    {
        $next_submitter_information = $this->get_next_submitter_information(
            $this->get_submitter_type(), $this->get_target_id()
        );
        if (!$next_submitter_information)
        {
            return null;
        }
        return $this->get_url(
            array(
                AssignmentTool :: PARAM_TARGET_ID =>
                $next_submitter_information[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID],
                AssignmentTool :: PARAM_SUBMITTER_TYPE =>
                $next_submitter_information[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE],
                AssignmentTool :: PARAM_SUBMISSION =>
                $next_submitter_information[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_ID]
            )
        );
    }

    /**
     * Recursively iterates over a platform group and its subgroups to identify whether the current user is a member of
     * the platform group at any level.
     *
     * @param Group $group the platform group.
     * @return boolean whether the user is a member of the platform group
     */
    private function is_group_member($group)
    {
        // DMTODO The GroupDataManager should provide a simpler way to find out
        if (GroupDataManager :: is_group_member($group->get_id(), $this->get_user_id()))
        {
            return true;
        }
        if ($group->has_children())
        {
            foreach ($group->get_subgroups() as $subgroup)
            {
                if ($this->is_group_member($subgroup))
                {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Retrieves the members of a group.
     *
     * @return String a comma separated list of names.
     */
    private function display_group_members()
    {
        if ($this->submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP)
        {
            $users = CourseGroupDataManager :: retrieve_course_group_users($this->submitter_id)->as_array();
        }
        else
        {
            $users = UserDataManager :: get_instance()->retrieve_users(
                new InCondition(
                    User :: PROPERTY_ID,
                        GroupDataManager :: retrieve_by_id(Group :: class_name(),
                        $this->submitter_id
                    )->get_users(true, true)
                )
            )->as_array();
        }

        $user_names = array();

        foreach ($users as $user)
        {
            $user_names[$user->get_lastname()] = $user->get_fullname();
        }

        ksort($user_names);

        return implode(", ", $user_names);
    }

    public function get_search_condition()
    {
        $query = $this->action_bar->get_query();

        if (isset($query) && $query != '')
        {
            return new PatternMatchCondition(AssignmentTool :: PROPERTY_TITLE, '*' . $query . '*');
        }

        return null;
    }

    /**
     * Returns the reporting as an array of strings.
     *
     * @return array The reporting
     */
    private function get_reporting_as_html()
    {
        $title = array();
        if ($this->is_feedback_visible($this->assignment, $this->has_submissions()))
        {
            $title[] = Translation :: get('AutomaticFeedback');
        }
        $title[] = Translation :: get('Reporting');

        $html = array();

        $html[] = '<div class="title" style="border-top:1px dotted #D3D3D3;padding-top:5px;width:100%;">';
        $html[] = implode(' & ', $title);
        $html[] = '</div><div class="clear">&nbsp;</div>';

        if ($this->is_feedback_visible($this->assignment, $this->has_submissions()))
        {
            $html[] = $this->generate_automatic_feedback_html();
        }

        if (!$this->is_feedback_visible($this->assignment, $this->has_submissions()))
        {
            $html[] = '<br />';
        }

        $html[] = '<div style="font-weight:bold;float:left;">';
        $html[] = Translation :: get('SubmissionsWithScore') . ':&nbsp;<br />';
        $html[] = Translation :: get('SubmissionsWithFeedback') . ':&nbsp;<br/>';
        $html[] = Translation :: get('AverageScore') . ':&nbsp;<br />';
        $html[] = '</div>';

        $html[] = '<div style="float:left;">';
        $html[] = $this->get_reporting_data_as_html();
        $html[] = '</div>';

        return implode("\n", $html);
    }

    /**
     * Checks whether the table has submissions.
     *
     * @return boolean True if the table has submissions
     */
    private function has_submissions()
    {
        return \application\weblcms\integration\tracking\AssignmentSubmission :: count_data(
            \application\weblcms\integration\tracking\AssignmentSubmission :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME,
            $this->get_table_conditions()
        ) > 0;
    }

    /**
     * Returns the reporting data as an array of strings.
     *
     * @return array The reporting data
     */
    private function get_reporting_data_as_html()
    {
        $score = 0;
        $count_score = 0;
        $count_feedback = 0;
        $count_submissions = 0;

        foreach ($this->submissions->as_array() as $submission)
        {
            $score_tracker = $this->get_score_tracker_for_submission($submission->get_id());

            if ($score_tracker)
            {
                $score += $score_tracker->get_score();
                $count_score++;
            }

            $feedback_tracker = $this->get_feedback_tracker_for_submission($submission->get_id());

            if ($feedback_tracker)
            {
                $count_feedback++;
            }

            $count_submissions++;
        }

        $html = array();
        $html[] = $count_score . '/' . $count_submissions . '<br />';
        $html[] = $count_feedback . '/' . $count_submissions . '<br/>';

        if ($count_score == 0)
        {
            $html[] = '-';
        }
        else
        {
            $html[] = round($score / $count_score, 2) . '%<br />';
        }

        return implode("\n", $html);
    }

    /**
     * Gets a toolbar with common actions
     *
     * @return ActionBarRenderer
     */
    public function get_toolbar($display_add)
    {
        // $display_add = false;
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($this->get_submissions_tracker($this->get_submitter_type(), $this->get_target_id()))
            {
                $action_bar->add_common_action(
                    new ToolbarItem(
                        Translation :: get('DownloadAllSubmissions'),
                        Theme :: get_common_image_path() . 'action_download.png', $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS
                            )
                        ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
                    )
                );
            }

            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('ScoresOverview'),
                    Theme :: get_common_image_path() . 'action_statistics.png', $this->get_url(
                        array(
                            WeblcmsManager :: PARAM_TOOL => WeblcmsManager :: ACTION_REPORTING,
                            ReportingManager :: PARAM_TEMPLATE_ID => \application\weblcms\integration\reporting\CourseSubmitterSubmissionsTemplate :: class_name(),
                            WeblcmsManager :: PARAM_PUBLICATION => $this->get_publication_id(),
                            AssignmentTool :: PARAM_TARGET_ID => $this->get_target_id(),
                            AssignmentTool :: PARAM_SUBMITTER_TYPE => $this->get_submitter_type(),
                            WeblcmsManager :: PARAM_TOOL_ACTION => ReportingTool :: ACTION_VIEW
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        if ($display_add)
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('SubmissionSubmit'),
                    Theme :: get_common_image_path() . 'action_add.png', $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_SUBMIT_SUBMISSION,
                            Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id()
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }
        return $action_bar;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $this->publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        $pub = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id()
        );

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $pub))
        {
            $this->redirect(Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(),
                    array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID,
                            AssignmentTool :: PARAM_SUBMITTER_TYPE));
        }

        $this->assignment = $pub->get_content_object();

//         $breadcrumbtrail->add(
//                 new Breadcrumb($this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE)),
//                         Translation :: get('AssignmentToolBrowserComponent')));
//         $breadcrumbtrail->add(
//                 new Breadcrumb($this->define_browse_submitters_action_url(), $this->assignment->get_title()));
    }

    /*private function define_browse_submitters_action_url()
    {
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            return $this->get_url(
                array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                    Tool :: PARAM_PUBLICATION_ID => $this->publication_id
                )
            );
        }
        return $this->get_url(
            array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS,
                Tool :: PARAM_PUBLICATION_ID => $this->publication_id
            )
        );
        return AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS;
    }*/

    public function display_header($display_add)
    {
        parent :: display_header();

        $this->action_bar = $this->get_toolbar($display_add);

        if ($this->action_bar)
        {
            echo $this->action_bar->as_html();
        }
    }

    public function get_additional_parameters()
    {
        return array(
            Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID,
            AssignmentTool :: PARAM_SUBMITTER_TYPE
        );
    }
}
