<?php
namespace application\weblcms\tool\assignment;

use common\libraries\DelegateComponent;
use common\libraries\PropertyConditionVariable;
use common\libraries\TableSupport;
use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\Tool;
use application\weblcms\tool\reporting\ReportingTool;
use application\weblcms\tool\ephorus\EphorusTool;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\ActionBarRenderer;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DynamicVisualTab;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\Redirect;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\Group;
use reporting\Reporting;
use reporting\ReportingManager;
use user\User;
use application\weblcms\integration\tracking\AssignmentSubmission;

/**
 *
 * @package application.weblcms.tool.assignment.php.component Browser of the users and groups the assignment was
 *          published for, with their submission details
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolSubmittersBrowserComponent extends SubmissionsManager implements DelegateComponent, TableSupport
{
    /**
     * The submitter type
     *
     * @var int
     */
    private $submitter_type;

    /**
     * The assignment content object
     *
     * @var Assignment
     */
    public $assignment;

    /**
     * All users that need to be shown in the individual submitters list.
     *
     * @var array \application\weblcms\integration\tracking\AssignmentSubmission
     */
    private $users;

    /**
     * All course groups that need to be shown on the course groups tab.
     *
     * @var array \application\weblcms\integration\tracking\AssignmentSubmission
     */
    private $course_groups;

    /**
     * All platform groups that need to be shown on the platform groups tab.
     *
     * @var array \application\weblcms\integration\tracking\AssignmentSubmission
     */
    private $platform_groups;

    public function run()
    {
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id());

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication) ||
             ! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            // if ($this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication) &&
            // $this->is_allowed(WeblcmsRights :: VIEW_RIGHT))
            {
                $this->redirect(
                    null,
                    false,
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS,
                        Tool :: PARAM_PUBLICATION_ID => $publication->get_id()));
            }
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }

        $this->assignment = $publication->get_content_object();

        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();
        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb(
            $this->get_url(
                array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                    Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id())),
            $this->assignment->get_title());
        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);

        $this->get_submitters_data();
        $this->browse_submitters();
    }

    /**
     * Returns the assignment.
     *
     * @return Assignment
     */
    public function get_assignment()
    {
        return $this->assignment;
    }

    /**
     * Gets the users, course groups and platform groups the assignment was published for plus the number of their
     * submissions and feedbacks
     */
    private function get_submitters_data()
    {
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id());

        // Users
        $this->users = \application\weblcms\DataManager :: get_publication_target_users_by_publication_id(
            $this->get_publication_id());
        // Course groups
        $this->course_groups = \application\weblcms\DataManager :: retrieve_publication_target_course_groups(
            $this->get_publication_id(),
            $publication->get_course_id())->as_array();
        // Platform groups
        $this->platform_groups = \application\weblcms\DataManager :: retrieve_publication_target_platform_groups(
            $this->get_publication_id(),
            $publication->get_course_id())->as_array();
    }

    /**
     * Displays assignment details and tabs with a submitters browser table per submitter type
     */
    public function browse_submitters()
    {
        $visible = $this->display_header();
        if (! $visible)
        {
            return;
        }

        echo '<div class="announcements level_1" style="background-image: url(' . Theme :: get_common_image_path() .
             'content_object/introduction.png);">';
        echo $this->generate_assignment_details_html();
        echo $this->get_reporting_html();
        echo '</div><br />';

        // retrieve group submissions allowed
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id());

        $assignment = $publication->get_content_object();
        $allow_group_submissions = $assignment->get_allow_group_submissions();

        if ($allow_group_submissions == 0)
        {
            $this->submitter_type = AssignmentSubmission :: SUBMITTER_TYPE_USER;

            $users_table = new SubmissionUsersBrowserTable($this);
            $users_html = $users_table->as_html();
            echo $users_html;
        }
        else
        {
            // Create tabs per submitter type
            $type = Request :: get(AssignmentTool :: PARAM_TYPE);
            if ($type == null)
            {
                $type = self :: TYPE_COURSE_GROUP;
            }
            switch ($type)
            {
                case self :: TYPE_COURSE_GROUP :
                    $this->submitter_type = AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP;
                    $course_groups_table = new SubmissionCourseGroupsBrowserTable($this);
                    $content_table = $course_groups_table->as_html();
                    $selected_course_group = true;
                    break;
                case self :: TYPE_GROUP :
                    $this->submitter_type = AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP;
                    $groups_table = new SubmissionGroupsBrowserTable($this);
                    $content_table = $groups_table->as_html();
                    $parameters[] = $parameters[AssignmentTool :: PARAM_TYPE] = self :: TYPE_GROUP;
                    $selected_group = true;
                    break;
            }
            $parameters_course_group = $this->get_parameters();
            $parameters_course_group[AssignmentTool :: PARAM_TYPE] = self :: TYPE_COURSE_GROUP;
            $link_course_group = $this->get_url($parameters_course_group);
            $parameters_group = $this->get_parameters();
            $parameters_group[AssignmentTool :: PARAM_TYPE] = self :: TYPE_GROUP;
            $link_group = $this->get_url($parameters_group);

            $tabs = new DynamicVisualTabsRenderer('submissions', $content_table);
            $tab_course_group = new DynamicVisualTab(
                'tab_course_group',
                Translation :: get('CourseGroups'),
                null,
                $link_course_group,
                $selected_course_group);
            $tabs->add_tab($tab_course_group);
            $tab_group = new DynamicVisualTab(
                'tab_group',
                Translation :: get('Groups'),
                null,
                $link_group,
                $selected_group);
            $tabs->add_tab($tab_group);
            echo $tabs->render();
        }

        $this->display_footer();
    }

    /**
     * Returns the submitter type
     *
     * @return int
     */
    public function get_submitter_type()
    {
        return $this->submitter_type;
    }

    /**
     * Returns the reporting as an array of strings.
     *
     * @return array The reporting
     */
    private function get_reporting_html()
    {
        $html = array();

        // Reporting title
        $html[] = '<div class="title" style="border-bottom:1px dotted #D3D3D3;width:100%;border-top:1px dotted #D3D3D3;
            padding-top:5px">';
        $html[] = Translation :: get('Reporting');
        $html[] = '</div><div class="clear">&nbsp;</div><br />';

        $html[] = '<div style="font-weight:bold;float:left">';

        if (! $this->assignment->get_allow_group_submissions())
        {
            $html[] = Translation :: get('UsersSubmissions') . ':&nbsp;<br />';
            $html[] = Translation :: get('UsersFeedback') . ':&nbsp;<br />';

            if ($this->assignment->get_allow_late_submissions())
            {
                $html[] = Translation :: get('UsersLateSubmissions') . ':&nbsp;<br />';
            }
        }
        else
        {
            $type = Request :: get(AssignmentTool :: PARAM_TYPE);

            if ($type == null)
            {
                $type = self :: TYPE_COURSE_GROUP;
            }
            switch ($type)
            {
                case self :: TYPE_COURSE_GROUP :
                    $html[] = Translation :: get('CourseGroupsSubmissions') . ':&nbsp;<br />';
                    $html[] = Translation :: get('CourseGroupsFeedback') . ':&nbsp;<br />';
                    if ($this->assignment->get_allow_late_submissions())
                    {
                        $html[] = Translation :: get('CourseGroupsLateSubmissions') . ':&nbsp;<br />';
                    }
                    break;

                case self :: TYPE_GROUP :
                    $html[] = Translation :: get('PlatformGroupsSubmissions') . ':&nbsp;<br />';
                    $html[] = Translation :: get('PlatformGroupsFeedback') . ':&nbsp;<br />';
                    if ($this->assignment->get_allow_late_submissions())
                    {
                        $html[] = Translation :: get('PlatformGroupsLateSubmissions') . ':&nbsp;<br />';
                    }
                    break;
            }
        }
        $html[] = '</div>';

        // Reporting data
        $html[] = '<div style="float:left">';
        $html[] = $this->get_reporting_block_html();
        $html[] = '</div>';

        return implode("\n", $html);
    }

    /**
     * Returns the reporting data block as an array of strings.
     *
     * @return array The reporting data block
     */
    private function get_reporting_block_html()
    {
        if (! $this->assignment->get_allow_group_submissions())
        {
            return $this->get_reporting_data_html(AssignmentSubmission :: SUBMITTER_TYPE_USER);
        }
        else
        {
            $type = Request :: get(AssignmentTool :: PARAM_TYPE);

            if ($type == null)
            {
                $type = self :: TYPE_COURSE_GROUP;
            }
            switch ($type)
            {
                case self :: TYPE_COURSE_GROUP :
                    return $this->get_reporting_data_html(AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP);
                case self :: TYPE_GROUP :
                    return $this->get_reporting_data_html(AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP);
            }
        }
    }

    /**
     * Returns the reporting data from the specified type as an array of strings.
     *
     * @param $submitter_type int The type of submitter
     * @return array The reporting data
     */
    private function get_reporting_data_html($submitter_type)
    {
        $count_submissions = 0;
        $count_feedbacks = 0;
        $count_late_submissions = 0;

        $submitters = array();

        $html = array();

        switch ($submitter_type)
        {
            case AssignmentSubmission :: SUBMITTER_TYPE_USER :
                $submitters = $this->users;
                break;
            case AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                $submitters = $this->course_groups;
                break;
            case AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $submitters = $this->platform_groups;
                break;
        }

        foreach ($submitters as $submitter)
        {

            switch ($submitter_type)
            {
                case AssignmentSubmission :: SUBMITTER_TYPE_USER :
                    $submissions_tracker = $this->get_submissions_tracker(
                        $submitter_type,
                        $submitter[User :: PROPERTY_ID]);
                    $feedbacks_tracker = $this->get_submitter_feedbacks(
                        $submitter_type,
                        $submitter[User :: PROPERTY_ID]);
                    break;
                case AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                    break;
                case AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                    $submissions_tracker = $this->get_submissions_tracker($submitter_type, $submitter->get_id());
                    $feedbacks_tracker = $this->get_submitter_feedbacks($submitter_type, $submitter->get_id());
                    break;
            }

            if ($submissions_tracker)
            {
                $count_submissions ++;

                if ($submissions_tracker['last_date'] > $this->assignment->get_end_time())
                {
                    $count_late_submissions ++;
                }
            }

            if ($feedbacks_tracker)
            {
                $count_feedbacks ++;
            }
        }

        $html[] = $count_submissions . '/' . count($submitters) . '<br />';
        $html[] = $count_feedbacks . '/' . count($submitters) . '<br />';
        if ($this->assignment->get_allow_late_submissions())
        {
            $html[] = $count_late_submissions . '/' . count($submitters) . '<br />';
        }

        return implode("\n", $html);
    }

    /**
     * Gets a toolbar with common actions
     *
     * @return ActionBarRenderer
     */
    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($this->get_submission_trackers_by_publication($this->get_publication_id()))
            {
                $action_bar->add_common_action(
                    new ToolbarItem(
                        Translation :: get('DownloadAllSubmissions'),
                        Theme :: get_common_image_path() . 'action_download.png',
                        $this->get_url(array(Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS)),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }

            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('ScoresOverview'),
                    Theme :: get_common_image_path() . 'action_statistics.png',
                    $this->get_url(
                        array(
                            WeblcmsManager :: PARAM_TOOL => WeblcmsManager :: ACTION_REPORTING,
                            ReportingManager :: PARAM_TEMPLATE_ID => \application\weblcms\integration\reporting\CourseAssignmentSubmittersTemplate :: class_name(),
                            WeblcmsManager :: PARAM_TOOL_ACTION => ReportingTool :: ACTION_VIEW)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));

            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('SubmissionsOverview'),
                    Theme :: get_common_image_path() . 'action_statistics.png',
                    $this->get_url(
                        array(
                            WeblcmsManager :: PARAM_TOOL => WeblcmsManager :: ACTION_REPORTING,
                            ReportingManager :: PARAM_TEMPLATE_ID => \application\weblcms\integration\reporting\AssignmentSubmissionsTemplate :: class_name(),
                            WeblcmsManager :: PARAM_TOOL_ACTION => ReportingTool :: ACTION_VIEW)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));

            $course = $this->get_course();
            $ephorus_tool = \application\weblcms\DataManager :: retrieve_course_tool_by_name('ephorus');
            if ($ephorus_tool && $course->get_course_setting(
                \application\weblcms\CourseSetting :: COURSE_SETTING_TOOL_ACTIVE,
                $ephorus_tool->get_id()))
            {
                $action_bar->add_tool_action(
                    new ToolbarItem(
                        Translation :: get('EphorusOverview'),
                        Theme :: get_image_path(Tool :: get_tool_type_namespace('ephorus')) . 'logo/16.png',
                        $this->get_url(
                            array(
                                WeblcmsManager :: PARAM_TOOL => 'ephorus',
                                WeblcmsManager :: PARAM_PUBLICATION => $this->get_publication_id(),
                                WeblcmsManager :: PARAM_TOOL_ACTION => EphorusTool :: ACTION_ASSIGNMENT_BROWSER)),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
        }

        $action_bar->add_common_action(
            new ToolbarItem(
                Translation :: get('SubmissionSubmit'),
                Theme :: get_common_image_path() . 'action_add.png',
                $this->get_url($this->generate_add_submission_url()),
                ToolbarItem :: DISPLAY_ICON_AND_LABEL));

        $type = Request :: get(AssignmentTool :: PARAM_TYPE);
        $url = $this->get_url(array(AssignmentTool :: PARAM_TYPE => $type));
        $action_bar->set_search_url($url);
        $action_bar->add_common_action(
            new ToolbarItem(
                Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_browser.png',
                $url,
                ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                false));

        return $action_bar;
    }

    /**
     * Returns the url for the add submission button. When it's not a group assignment the url will have the extra
     * parameters target id and submitter type.
     *
     * @return array The url
     */
    private function generate_add_submission_url()
    {
        if ($this->assignment->get_allow_group_submissions())
        {
            return array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_SUBMIT_SUBMISSION,
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id());
        }
        else
        {
            return array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_SUBMIT_SUBMISSION,
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(),
                AssignmentTool :: PARAM_TARGET_ID => $this->get_user_id(),
                AssignmentTool :: PARAM_SUBMITTER_TYPE => AssignmentSubmission :: SUBMITTER_TYPE_USER);
        }
    }

    /**
     * Returns the condition
     *
     * @param string $table_class_name
     *
     * @return \common\libraries\Condition
     */
    public function get_table_condition($table_class_name)
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            switch ($this->get_submitter_type())
            {
                case AssignmentSubmission :: SUBMITTER_TYPE_USER :
                    $conditions = array();
                    $conditions[] = new PatternMatchCondition(
                        new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_FIRSTNAME),
                        '*' . $query . '*');
                    $conditions[] = new PatternMatchCondition(
                        new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_LASTNAME),
                        '*' . $query . '*');
                    return new OrCondition($conditions);
                case AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                    return new PatternMatchCondition(
                        new PropertyConditionVariable(CourseGroup :: class_name(), CourseGroup :: PROPERTY_NAME),
                        '*' . $query . '*');
                case AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                    return new PatternMatchCondition(
                        new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_NAME),
                        '*' . $query . '*');
            }
        }
        return null;
    }

    public function display_header()
    {
        parent :: display_header();

        $this->action_bar = $this->get_action_bar();

        if ($this->action_bar)
        {
            echo $this->action_bar->as_html();
        }

        return true;
    }

    public function get_additional_parameters()
    {
        return array(
            Tool :: PARAM_PUBLICATION_ID,
            AssignmentTool :: PARAM_TARGET_ID,
            AssignmentTool :: PARAM_TYPE);
    }
}
