<?php
namespace common\libraries\package;

use DOMDocument;
use common\libraries\CommonDataManager;
use common\libraries\Path;
use common\libraries\Registration;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
abstract class Upgrader extends Action
{

    /**
     *
     * @var \common\libraries\DataManager
     */
    private $data_manager;

    /**
     *
     * @param \common\libraries\DataManager $data_manager
     */
    public function __construct($data_manager = null)
    {
        $this->data_manager = $data_manager;
    }

    /**
     *
     * @param string $context
     * @param \common\libraries\DataManager $data_manager
     *
     * @return \common\libraries\package\Upgrader
     */
    public static function factory($context)
    {
        $package = Package :: get($context);
        $version = str_replace(array('-', '+', '.'), '_', $package->get_version());

        $class = $context . '\upgrade\v' . $version . '\Upgrader';

        if (! class_exists($class))
        {
            throw new \Exception(Translation :: get('UpgraderNotFound', array('CONTEXT' => $context)));
        }

        return new $class();
    }

    /**
     * Upgrades the package
     *
     * @return boolean
     */
    abstract function run();

    /**
     *
     * @param \common\libraries\DataManager $data_manager
     */
    public function set_data_manager($data_manager)
    {
        $this->data_manager = $data_manager;
    }

    /**
     *
     * @return \common\libraries\DataManager
     */
    public function get_data_manager()
    {
        return $this->data_manager;
    }

    /**
     * Parses an XML file describing a storage unit. For defining the 'type' of the field, the same definition is used
     * as the PEAR::MDB2 package. See http://pear.php.net/manual/en/package.database. mdb2.datatypes.php
     *
     * @param $file string The complete path to the XML-file from which the storage unit definition should be read.
     *
     * @return array An with values for the keys 'name','properties' and 'indexes'
     */
    public static function parse_xml_file($file)
    {
        $name = '';
        $properties = array();
        $indexes = array();

        $doc = new DOMDocument();
        $doc->load($file);
        $object = $doc->getElementsByTagname('object')->item(0);
        $name = $object->getAttribute('name');
        $xml_properties = $doc->getElementsByTagname('property');
        $attributes = array('type', 'length', 'unsigned', 'notnull', 'default', 'autoincrement', 'fixed');
        foreach ($xml_properties as $index => $property)
        {
            $property_info = array();
            foreach ($attributes as $index => $attribute)
            {
                if ($property->hasAttribute($attribute))
                {
                    $property_info[$attribute] = $property->getAttribute($attribute);
                }
            }
            $properties[$property->getAttribute('name')] = $property_info;
        }
        $xml_indexes = $doc->getElementsByTagname('index');
        foreach ($xml_indexes as $key => $index)
        {
            $index_info = array();
            $index_info['type'] = $index->getAttribute('type');
            $index_properties = $index->getElementsByTagname('indexproperty');
            foreach ($index_properties as $subkey => $index_property)
            {
                $index_info['fields'][$index_property->getAttribute('name')] = array(
                    'length' => $index_property->getAttribute('length')
                );
            }
            $indexes[$index->getAttribute('name')] = $index_info;
        }
        $result = array();
        $result['name'] = $name;
        $result['properties'] = $properties;
        $result['indexes'] = $indexes;

        return $result;
    }

    /**
     * Returns the list with extra upgradable packages that are connected to this package
     *
     * @return multitype:string
     */
    public function get_additional_packages()
    {
        return array();
    }

    /**
     * Common functionality used in upgrade scripts
     */

    /**
     * Adds a registration for a new package
     *
     * @param $package
     *
     * @return bool
     */
    protected function add_package_registration($package)
    {
        $package_info = Package :: get($package);
        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent($package);
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context($package);

        if (! $application_registration->create())
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get(
                    'PackageRegistrationCreationFailed', array('PACKAGE' => $package), Utilities :: COMMON_LIBRARIES
                )
            );

            return false;
        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get(
                'PackageRegistrationCreated', array('PACKAGE' => $package), Utilities :: COMMON_LIBRARIES
            )
        );

        return true;
    }

    /**
     * Upgrades the registration for a given package
     *
     * @param string $package
     *
     * @return bool
     */
    protected function upgrade_package_registration($package)
    {
        $package_info = Package :: get($package);

        $registration = CommonDataManager :: get_registration($package);
        $registration->set_version($package_info->get_version());

        if (! $registration->update())
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES)
            );

            return false;
        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES)
        );

        return true;
    }

    /**
     * Creates a storage unit in the database
     *
     * @param string $storage_unit - The table name of the storage unit
     *
     * @return bool
     */
    protected function create_storage_unit($storage_unit)
    {
        $data_manager = $this->get_data_manager();

        $path = Path :: namespace_to_full_path($data_manager :: context()) . 'php/package/install/' .
            $storage_unit . '.xml';

        if(!file_exists($path))
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get('NoStorageUnitFound', array('UNIT' => $storage_unit), Utilities :: COMMON_LIBRARIES)
            );
            return false;
        }

        $storage_unit_info = self :: parse_xml_file($path);

        if (! $data_manager->storage_unit_exist($storage_unit_info['name']))
        {
            if (! $data_manager->create_storage_unit(
                $storage_unit_info['name'],
                $storage_unit_info['properties'],
                $storage_unit_info['indexes']
            )
            )
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                        'CreateStorageUnitFailed', array('UNIT' => $storage_unit_info['name']), 'core\lynx'
                    )
                );

                return false;
            }
            else
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get(
                        'CreateStorageUnitSuccessful', array('UNIT' => $storage_unit_info['name']), 'core\lynx'
                    )
                );
            }
        }

        return true;
    }
}
