<?php
namespace common\libraries;

/**
 *
 * @package common.libraries
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
abstract class Application
{

    /**
     *
     * @var \user\User
     */
    private $user;

    /**
     *
     * @var multitype:string
     */
    private $parameters;

    private $search_parameters;
    const PARAM_ACTION = 'go';
    const PARAM_MESSAGES = 'messages';
    const PARAM_MESSAGE = 'message';
    const PARAM_MESSAGE_TYPE = 'message_type';
    const PARAM_ERROR_MESSAGE = 'error_message';
    const PARAM_WARNING_MESSAGE = 'warning_message';
    const PARAM_APPLICATION = 'application';
    const PLACEHOLDER_APPLICATION = '__APPLICATION__';
    const RESULT_TYPE_CREATED = 'Created';
    const RESULT_TYPE_UPDATED = 'Updated';
    const RESULT_TYPE_DELETED = 'Deleted';
    const RESULT_TYPE_MOVED = 'Moved';

    /**
     *
     * @param \user\User $user
     */
    public function __construct($user)
    {
        $this->user = $user;
        $this->parameters = array();
        $this->search_parameters = array();
        $this->breadcrumbs = array();
    }

    /**
     * Gets the URL of the current page in the application. Optionally takes an associative array of name/value pairs
     * representing additional query string parameters; these will either be added to the parameters already present, or
     * override them if a value with the same name exists.
     *
     * @param multitype:string $parameters
     * @param multitype:string $filter
     * @param boolean $encode_entities Whether or not to encode HTML entities. Defaults to false.
     * @return string
     */
    public function get_url($parameters = array(), $filter = array(), $encode_entities = false)
    {
        $parameters = (count($parameters) ? array_merge($this->get_parameters(), $parameters) : $this->get_parameters());
        return Redirect :: get_url($parameters, $filter, $encode_entities);
    }

    /**
     * Redirect the end user to another location. The current url will be used as the basis.
     *
     * @param multitype:string $parameters
     * @param multitype:string $filter
     * @param boolean $encode_entities Whether or not to encode HTML entities. Defaults to false.
     * @param string $redirect_type
     */
    public function simple_redirect($parameters = array(), $filter = array(), $encode_entities = false,
        $redirect_type = Redirect :: TYPE_URL)
    {
        switch ($redirect_type)
        {
            case Redirect :: TYPE_URL :
                $parameters = (count($parameters) ? array_merge($this->get_parameters(), $parameters) : $this->get_parameters());
                Redirect :: url($parameters, $filter, $encode_entities);
                break;
            case Redirect :: TYPE_LINK :
                // Use this untill PHP 5.3 is available
                // Then use get_class($this) :: APPLICATION_NAME
                // and remove the get_application_name function();
                if ($parameters[self :: PARAM_APPLICATION])
                {
                    // if an application is given as a parameter this should not
                    // be overwritten
                    $application = $parameters[self :: PARAM_APPLICATION];
                }
                else
                {
                    $application = $this->get_application_name();
                }
                Redirect :: link($application, $parameters, $filter, $encode_entities);
                break;
        }
        exit();
    }

    /**
     * Redirect the end user to another location. The current url will be used as the basis. This method allows passing
     * on messages directly instead of using the parameters array
     *
     * @param string $message
     * @param boolean $error_message
     * @param multitype:string $parameters
     * @param multitype:string $filter
     * @param boolean $encode_entities Whether or not to encode HTML entities. Defaults to false.
     * @param string $redirect_type
     */
    public function redirect($message = '', $error_message = false, $parameters = array(), $filter = array(), $encode_entities = false,
        $redirect_type = Redirect :: TYPE_URL)
    {
        $message_type = (! $error_message) ? NotificationMessage :: TYPE_NORMAL : NotificationMessage :: TYPE_ERROR;

        $messages = Session :: retrieve(self :: PARAM_MESSAGES);
        $messages[self :: PARAM_MESSAGE_TYPE][] = $message_type;
        $messages[self :: PARAM_MESSAGE][] = $message;

        Session :: register(self :: PARAM_MESSAGES, $messages);

        $this->simple_redirect($parameters, $filter, $encode_entities, $redirect_type);
    }

    /**
     * Returns the current URL parameters.
     *
     * @return multitype:string
     */
    public function get_parameters()
    {
        return $this->parameters;
    }

    /**
     * Returns the value of the given URL parameter.
     *
     * @param string $name
     * @return string
     */
    public function get_parameter($name)
    {
        if (array_key_exists($name, $this->parameters))
            return $this->parameters[$name];
    }

    /**
     * Sets the value of a URL parameter.
     *
     * @param string $name
     * @param string $value
     */
    public function set_parameter($name, $value)
    {
        $this->parameters[$name] = $value;
    }

    /**
     *
     * @param multitype:string $parameters
     */
    public function set_parameters($parameters)
    {
        $this->parameters = $parameters;
    }

    /**
     * Displays the header
     */
    public function display_header()
    {
        $breadcrumbtrail = BreadcrumbTrail :: get_instance();
        if ($breadcrumbtrail->size() == 1)
        {
            $breadcrumbtrail->add(
                new Breadcrumb($this->get_url(), Translation :: get('TypeName', null, static :: context())));
        }

        $title = $breadcrumbtrail->get_last()->get_name();
        Display :: header();

        // If there is an application-wide menu, show it
        if ($this->has_menu())
        {
            echo '<div style="float: left; width: 17%;">';
            echo $this->get_menu();
            echo '</div>';
            echo '<div style="float: right; width: 82%;">';
        }

        echo '<h3 title="' . strip_tags($title) . '">' . $title . '</h3>';
        echo '<div class="clear">&nbsp;</div>';

        if (PlatformSetting :: get('maintenance_mode'))
        {
            Display :: error_message(Translation :: get('MaintenanceModeMessage'));
        }

        // Display messages
        $messages = Session :: retrieve(self :: PARAM_MESSAGES);
        Session :: unregister(self :: PARAM_MESSAGES);
        if (is_array($messages))
        {
            $this->display_messages($messages[self :: PARAM_MESSAGE], $messages[self :: PARAM_MESSAGE_TYPE]);
        }

        // DEPRECATED
        // Display messages
        $message = Request :: get(self :: PARAM_MESSAGE);
        $type = Request :: get(self :: PARAM_MESSAGE_TYPE);

        if ($message)
        {
            $this->display_message($message);
        }

        $message = Request :: get(self :: PARAM_ERROR_MESSAGE);
        if ($message)
        {
            $this->display_error_message($message);
        }

        $message = Request :: get(self :: PARAM_WARNING_MESSAGE);
        if ($message)
        {
            $this->display_warning_message($message);
        }
    }

    /**
     * Displays the footer
     */
    public function display_footer()
    {
        // In case there is an application-wide menu, properly end it
        if ($this->has_menu())
        {
            echo '<div class="clear">&nbsp;</div>';
            echo '</div>';
        }

        echo '<div class="clear">&nbsp;</div>';
        Display :: footer();
    }

    /**
     * Displays the small header
     */
    public function small_header()
    {
        Display :: small_header();

        // Display messages
        $messages = Session :: retrieve(self :: PARAM_MESSAGES);
        Session :: unregister(self :: PARAM_MESSAGES);
        if (is_array($messages))
        {
            $this->display_messages($messages[self :: PARAM_MESSAGE], $messages[self :: PARAM_MESSAGE_TYPE]);
        }
    }

    /**
     * Displays the small footer
     */
    public function small_footer()
    {
        Display :: small_footer();
    }

    /**
     * Displays a normal message.
     *
     * @param string $message
     */
    public function display_message($message)
    {
        echo '<div class="notifications">';
        echo NotificationMessage :: normal($message)->to_html();
        echo '</div>';
    }

    /**
     *
     * @param multitype:string $messages
     * @param multitype:int $types
     */
    public function display_messages($messages, $types)
    {
        echo '<div class="notifications">';
        foreach ($types as $key => $type)
        {
            echo NotificationMessage :: create($messages[$key], $type)->to_html();
        }
        echo '</div>';
    }

    /**
     * Displays an error message.
     *
     * @param string $message
     */
    public function display_error_message($message)
    {
        echo '<div class="notifications">';
        echo NotificationMessage :: error($message)->to_html();
        echo '</div>';
    }

    /**
     * Displays a warning message.
     *
     * @param string $message
     */
    public function display_warning_message($message)
    {
        echo '<div class="notifications">';
        echo NotificationMessage :: warning($message)->to_html();
        echo '</div>';
    }

    /**
     * Displays an error page.
     *
     * @param string $message
     */
    public function display_error_page($message)
    {
        $this->display_header();
        $this->display_error_message($message);
        $this->display_footer();
    }

    /**
     * Displays a warning page.
     *
     * @param string $message
     */
    public function display_warning_page($message)
    {
        $this->display_header();
        $this->display_warning_message($message);
        $this->display_footer();
    }

    /**
     * Wrapper for Display :: not_allowed();
     *
     * @param boolean $show_login_form
     */
    public function not_allowed($show_login_form = true)
    {
        Display :: not_allowed($show_login_form);
    }

    /**
     * Gets the user id of this personal calendars owner
     *
     * @return int
     */
    public function get_user_id()
    {
        if($this->user)
        {
            return $this->user->get_id();
        }

        return 0;
    }

    /**
     * Gets the user.
     *
     * @return \user\User
     */
    public function get_user()
    {
        return $this->user;
    }

    /**
     *
     * @return string
     */
    public function get_action()
    {
        return $this->get_parameter(self :: PARAM_ACTION);
    }

    /**
     * Sets the current action.
     *
     * @param string $action
     */
    public function set_action($action)
    {
        return $this->set_parameter(self :: PARAM_ACTION, $action);
    }

    public function get_application_name()
    {
        return array_pop(explode('\\', static :: context()));
    }

    /**
     * Returns a list of actions available to the admin.
     *
     * @param string $application
     * @return multitype:mixed
     */
    public static function get_application_platform_admin_links($application = self :: PARAM_APPLICATION)
    {
        $info = array();
        $info['application'] = array(
            'name' => Translation :: get('TypeName', null, Application :: determine_namespace($application)),
            'class' => $application);
        $info['links'] = array();
        $info['search'] = null;

        return $info;
    }

    /**
     * Returns a list of actions available to the admin.
     *
     * @return multitype:mixed
     */
    public static function get_application_platform_import_links()
    {
        return array();
    }

    /**
     * Does the entire application have a leftside menu? False per default. Can be overwritten by the specific
     * application
     *
     * @return boolean
     */
    public function has_menu()
    {
        return false;
    }

    /**
     * Returns the html for the application-menu. Empty per default Can be overwritten by the specific application
     *
     * @return string
     */
    public function get_menu()
    {
        return '';
    }

    /**
     * Determines if a given name is the name of an application
     *
     * @param string $string
     * @return boolean
     */
    public static function is_application_name($name)
    {
        return (preg_match('/^[a-z][a-z_]+$/', $name) > 0);
    }

    /**
     * Create a new application component
     *
     * @param string $type
     * @param \common\libraries\Application $manager
     * @deprecated Old code
     */
    public function create_component($type, $application = null)
    {
        if ($application == null)
        {
            $application = $this;
        }

        $manager_class = get_class($application);

        $class = $manager_class . $type . 'Component';

        if (! class_exists($class))
        {
            $message = array();
            $message[] = Translation :: get('ComponentFailedToLoad') . '<br /><br />';
            $message[] = '<b>' . Translation :: get('File') . ':</b><br />';
            $message[] = $class . '<br /><br />';
            $message[] = '<b>' . Translation :: get('Stacktrace') . ':</b>';
            $message[] = '<ul>';
            $message[] = '<li>' . Translation :: get($manager_class) . '</li>';
            $message[] = '<li>' . Translation :: get($type) . '</li>';
            $message[] = '</ul>';

            Display :: header();
            Display :: error_message(implode("\n", $message));
            Display :: footer();
            exit();
        }

        if (is_subclass_of($application, 'common\libraries\SubManager'))
        {
            $component = new $class($application->get_parent());
        }
        else
        {
            $component = new $class($this->get_user());
            $component->set_parameters($this->get_parameters());
        }
        return $component;
    }

    /**
     *
     * @param int $failures
     * @param int $count
     * @param string $fail_message_single
     * @param string $fail_message_multiple
     * @param string $succes_message_single
     * @param string $succes_message_multiple
     * @return string
     */
    public function get_result($failures, $count, $fail_message_single, $fail_message_multiple, $succes_message_single,
        $succes_message_multiple)
    {
        if ($failures)
        {
            if ($count == 1)
            {
                $message = $fail_message_single;
            }
            else
            {
                $message = $fail_message_multiple;
            }
        }
        else
        {
            if ($count == 1)
            {
                $message = $succes_message_single;
            }
            else
            {
                $message = $succes_message_multiple;
            }
        }

        return Translation :: get($message);
    }

    /**
     * Generates a general results message like ObjectCreated, ObjectUpdated, ObjectDeleted
     *
     * @param int $failures
     * @param int $count
     * @param string $single_object
     * @param string $multiple_object
     * @param string $type
     * @return string
     */
    public function get_general_result($failures, $count, $single_object, $multiple_object,
        $type = Application :: RESULT_TYPE_CREATED)
    {
        if ($count == 1)
        {
            $param = array('OBJECT' => $single_object);

            if ($failures)
            {
                $message = 'ObjectNot' . $type;
            }
            else
            {
                $message = 'Object' . $type;
            }
        }
        else
        {
            $param = array('OBJECTS' => $multiple_object);

            if ($failures)
            {
                $message = 'ObjectsNot' . $type;
            }
            else
            {
                $message = 'Objects' . $type;
            }
        }

        return Translation :: get($message, $param);
    }

    /**
     *
     * @param string $application
     * @return string
     */
    public static function get_application_class_name($application)
    {
        if (class_exists('application\\' . $application . '\Manager'))
        {
            return 'Manager';
        }
        else
        {
            return Utilities :: underscores_to_camelcase($application) . 'Manager';
        }
    }

    /**
     *
     * @return string
     */
    public function process_table_action()
    {
        $table_name = Request :: post('table_name');
        if (isset($table_name))
        {
            $namespace = Request :: post($table_name . '_namespace');
            $class = Utilities :: underscores_to_camelcase($table_name);
            $classname = $namespace . '\\' . $class;

            if (class_exists($classname))
            {
                call_user_func(array($classname, 'handle_table_action'));

                $table_action_name = Request :: post($table_name . '_action_name');
                $table_action_value = Request :: post($table_name . '_action_value');
                Request :: set_get($table_action_name, $table_action_value);

                if ($table_action_name == self :: PARAM_ACTION)
                {
                    return $table_action_value;
                }
            }
        }

        return null;
    }

    /**
     *
     * @param string $application_name
     * @param string $type
     * @return string
     */
    private static function load_class($application_name, $type)
    {
        $component_class = self :: determine_namespace($application_name) . '\\' .
             Utilities :: underscores_to_camelcase($type) . 'Component';
        if (! class_exists($component_class))
        {
            $manager_class = self :: get_application_class_name($application_name);
            $component_class = self :: determine_namespace($application_name) . '\\' . $manager_class .
                 Utilities :: underscores_to_camelcase($type) . 'Component';

            if (! class_exists($component_class))
            {
                $message = array();
                $message[] = Translation :: get('ComponentFailedToLoad') . '<br /><br />';
                $message[] = '<b>' . Translation :: get('Stacktrace') . ':</b>';
                $message[] = '<ul>';
                $message[] = '<li>' . Translation :: get($manager_class) . '</li>';
                $message[] = '<li>' . Translation :: get($type) . '</li>';
                $message[] = '</ul>';

                $trail = BreadcrumbTrail :: get_instance();
                $trail->add(new Breadcrumb('#', Translation :: get($manager_class)));

                Display :: header($trail);
                Display :: error_message(implode("\n", $message));
                Display :: footer();
                exit();
            }
        }
        return $component_class;
    }

    /**
     *
     * @param string $application_name
     */
    public static function determine_namespace($application_name)
    {
        $application_type = self :: get_application_type($application_name);
        return $application_type :: get_application_namespace($application_name);
    }

    /**
     *
     * @param string $application_name
     * @param \user\User $user
     * @param string $type
     * @return \common\libraries\Application
     */
    public static function component($application_name, $user, $type)
    {
        $class = self :: load_class($application_name, $type);
        return new $class($user);
    }

    /**
     *
     * @param string $application_name
     * @return string
     */
    public static function get_component_action($application_name)
    {
        $manager_class = self :: determine_namespace($application_name) . '\Manager';
        if (! class_exists($manager_class))
        {
            $manager_class = self :: determine_namespace($application_name) . '\\' .
                 self :: get_application_class_name($application_name);
        }
        $default_action = call_user_func(array($manager_class, 'get_default_action'));

        $action = Request :: get(self :: PARAM_ACTION);
        $action = ! isset($action) ? $default_action : $action;

        $table_action = self :: process_table_action();
        if ($table_action)
        {
            $action = $table_action;
        }

        return $action;
    }

    /**
     *
     * @param string $application_name
     * @param \user\User $user
     * @return \common\libraries\Application
     */
    public static function construct($application_name, $user)
    {
        $type = self :: get_application_type($application_name);
        $action = self :: get_component_action($application_name);
        $component = self :: component($application_name, $user, $action);
        $component->set_parameter(self :: PARAM_APPLICATION, $application_name);

        $context = Utilities :: get_namespace_from_object($component);

        if (! $component instanceof NoContextComponent)
        {
            $trail = BreadcrumbTrail :: get_instance();
            $trail->add(new Breadcrumb($component->get_url(), Translation :: get('TypeName', null, $context)));
            $component->add_additional_breadcrumbs($trail);
        }

        $parameters = $component->get_additional_parameters();
        foreach ($parameters as $parameter)
        {
            $component->set_parameter($parameter, Request :: get($parameter));
        }

        $component->set_action($action);

        if (! $component instanceof DelegateComponent && ! $component instanceof NoContextComponent)
        {
            $trail->add_help($context, Utilities :: get_classname_from_object($component, true));
            $trail->add(
                new Breadcrumb(
                    $component->get_url(array(self :: PARAM_ACTION => $action)),
                    Translation :: get(Utilities :: get_classname_from_namespace(get_class($component)), null, $context)));
        }
        return $component;
    }

    /**
     *
     * @param string $application_name
     * @param \user\User $user
     */
    public static function launch($application_name, $user)
    {
        self :: construct($application_name, $user)->run();
    }

    /**
     *
     * @param \common\libraries\BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
    }

    /**
     *
     * @return multitype:string
     */
    public function get_additional_parameters()
    {
        return array();
    }

    /**
     *
     * @param string $application
     * @return string
     */
    private static function get_application_type($application)
    {
        if (WebApplication :: exists($application))
        {
            return WebApplication :: CLASS_NAME;
        }

        if (CoreApplication :: exists($application))
        {
            return CoreApplication :: CLASS_NAME;
        }

        return false;
    }

    /**
     *
     * @param string $context
     * @return boolean
     */
    public static function is_active($context = null)
    {
        if (self :: exists($context))
        {
            $registration = CommonDataManager :: get_registration($context);

            if ($registration && $registration->is_active())
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    /**
     * Get the fully qualified class name of the Application object
     *
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the Application object
     *
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    /**
     *
     * @param multitype:string $parameters
     * @param multitype:string $filter
     * @param boolean $encode_entities
     * @return string
     */
    public function get_link($parameters = array (), $filter = array(), $encode_entities = false)
    {
        // Use this untill PHP 5.3 is available
        // Then use get_class($this) :: APPLICATION_NAME
        // and remove the get_application_name function();
        $application = $this->get_application_name();
        return Redirect :: get_link($application, $parameters, $filter, $encode_entities);
    }

    /**
     *
     * @param string $application
     * @return boolean
     */
    public static function exists($application)
    {
        if (WebApplication :: exists($application))
        {
            return true;
        }

        if (CoreApplication :: exists($application))
        {
            return true;
        }

        return false;
    }
}
