<?php
namespace common\libraries;

use user\User;

use repository\RepositoryDataManager;
use repository\ContentObject;

use home\HomeManager;
use home\HomeRenderer;
use home\HomeBlock;

use common\libraries\StringUtilities;

use Exception;

/**
 * @package common.libraries
 */
class Block
{
    const PARAM_ACTION = 'block_action';
    const BLOCK_LIST_SIMPLE = 'simple';
    const BLOCK_LIST_ADVANCED = 'advanced';

    // display the block view for integration into chamil's home page
    const BLOCK_VIEW = 'block_view';
    // display the widget view for integration into a third party application such as a portal
    const WIDGET_VIEW = 'widget_view';

    const BLOCK_PROPERTY_ID = 'id';
    const BLOCK_PROPERTY_NAME = 'name';
    const BLOCK_PROPERTY_IMAGE = 'image';

    private $parent;
    private $type;
    private $block_info;
    private $configuration;
    private $view = self :: BLOCK_VIEW;

    /**
     * @param HomeRenderer $renderer
     * @param HomeBlock $block_info
     * @return Block
     */
    static public function factory(HomeRenderer $renderer, HomeBlock $block_info, $configuration = null)
    {
        $type = $block_info->get_component();
        $context = $block_info->get_context();

        $path = Path :: get(SYS_PATH) . Path :: namespace_to_path($context) . '/php/blocks/type/' . $type . '.class.php';
        if (! file_exists($path) || ! is_file($path))
        {
            die('Failed to load "' . $type . '" block');
        }
        $class = $context . '\\' . Utilities :: get_package_name_from_namespace($context, true) . Utilities :: underscores_to_camelcase($type);

        return new $class($renderer, $block_info, $configuration);
    }

    /**
     * Returns the default image to be displayed for block's creation. Can be redefined in subclasses to change the default icon.
     *
     * @todo: not so good. would be better to make the whole "create block" a view.
     *
     * @param string $application
     * @param string $type
     * @param string $size
     * @return string
     */
    public static function get_image_path($context, $type)
    {
        $class = $context . '\\' . Utilities :: underscores_to_camelcase(Utilities :: get_package_name_from_namespace($context) . '_' . $type);

        if (method_exists($class, 'get_default_image_path'))
        {
            return $class :: get_default_image_path();
        }
        else
        {
            $image_path = Theme :: get_image_system_path($context) . 'blocks/' . $type . '.png';

            if (! file_exists($image_path) || ! is_file($image_path))
            {
                return Theme :: get_image_path($context) . 'logo/' . Theme :: ICON_MEDIUM . '.png';
            }
            else
            {
                return Theme :: get_image_path($context) . 'blocks/' . $type . '.png';
            }
        }
    }

    function __construct($parent, $block_info, $configuration = null)
    {
        $this->parent = $parent;
        $this->block_info = $block_info;
        $this->configuration = $configuration ? $configuration : $block_info->get_configuration();
    }

    /**
     * The type of view: block or widget. Block - default - is for integration into Chamilo's homepage. Widget is for integration into a third party application - i.e. external portal.
     * @return string
     */
    function get_view()
    {
        return $this->view;
    }

    function set_view($view)
    {
        $this->view = $view;
    }

    /**
     * Returns the tool which created this publisher.
     * @return Tool The tool.
     */
    function get_parent()
    {
        return $this->parent;
    }

    function get_configuration()
    {
        return $this->configuration;
    }

    /**
     * @see Tool::get_user_id()
     */
    function get_user_id()
    {
        return $this->get_parent()->get_user_id();
    }

    function get_user()
    {
        return $this->get_parent()->get_user();
    }

    /**
     * Returns the types of learning object that this object may publish.
     * @return array The types.
     */
    function get_type()
    {
        return $this->type;
    }

    function get_block_info()
    {
        return $this->block_info;
    }

    function is_editable()
    {
        return true;
    }

    function is_hidable()
    {
        return true;
    }

    function is_deletable()
    {
        return true;
    }

    /**
     * Returns true if the block is to be displayed, false otherwise. By default do not show on home page when user is not connected.
     *
     * @return bool
     */
    function is_visible()
    {
        return Session :: get_user_id() != 0;
    }

    /**
     * Returns the block's title to display.
     *
     * @return string
     */
    function get_title()
    {
        return $this->get_block_info()->get_title();
    }

    /**
     * Link target for external links. I.e. links that do not modify the widget itself. In widget mode they should point to a new windows.
     */
    public function get_link_target()
    {
        return Request :: get(HomeRenderer :: PARAM_WIDGET_ID, false) ? '_blank' : '';
    }

    /**
     * Returns the url to the icon.
     *
     * @return string
     */
    function get_icon()
    {
        return Theme :: get_image_path($this->get_block_info()->get_context()) . 'logo/' . Theme :: ICON_MEDIUM . '.png';
    }

    function as_html($view = '')
    {
        if (! $this->is_visible())
        {
            return '';
        }
        if ($view)
        {
            $this->set_view($view);
        }

        $config = $this->get_configuration();

        $html = array();
        $html[] = $this->display_header();
        $html[] = $this->display_content();
        $html[] = $this->display_footer();

        return implode(StringUtilities :: NEW_LINE, $html);
    }

    function display_header()
    {

        $block_id = $this->get_block_info()->get_id();
        $icon_url = $this->get_icon();
        $widget_view_url = htmlspecialchars($this->get_block_widget_view_link($this->block_info));

        $title = $this->display_title();
        if ($this->get_view() == self :: BLOCK_VIEW)
        { //i.e. in widget view it is the portal configuration that decides to show/hide
            $description_style = $this->get_block_info()->is_visible() ? '' : ' style="display: none"';
        }
        else
        {
            $description_style = '';
        }

        $html = array();
        $html[] = '<div class="portal_block" id="portal_block_' . $block_id . '" style="background-image: url(' . $icon_url . ');">';
        $html[] = $title;
        $html[] = '<div class="entry-content description"' . $description_style . '>';

        return implode("\n", $html);
    }

    function display_title()
    {

        $title = htmlspecialchars($this->get_title());
        $actions = $this->display_actions();

        $html = array();
        $html[] = '<div class="title"><div style="float: left;" class="entry-title">' . $title . '</div>';
        $html[] = $actions;
        $html[] = '<div style="clear: both;"></div>';
        $html[] = '</div>';

        return implode("\n", $html);
    }

    function display_actions()
    {
        if ($this->get_view() != self :: BLOCK_VIEW)
        {
            return '';
        }

        $html = array();

        $user_home_allowed = PlatformSetting :: get('allow_user_home', HomeManager :: APPLICATION_NAME);

        $img_path = htmlspecialchars(Theme :: get_common_image_path());

        if ($this->get_user() instanceof User && ($user_home_allowed || $this->get_user()->is_platform_admin()) && ! $this->get_user()->is_anonymous_user())
        {
            if ($this->is_deletable())
            {
                $delete_text = Translation :: get('Delete');
                $html[] = '<a href="' . htmlspecialchars($this->get_block_deleting_link($this->get_block_info())) . '" class="deleteEl" title="' . $delete_text . '">';
                $html[] = '<img src="' . $img_path . 'action_delete.png" alt="' . $delete_text . '" title="' . $delete_text . '"/></a>';
            }

            if ($this->block_info->is_configurable())
            {
                $configure_text = Translation :: get('Configure');
                $html[] = '<a href="' . htmlspecialchars($this->get_block_configuring_link($this->get_block_info())) . '" class="configEl" title="' . $configure_text . '">';
                $html[] = '<img src="' . $img_path . 'action_config.png" alt="' . $configure_text . '" title="' . $configure_text . '"/></a>';
            }

            if ($this->is_editable())
            {
                $edit_text = Translation :: get('Edit');
                $html[] = '<a href="' . htmlspecialchars($this->get_block_editing_link($this->get_block_info())) . '" class="editEl" title="' . $edit_text . '">';
                $html[] = '<img src="' . $img_path . 'action_edit.png" alt="' . $edit_text . '" title="' . $edit_text . '"/></a>';
            }

            if ($this->is_hidable())
            {
                $toggle_visibility_text = Translation :: get('ToggleVisibility');
                $html[] = '<a href="' . htmlspecialchars($this->get_block_visibility_link($this->get_block_info())) . '" class="closeEl" title="' . $toggle_visibility_text . '">';
                $html[] = '<img class="visible"' . ($this->get_block_info()->is_visible() ? '' : ' style="display: none;"') . ' src="' . $img_path . 'action_visible.png" alt="' . $toggle_visibility_text . '" title="' . $toggle_visibility_text . '"/>';
                $html[] = '<img class="invisible"' . ($this->get_block_info()->is_visible() ? ' style="display: none;"' : '') . ' src="' . $img_path . 'action_invisible.png" alt="' . $toggle_visibility_text . '" title="' . $toggle_visibility_text . '"/></a>';
            }

            $drag_text = Translation :: get('Drag');
            $html[] = '<a href="#" id="drag_block_' . $this->get_block_info()->get_id() . '" class="dragEl" title="' . $drag_text . '">';
            $html[] = '<img src="' . $img_path . 'action_drag.png" alt="' . $drag_text . '" title="' . $drag_text . '"/></a>';
        }

        return implode("\n", $html);
    }

    function display_content()
    {
        return '';
    }

    function display_footer()
    {
        $html = array();

        $html[] = '<div style="clear: both;"></div>';
        $html[] = '</div>';
        $html[] = '</div>';

        return implode("\n", $html);
    }

    /**
     * Returns the URL used to view a block in "WIDGET" mode. I.e. for integration into an external application - web portal, IE webslice, etc.
     *
     * @param HomeBlock $home_block
     * @return string
     */
    function get_block_widget_view_link($home_block)
    {
        $result = Redirect :: get_link('', array(HomeRenderer :: PARAM_VIEW_TYPE => HomeRenderer :: TYPE_WIDGET,
                HomeRenderer :: PARAM_WIDGET_ID => $home_block->get_id()), array(), false, Redirect :: TYPE_INDEX);
        $result = Path :: get(WEB_PATH) . $result;
        return $result;
    }

    function get_block_visibility_link($home_block)
    {
        return $this->get_manipulation_link(array(HomeManager :: PARAM_ACTION => HomeManager :: ACTION_EDIT_HOME,
                HomeManager :: PARAM_HOME_TYPE => HomeManager :: TYPE_BLOCK,
                HomeManager :: PARAM_HOME_ID => $home_block->get_id()));
    }

    function get_block_deleting_link($home_block)
    {
        return '#';
    }

    function get_block_editing_link($home_block)
    {
        return $this->get_manipulation_link(array(
                HomeManager :: PARAM_ACTION => HomeManager :: ACTION_EDIT_HOME_PERSONAL,
                HomeManager :: PARAM_HOME_TYPE => HomeManager :: TYPE_BLOCK,
                HomeManager :: PARAM_HOME_ID => $home_block->get_id()));
    }

    function get_block_configuring_link($home_block)
    {
        return $this->get_manipulation_link(array(
                HomeManager :: PARAM_ACTION => HomeManager :: ACTION_CONFIGURE_HOME_PERSONAL,
                HomeManager :: PARAM_HOME_TYPE => HomeManager :: TYPE_BLOCK,
                HomeManager :: PARAM_HOME_ID => $home_block->get_id()));
    }

    public function get_manipulation_link($parameters)
    {
        return Redirect :: get_link(HomeManager :: APPLICATION_NAME, $parameters, array(), false, Redirect :: TYPE_CORE);
    }

    public function get_link($application, $parameters = array(), $encode = false, $type = Redirect :: TYPE_CORE)
    {
        return Redirect :: get_link($application, $parameters, array(), $encode, $type);
    }

    /**
     * @deprecated
     * @param string $type
     */
    function get_platform_blocks($type = self :: BLOCK_LIST_SIMPLE)
    {
        $blocks = array();

        // Get blocks for core applications
        $core_applications = CoreApplication :: get_list();

        foreach ($core_applications as $core_application)
        {
            $application_path = CoreApplication :: get_application_class_path($core_application) . 'blocks/type/';

            if (is_dir($application_path))
            {
                $files = Filesystem :: get_directory_content($application_path, Filesystem :: LIST_FILES, false);

                foreach ($files as $file)
                {
                    if (stripos($file, '.class.php') !== false)
                    {
                        $component = str_replace('.class.php', '', $file);
                        $context = CoreApplication :: determine_namespace($core_application);

                        $blocks[$context]['name'] = Translation :: get('TypeName', null, $context);
                        $blocks[$context]['image'] = Theme :: get_image_path($context) . 'logo/16.png';
                        $blocks[$context]['components'][] = array(self :: BLOCK_PROPERTY_ID => $component,
                                self :: BLOCK_PROPERTY_NAME => Translation :: get(Utilities :: underscores_to_camelcase($component), null, $context),
                                self :: BLOCK_PROPERTY_IMAGE => self :: get_image_path($context, $component));
                    }
                }
            }
        }

        // Get blocks for optional applications
        $web_applications = WebApplication :: load_all(false);

        foreach ($web_applications as $web_application)
        {
            $application_path = WebApplication :: get_application_class_path($web_application) . 'blocks/type/';

            if (is_dir($application_path))
            {
                $files = Filesystem :: get_directory_content($application_path, Filesystem :: LIST_FILES, false);

                foreach ($files as $file)
                {
                    if (stripos($file, '.class.php') !== false)
                    {
                        $component = str_replace('.class.php', '', $file);
                        $context = WebApplication :: determine_namespace($web_application);

                        $blocks[$context]['name'] = Translation :: get('TypeName', null, $context);
                        $blocks[$context]['image'] = Theme :: get_image_path($context) . 'logo/16.png';
                        $blocks[$context]['components'][] = array(self :: BLOCK_PROPERTY_ID => $component,
                                self :: BLOCK_PROPERTY_NAME => Translation :: get(Utilities :: underscores_to_camelcase($component), null, $context),
                                self :: BLOCK_PROPERTY_IMAGE => self :: get_image_path($context, $component));
                    }
                }
            }
        }

        // Get blocks for content objects
        $content_objects = RepositoryDataManager :: get_registered_types();

        foreach ($content_objects as $content_object)
        {
            $content_object_path = Path :: get_repository_content_object_path() . $content_object . '/php/blocks/type/';

            if (is_dir($content_object_path))
            {
                $files = Filesystem :: get_directory_content($content_object_path, Filesystem :: LIST_FILES, false);

                foreach ($files as $file)
                {
                    if (stripos($file, '.class.php') !== false)
                    {
                        $component = str_replace('.class.php', '', $file);
                        $context = ContentObject :: get_content_object_type_namespace($content_object);

                        $blocks[$context]['name'] = Translation :: get('TypeName', null, $context);
                        $blocks[$context]['image'] = Theme :: get_image_path($context) . 'logo/16.png';
                        $blocks[$context]['components'][] = array(self :: BLOCK_PROPERTY_ID => $component,
                                self :: BLOCK_PROPERTY_NAME => Translation :: get(Utilities :: underscores_to_camelcase($component), null, $context),
                                self :: BLOCK_PROPERTY_IMAGE => self :: get_image_path($context, $component));
                    }
                }
            }
        }

        return $blocks;
    }

    /**
     * We keep this since Quickform's hierselect element
     * only works if javascript is enabled
     * @deprecated
     */
    function get_platform_blocks_deprecated()
    {
        $application_components = array();
        $applications = WebApplication :: load_all(false);

        $path = Path :: get_application_path() . '/lib/';

        foreach ($applications as $application)
        {
            $application_path = WebApplication :: get_application_class_path($application) . 'blocks/type/';
            if ($handle = opendir($application_path))
            {
                while (false !== ($file = readdir($handle)))
                {
                    if (! is_dir($file) && stripos($file, '.class.php') !== false)
                    {
                        $component = str_replace('.class.php', '', $file);
                        $component = str_replace($application . '_', '', $component);
                        $value = $application . '.' . $component;
                        $display = Translation :: get(Application :: application_to_class($application)) . '&nbsp;>&nbsp;' . Utilities :: underscores_to_camelcase($component);
                        $application_components[$value] = $display;
                    }
                }
                closedir($handle);
            }
        }

        $core_applications = array('admin', 'tracking', 'repository', 'user', 'group', 'rights', 'home', 'menu');

        $path = Path :: get(SYS_PATH);

        foreach ($core_applications as $core_application)
        {
            $application_path = CoreApplication :: get_application_class_path($core_application) . 'blocks/type/';
            if ($handle = opendir($application_path))
            {
                while (false !== ($file = readdir($handle)))
                {
                    if (! is_dir($file) && stripos($file, '.class.php') !== false)
                    {
                        $component = str_replace('.class.php', '', $file);
                        $component = str_replace($core_application . '_', '', $component);
                        $value = $core_application . '.' . $component;
                        $display = Translation :: get(Application :: application_to_class($core_application)) . '&nbsp;>&nbsp;' . Utilities :: underscores_to_camelcase($component);
                        $application_components[$value] = $display;
                    }
                }
                closedir($handle);
            }
        }

        asort($application_components);

        return $application_components;
    }

    function get_url($parameters = array(), $filter = array(), $encode_entities = false)
    {
        if ($widget_id = Request :: get(HomeRenderer :: PARAM_WIDGET_ID))
        {
            $parameters[HomeRenderer :: PARAM_WIDGET_ID] = $widget_id;
        }
        $result = $this->get_parent()->get_url($parameters, $filter, $encode_entities);
        return $result;
    }

    function get_parameter($name)
    {
        return $this->get_parent()->get_parameter($name);
    }

    function get_parameters()
    {
        $result = $this->get_parent()->get_parameters();
        if ($widget_id = Request :: get(HomeRenderer :: PARAM_WIDGET_ID))
        {
            $result[HomeRenderer :: PARAM_WIDGET_ID] = $widget_id;
        }

        return $result;
    }

    /**
     * Default response for blocks who use an attachment viewer. Override for different functionality.
     * @param ContentObject $object The content object to be tested.
     * @return boolean default response: false.
     */
    function is_view_attachment_allowed($object)
    {
        return false;
    }
}

?>