<?php
namespace common\libraries;

class NewGalleryObjectTable
{
    /**
     * Application or submanager component calling the ObjectTable
     *
     * @var Ambigous <Application, SubManager>
     */
    private $component;

    /**
     * Suffix for checkbox name when using actions on selected learning objects.
     */
    const CHECKBOX_NAME_SUFFIX = '_id';
    /**
     * The column model assigned to this table
     */
    private $column_model;
    /**
     * The data provider assigned to this table
     */
    private $data_provider;
    /**
     * The cell renderer assigned to this table
     */
    private $cell_renderer;

    /**
     * Caching of form actions
     *
     * @var ObjectTableFormActions
     */
    private $form_actions;

    public function __construct($component)
    {
        if (! $component instanceof NewGalleryObjectTableSupport)
        {
            throw new \Exception(Utilities :: get_classname_from_object($component) . " doesn't seem to support object tables, please implement the NewGalleryObjectTableSupport interface");
        }
        $this->component = $component;
    }

    public function get_class($type = null)
    {
        $class_name = get_class($this);

        if (! is_null($type))
        {
            $class_name .= $type;
        }

        return $class_name;
    }

    /**
     * Creates an HTML representation of the table.
     *
     * @return string The HTML.
     */
    public function as_html()
    {
        $count = $this->get_default_column_count() * $this->get_default_row_count();
        $table = new GalleryHTMLTable($this->get_name(), array($this, 'get_object_count'), array($this, 'get_objects'), array(
                $this,
                'get_property_model'), $count, 0, SORT_ASC, ! $this->prohibits_order_direction(), $this->supports_ajax(), $this->prohibits_page_selection());

        $table->set_additional_parameters($this->get_parameters());
        if ($this->has_form_actions())
        {
            $table->set_form_actions($this->get_form_actions(), $this->get_checkbox_name());
        }

        return $table->as_html();
    }

    /**
     *
     * @return boolean
     */
    public function prohibits_page_selection()
    {
        return $this instanceof NewObjectTablePageSelectionProhibition;
    }

    /**
     *
     * @return boolean
     */
    public function supports_ajax()
    {
        return $this instanceof NewObjectTableAjaxSupport;
    }

    /**
     * Gets the name of the HTML table element
     *
     * @return string The name
     */
    public static function get_name()
    {
        return Utilities :: get_classname_from_namespace(get_called_class(), true);
    }

    /**
     * Gets the default row count of the table.
     *
     * @return int The number of rows.
     */
    public function get_default_row_count()
    {
        return static :: DEFAULT_ROW_COUNT;
    }

    /**
     * Gets the default column count of the table.
     *
     * @return int The number of columns.
     */
    public function get_default_column_count()
    {
        return static :: DEFAULT_COLUMN_COUNT;
    }

    /**
     *
     * @return boolean
     */
    public function prohibits_order_direction()
    {
        return $this instanceof NewGalleryObjectTableOrderDirectionProhibition;
    }

    /**
     * Gets the table's data provider
     *
     * @return NewObjectTableDataProvider The data provider
     */
    public function get_data_provider()
    {
        if (! isset($this->data_provider))
        {
            $classname = $this->get_class('DataProvider');
            $this->data_provider = new $classname($this);
        }
        return $this->data_provider;
    }

    /**
     * Gets the table's properties.
     *
     * @return Array The properties.
     */
    public function get_property_model()
    {
        if (! isset($this->property_model))
        {
            $classname = $this->get_class('PropertyModel');
            $this->property_model = new $classname($this);
        }
        return $this->property_model;
    }

    /**
     * Gets the table's cell renderer
     *
     * @return NewObjectTableCellRenderer The cell renderer
     */
    public function get_cell_renderer()
    {
        if (! isset($this->cell_renderer))
        {
            $classname = $this->get_class('CellRenderer');
            $this->cell_renderer = new $classname($this);
        }
        return $this->cell_renderer;
    }

    /**
     * Gets the actions for the mass-update form at the bottom of the table.
     *
     * @return ObjectTableFormActions The actions as an associative array.
     */
    public function get_form_actions()
    {
        if (! isset($this->form_actions))
        {
            $this->form_actions = $this->get_implemented_form_actions();
        }

        return $this->form_actions;
    }

    /**
     * Returns whether or not the table has form actions
     *
     * @return boolean
     */
    public function has_form_actions()
    {
        return ($this instanceof NewObjectTableFormActionsSupport && $this->get_form_actions()->has_form_actions());
    }

    /**
     *
     * @return string
     */
    public function get_checkbox_name()
    {
        return $this->get_name() . static :: CHECKBOX_NAME_SUFFIX;
    }

    public function get_objects($offset, $count, $order_property, $order_direction)
    {
        $property_model = $this->get_property_model();
        $objects = $this->get_data_provider()->get_objects($offset, $count, $property_model->get_order_property($order_property - ($this->has_form_actions() ? 1 : 0), $order_direction));

        $column_count = $this->get_default_column_count();
        $row_count = $this->get_default_row_count();

        $table_data = array();

        $row = array();
        // $actions_row = array();

        $i = 0;
        while ($objects != null && $object = $objects->next_result())
        {
            if ($i >= $column_count)
            {
                $table_data[] = $row;
                $row = array();
                $row[] = array($object->get_id(), $this->get_cell_renderer()->render_cell($object));

                // $table_data[] = $actions_row;
                // $actions_row = array();
                // $actions_row[] = array($object->get_id(), $this->get_cell_renderer()->get_object_actions($object));
                $i = 1;
            }
            else
            {
                $row[] = array($object->get_id(), $this->get_cell_renderer()->render_cell($object));
                // $actions_row[] = array($object->get_id(), $this->get_cell_renderer()->get_object_actions($object));
                $i ++;
            }
        }
        $table_data[] = $row;
        // $table_data[] = $actions_row;

        return $table_data;
    }

    public function get_object_count()
    {
        return $this->get_data_provider()->get_object_count();
    }

    /**
     *
     * @param $table_name unknown_type
     * @return multitype:int
     */
    public static function get_selected_ids()
    {
        $selected_ids = Request :: post(static :: get_name() . self :: CHECKBOX_NAME_SUFFIX);

        if (empty($selected_ids))
        {
            $selected_ids = array();
        }
        elseif (! is_array($selected_ids))
        {
            $selected_ids = array($selected_ids);
        }

        return $selected_ids;
    }

    public static function handle_table_action()
    {
        $ids = static :: get_selected_ids();
        Request :: set_get(static :: TABLE_IDENTIFIER, $ids);
    }

    /**
     *
     * @return \common\libraries\Application
     */
    public function get_component()
    {
        return $this->component;
    }

    public function get_condition()
    {
        return $this->get_component()->get_object_table_condition(get_called_class());
    }

    public function get_parameters()
    {
        return $this->get_component()->get_parameters();
    }

    /**
     * Get the fully qualified class name of the GalleryObjectTable object
     *
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the GalleryObjectTable object
     *
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }
}
