<?php
namespace common\libraries;

/**
 * This class describes a table column model
 *
 * @package \common\libraries;
 * @author Sven Vanpoucke
 */
abstract class NewObjectTableColumnModel
{
    /**
     * **************************************************************************************************************
     * Default constants *
     * **************************************************************************************************************
     */
    const DEFAULT_ORDER_COLUMN_INDEX = 0;
    const DEFAULT_ORDER_COLUMN_DIRECTION = SORT_ASC;

    /**
     * **************************************************************************************************************
     * Properties *
     * **************************************************************************************************************
     */

    /**
     * The object table
     *
     * @var NewObjectTable
     */
    private $object_table;

    /**
     * The columns in the table.
     *
     * @var TableColumn[]
     */
    private $columns;

    /**
     * The column by which the table is currently sorted.
     *
     * @var int
     */
    private $order_column;

    /**
     * The direction in which the table is currently sorted.
     *
     * @var int
     */
    private $order_direction;

    /**
     * **************************************************************************************************************
     * Main Functionality *
     * **************************************************************************************************************
     */

    /**
     * Constructor.
     */
    public function __construct($object_table)
    {
        $this->object_table = $object_table;

        $this->initialize_columns();

        if ($this instanceof NewObjectTableColumnModelActionsColumnSupport)
        {
            $this->add_column(new NewObjectTableActionsColumn());
        }

        $this->set_default_order_column(static :: DEFAULT_ORDER_COLUMN_INDEX);
        $this->set_default_order_direction(static :: DEFAULT_ORDER_COLUMN_DIRECTION);
    }

    /**
     *
     * @return \common\libraries\NewObjectTable
     */
    public function get_object_table()
    {
        return $this->object_table;
    }

    /**
     * Returns the component of the object table
     *
     * @param
     *            Application
     */
    public function get_component()
    {
        return $this->get_object_table()->get_component();
    }

    /**
     * Returns an ObjectTableOrder with the given order column index.
     * If the selected index is not sortable, the default column index will be
     * used.
     * If the default column is not sortable then no order will be given
     *
     * @param $column_number int
     * @param $order_direction int
     *
     * @return \common\libraries\ObjectTableOrder null
     */
    public function get_order_column($column_number, $order_direction)
    {
        $column = $this->get_column($column_number);

        if ($column instanceof ObjectTableColumn && $column->is_sortable())
        {
            return new ObjectTableOrder($column->get_property(), $order_direction, $column->get_storage_unit_alias());
        }
        else
        {
            $default_column = $this->get_column($this->get_default_order_column());

            if ($default_column instanceof ObjectTableColumn && $default_column->is_sortable())
            {
                return new ObjectTableOrder($default_column->get_property(), $order_direction, $default_column->get_storage_unit_alias());
            }
            else
            {
                return null;
            }
        }
    }

    /**
     * **************************************************************************************************************
     * Getters and Setters *
     * **************************************************************************************************************
     */

    /**
     * Returns the columns
     *
     * @return TableColumn[]
     */
    public function get_columns()
    {
        return $this->columns;
    }

    /**
     * Sets the columns
     *
     * @param $columns TableColumn[]
     */
    public function set_columns($columns)
    {
        $this->columns = $columns;
    }

    /**
     * Returns the index of the default column to order objects by
     *
     * @return int
     */
    public function get_default_order_column()
    {
        return $this->order_column;
    }

    /**
     * Sets the index of the default column to order objects by
     *
     * @param $column int
     */
    public function set_default_order_column($column_index)
    {
        $this->order_column = $column_index;
    }

    /**
     * Gets the default order direction.
     *
     * @return int - The direction. Either the PHP constant SORT_ASC or
     *         SORT_DESC.
     */
    public function get_default_order_direction()
    {
        return $this->order_direction;
    }

    /**
     * Sets the default order direction.
     *
     * @param $direction int
     *            - The direction. Either the PHP constant SORT_ASC or
     *            SORT_DESC.
     */
    public function set_default_order_direction($direction)
    {
        $this->order_direction = $direction;
    }

    /**
     * **************************************************************************************************************
     * List Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns the number of columns in the model.
     *
     * @return int
     */
    public function get_column_count()
    {
        return count($this->columns);
    }

    /**
     * Gets the column at the given index in the model.
     *
     * @param $index int
     *
     * @return TableColumn The column.
     */
    public function get_column($index)
    {
        return $this->columns[$index];
    }

    /**
     * Adds the given column at the end of the table.
     *
     * @param
     *            TableColumn The column.
     */
    public function add_column(TableColumn $column)
    {
        $this->columns[] = $column;
    }

    /**
     * Delete a column at a given index
     *
     * @param $column_index int
     */
    public function delete_column($column_index)
    {
        unset($this->columns[$column_index]);

        $this->columns = array_values($this->columns);
    }

    /**
     * **************************************************************************************************************
     * Abstract Functionality *
     * **************************************************************************************************************
     */

    /**
     * Initializes the columns for the table
     */
    abstract public function initialize_columns();
}
