<?php
namespace common\libraries;

use Exception;

/**
 * The optional parameters for a DataClass storage layer RETRIEVES action
 *
 * @author Hans De Bisschop
 * @author Sven Vanpoucke - Hogeschool Gent - Added GroupBy
 *
 * @package common.libraries
 */
class RecordRetrieveParameters extends DataClassRetrieveParameters
{
    /**
     * The Properties on which the query has to select (if empty then all the properties of the table will be
     * selected)
     *
     * @var DataClassProperties
     */
    private $properties;

    /**
     * The properties on which the query has to be grouped
     *
     * @var GroupBy
     */
    private $group_by;

    /**
     * Constructor
     *
     * @param $properties \common\libraries\DataClassProperties
     * @param $condition \common\libraries\Condition
     * @param array|multitype $order_by multitype:\common\libraries\ObjectTableOrder
     * @param $joins \common\libraries\Joins
     * @param GroupBy $group_by
     */
    public function __construct(DataClassProperties $properties, $condition = null, $order_by = array(), Joins $joins = null,
        GroupBy $group_by = null)
    {
        parent :: __construct($condition, $order_by, $joins);
        
        $this->properties = $properties;
        $this->group_by = $group_by;
    }

    /**
     * Get properties
     *
     * @return \common\libraries\DataClassProperties
     */
    public function get_properties()
    {
        return $this->properties;
    }

    /**
     * Set properties
     *
     * @param $properties \common\libraries\DataClassProperties
     */
    public function set_properties($properties)
    {
        $this->properties = $properties;
    }

    /**
     * Returns the group by parameter
     *
     * @return GroupBy
     */
    public function get_group_by()
    {
        return $this->group_by;
    }

    /**
     * Sets the group by parameter
     *
     * @param GroupBy $group_by
     */
    public function set_group_by(GroupBy $group_by)
    {
        $this->group_by = $group_by;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param string[] $hash_parts
     *
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = ($this->get_properties() instanceof DataClassProperties ? $this->get_properties()->hash() : null);
        $hash_parts[] = ($this->get_group_by() instanceof GroupBy ? $this->get_group_by()->hash() : null);

        return parent :: hash($hash_parts);
    }

    /**
     * Dynamically generate a valid DataClassRetrievesParameters object based on the type of the input parameter
     *
     * @param $parameter int|\common\libraries\Condition|\common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\DataClassRetrievesParameters boolean
     */
    public static function generate($parameter = null)
    {
        // So you think you're being funny, eh? Right back at you ... you dog-blasted, ornery, no-account, long-eared
        // varmint!
        if (is_object($parameter) && $parameter instanceof RecordRetrieveParameters)
        {
            return $parameter;
        }
        
        // If the parameter is a Condition, generate a new DataClassRetrievesParameters instance using the Condition
        // provided by the context
        elseif (is_object($parameter) && $parameter instanceof Condition)
        {
            return new self(null, $parameter);
        }
        
        // If the parameter is an array, determine whether it's an array of ObjectTableOrder objects and if so generate
        // a DataClassResultParameters
        elseif (is_array($parameter) && count($parameter) > 0 && $parameter[0] instanceof ObjectTableOrder)
        {
            return new self(null, null, $parameter);
        }
        elseif (is_object($parameter) && $parameter instanceof Joins)
        {
            return new self(null, null, null, $parameter);
        }
        elseif (is_object($parameter) && $parameter instanceof GroupBy)
        {
            return new self(null, null, null, null, $parameter);
        }
        elseif (is_object($parameter) && $parameter instanceof DataClassProperties)
        {
            return new self($parameter);
        }
        elseif (is_null($parameter))
        {
            return new self();
        }
        else
        {
            throw new Exception('Illegal parameter passed to the DataManager :: retrieves() method.');
        }
    }

}
