<?php
namespace core\admin\announcement;

use repository\ContentObjectPublicationAttributes;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use rights\NewPlatformGroupEntity;
use rights\NewUserEntity;
use common\libraries\Session;
use common\libraries\RecordRetrievesParameters;
use common\libraries\DataClassProperties;
use common\libraries\PropertiesConditionVariable;
use common\libraries\Joins;
use common\libraries\PropertyConditionVariable;
use common\libraries\EqualityCondition;
use repository\ContentObject;
use common\libraries\DataClassJoin;
use common\libraries\DataClassCountParameters;

class DataManager extends \common\libraries\DataManager
{

    /**
     * Gets the type of DataManager to be instantiated
     *
     * @return string
     */
    public static function get_type()
    {
        return 'doctrine';
    }

    /**
     * Counts publications
     *
     * @param \common\libraries\Condition $condition
     * @return int
     */
    public static function count_publications_for_me($condition, $user_id = null)
    {
        $entities = array();
        $entities[NewUserEntity :: ENTITY_TYPE] = NewUserEntity :: get_instance();
        $entities[NewPlatformGroupEntity :: ENTITY_TYPE] = NewPlatformGroupEntity :: get_instance();

        $condition = self :: get_publications_with_view_right_granted_location_condition(
            $entities,
            $condition,
            $user_id);

        $parameters = new DataClassCountParameters($condition, self :: get_publication_joins());
        return self :: count(Publication :: class_name(), $parameters);
    }

    /**
     * Counts publications
     *
     * @param \common\libraries\Condition $condition
     * @return int
     */
    public static function count_publications($condition)
    {
        $parameters = new DataClassCountParameters($condition, self :: get_publication_joins());

        return self :: count(Publication :: class_name(), $parameters);
    }

    /**
     * Retrieves publications joined with the repository content object table
     *
     * @param \common\libraries\Condition $condition
     * @param \common\libraries\ObjectTableOrder[] $order_by
     * @param int $offset
     * @param int $max_objects
     * @return \common\libraries\ResultSet
     */
    public static function retrieve_publications_for_me($condition = null, $order_by = array(), $offset = 0, $max_objects = -1,
        $user_id = null)
    {
        $entities = array();
        $entities[NewUserEntity :: ENTITY_TYPE] = NewUserEntity :: get_instance();
        $entities[NewPlatformGroupEntity :: ENTITY_TYPE] = NewPlatformGroupEntity :: get_instance();

        $condition = self :: get_publications_with_view_right_granted_location_condition(
            $entities,
            $condition,
            $user_id);

        return self :: retrieve_publications($condition, $order_by, $offset, $max_objects);
    }

    /**
     * Retrieves publications joined with the repository content object table
     *
     * @param \common\libraries\Condition $condition
     * @param \common\libraries\ObjectTableOrder[] $order_by
     * @param int $offset
     * @param int $max_objects
     * @return \common\libraries\ResultSet
     */
    public static function retrieve_publications($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $data_class_properties = array();

        $data_class_properties[] = new PropertiesConditionVariable(Publication :: class_name());

        $data_class_properties[] = new PropertyConditionVariable(
            ContentObject :: class_name(),
            ContentObject :: PROPERTY_TITLE);

        $data_class_properties[] = new PropertyConditionVariable(
            ContentObject :: class_name(),
            ContentObject :: PROPERTY_DESCRIPTION);

        $data_class_properties[] = new PropertyConditionVariable(
            ContentObject :: class_name(),
            ContentObject :: PROPERTY_TYPE);

        $data_class_properties[] = new PropertyConditionVariable(
            ContentObject :: class_name(),
            ContentObject :: PROPERTY_CURRENT);

        $data_class_properties[] = new PropertyConditionVariable(
            ContentObject :: class_name(),
            ContentObject :: PROPERTY_OWNER_ID);

        $properties = new DataClassProperties($data_class_properties);

        $parameters = new RecordRetrievesParameters(
            $properties,
            $condition,
            $max_objects,
            $offset,
            $order_by,
            self :: get_publication_joins());

        return self :: records(Publication :: class_name(), $parameters);
    }

    /**
     * Retrieves the conditions for the publications with view right granted
     *
     * @param RightsLocation $parent_location
     * @param RightsEntity[] $entities
     * @param Condition $condition
     * @param int $user_id
     * @return Condition
     */
    protected static function get_publications_with_view_right_granted_location_condition($entities, $condition,
        $user_id = null)
    {
        $conditions = self :: get_publication_conditions_with_right(
            Rights :: VIEW_RIGHT,
            $entities,
            $user_id,
            $condition);

        if (is_array($conditions) && count($conditions) > 0)
        {
            return new AndCondition($conditions);
        }
    }

    /**
     * Returns the conditions for the publications for a given right
     *
     * @param int $right
     * @param RightsLocation $parent_location
     * @param RightsEntity[] $entities
     * @param int $user_id
     * @param Condition $condition
     * @return Condition
     */
    protected static function get_publication_conditions_with_right($right, $entities, $user_id, $condition)
    {
        $conditions = array();

        $granted_location_ids = Rights :: get_instance()->get_identifiers_with_right_granted(
            $right,
            Manager :: context(),
            Rights :: get_instance()->get_root(Manager :: context()),
            Rights :: TYPE_PUBLICATION,
            $user_id,
            $entities);

        $conditions[] = new InCondition(
            new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_ID),
            $granted_location_ids);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        return $conditions;
    }

    /**
     * Returns the joins for the publication with the content object table
     *
     * @return \common\libraries\Joins
     */
    protected static function get_publication_joins()
    {
        $joins = array();

        $joins[] = new DataClassJoin(
            ContentObject :: class_name(),
            new EqualityCondition(
                new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_CONTENT_OBJECT_ID),
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID)));

        return new Joins($joins);
    }

    public static function get_content_object_publication_attributes(
            $object_id, $type = null, $condition = null,
            $count = null, $offset = null, $order_properties = null)
    {
        $order_by = array();

        if (isset($type))
        {
            if ($type == 'user')
            {
                $condition = new EqualityCondition(Publication :: PROPERTY_PUBLISHER_ID, Session :: get_user_id());

                foreach ($order_properties as $order_property)
                {
                    $property = $order_property->get_property();

                    if ($property != 'application' && $property != 'location')
                    {
                        $order_by[] = $order_property;
                    }
                }
            }
        }
        else
        {
            $condition = new EqualityCondition(Publication :: PROPERTY_CONTENT_OBJECT_ID, $object_id);
        }

        $result = self :: retrieve_publications($condition, $order_by, $offset, $count);

        $publication_attributes = array();

        while ($record = $result->next_result())
        {
            $publication_attributes[] = self :: create_publication_attributes_from_record($record);
        }

        return $publication_attributes;
    }

    public static function get_content_object_publication_attribute($publication_id)
    {
        $condition = new EqualityCondition(Publication :: PROPERTY_ID, $publication_id);
        $record = self :: record(Publication :: class_name(), $condition);

        return self :: create_publication_attributes_from_record($record);
    }

    /**
     * Checks if a content object is published
     *
     * @param int $object_id
     *
     * @return bool
     */
    public static function content_object_is_published($object_id)
    {
        $condition = new EqualityCondition(Publication :: PROPERTY_CONTENT_OBJECT_ID, $object_id);
        return self :: count(Publication :: class_name(), $condition) >= 1;
    }

    public static function any_content_object_is_published($object_ids)
    {
        $condition = new InCondition(Publication :: PROPERTY_CONTENT_OBJECT_ID, $object_ids);
        return self :: count(Publication :: class_name(), $condition) >= 1;
    }

    public static function count_publication_attributes($user = null, $object_id = null, $condition = null)
    {
        $conditions = array();

        if (! $object_id)
        {
            $conditions[] = new EqualityCondition(Publication :: PROPERTY_PUBLISHER_ID, $user->get_id());
        }
        else
        {
            $conditions[] = new EqualityCondition(Publication :: PROPERTY_CONTENT_OBJECT_ID, $object_id);
        }

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $condition = new AndCondition($conditions);

        $parameters = new DataClassCountParameters($condition, self :: get_publication_joins());
        return self :: count(Publication :: class_name(), $parameters);
    }

    public static function delete_content_object_publications($object_id)
    {
        $condition = new EqualityCondition(Publication :: PROPERTY_CONTENT_OBJECT_ID, $object_id);
        $publications = self :: retrieves(Publication :: class_name(), $condition);

        while ($publication = $publications->next_result())
        {
            if (! $publication->delete())
            {
                return false;
            }
        }

        return true;
    }

    public static function delete_content_object_publication($publication_id)
    {
        $publication = self :: retrieve_by_id(Publication :: class_name(), (int) $publication_id);

        if (! $publication instanceof Publication || ! $publication->delete())
        {
            return false;
        }

        return true;
    }

    /**
     * Creates a publication attributes object from a given record
     *
     * @param $record
     * @return \repository\ContentObjectPublicationAttributes
     */
    protected static function create_publication_attributes_from_record($record)
    {
        $attributes = new ContentObjectPublicationAttributes();

        $attributes->set_id($record[Publication :: PROPERTY_ID]);
        $attributes->set_publisher_user_id($record[Publication :: PROPERTY_PUBLISHER_ID]);
        $attributes->set_publication_date($record[Publication :: PROPERTY_PUBLICATION_DATE]);
        $attributes->set_application('admin');

        $url = 'index.php?application=admin&amp;go=' . \core\admin\AdminManager :: ACTION_SYSTEM_ANNOUNCEMENTS .
             '&amp;announcement_action=' . Manager :: ACTION_VIEW . '&amp;' . Manager :: PARAM_SYSTEM_ANNOUNCEMENT_ID .
             '=' . $attributes->get_id();

        $attributes->set_url($url);
        $attributes->set_publication_object_id($record[Publication :: PROPERTY_CONTENT_OBJECT_ID]);

        return $attributes;
    }
}
