<?php
namespace core\admin\announcement;

use repository\ContentObject;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\Theme;
use common\libraries\InCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\RepositoryRights;

class Publisher
{

    /**
     * The component on which this publisher will run
     * 
     * @var Application
     */
    private $parent;

    /**
     * The publications for the form
     * 
     * @var Array<ContentObjectPublication>
     */
    private $publications;

    /**
     * The html that displays the list of available content objects
     */
    private $content_object_html;

    public function __construct($parent, $content_object_ids)
    {
        $this->parent = $parent;
        $this->initialize($content_object_ids);
    }

    /**
     * Initializes the publisher with use of the content object ids
     * 
     * @param $content_object_ids Array
     */
    public function initialize($content_object_ids)
    {
        $html = array();
        
        if (! is_array($content_object_ids))
        {
            $content_object_ids = array($content_object_ids);
        }
        
        $items_to_publish = count($content_object_ids);
        $publications = array();
        
        if ($items_to_publish > 0)
        {
            $condition = new InCondition(
                ContentObject :: PROPERTY_ID, 
                $content_object_ids, 
                ContentObject :: get_table_name());
            $parameters = new DataClassRetrievesParameters($condition);
            
            $content_objects = \repository\DataManager :: retrieve_active_content_objects(
                ContentObject :: class_name(), 
                $parameters);
            
            $html[] = '<div class="content_object padding_10">';
            $html[] = '<div class="title">' . Translation :: get(
                'SelectedContentObjects', 
                null, 
                Utilities :: COMMON_LIBRARIES) . '</div>';
            $html[] = '<div class="description">';
            $html[] = '<ul class="attachments_list">';
            
            while ($content_object = $content_objects->next_result())
            {
                $namespace = ContentObject :: get_content_object_type_namespace($content_object->get_type());
                
                if ($content_object->has_right(RepositoryRights :: USE_RIGHT, $this->parent->get_user_id()))
                {
                    $html[] = '<li><img src="' . $content_object->get_icon_path(Theme :: ICON_MINI) . '" alt="' .
                         htmlentities(Translation :: get('TypeName', null, $namespace)) . '"/> ' .
                         $content_object->get_title() . '</li>';
                    
                    $publication = new Publication();
                    $publication->set_content_object_id($content_object->get_id());
                    $publication->set_publisher_id($this->parent->get_user_id());
                    $publication->set_publisher($this->parent->get_user());
                    $publications[] = $publication;
                }
                else
                {
                    $html[] = '<li><img src="' . Theme :: get_image_path($namespace) . 'logo/' . Theme :: ICON_MINI .
                         '.png" alt="' . htmlentities(Translation :: get('TypeName', null, $namespace)) . '"/> ' .
                         $content_object->get_title() . ' <span style="color:red;font-style:italic">' . Translation :: get(
                            'NotAllowed') . '</span></li>';
                }
            }
            
            $html[] = '</ul>';
            $html[] = '</div>';
            $html[] = '</div>';
        }
        
        $this->publications = $publications;
        $this->content_object_html = $html;
        
        $form = new PublicationForm(PublicationForm :: TYPE_CREATE, $publications, $this->parent->get_url());
        
        $this->content_object_publication_form = $form;
    }

    /**
     * Returns the publications for the content objects
     * 
     * @var Array<ContentObjectPublication>
     */
    public function get_publications()
    {
        return $this->publications;
    }

    /**
     * Validates the publication form
     * 
     * @return boolean
     */
    public function ready_to_publish()
    {
        return $this->content_object_publication_form->validate();
    }

    /**
     * Handles the submit of the publication form
     * 
     * @return boolean
     */
    public function publish()
    {
        return $this->content_object_publication_form->handle_form_submit();
    }

    /**
     * Renders the form with the content object listings
     * 
     * @return String
     */
    public function toHtml()
    {
        $html = $this->content_object_html;
        $html[] = $this->content_object_publication_form->toHtml();
        $html[] = '<div style="clear: both;"></div>';
        
        return implode("\n", $html);
    }
}
