<?php
namespace core\admin\announcement;

use rights\NewUserEntity;
use rights\NewPlatformGroupEntity;
use repository\ContentObject;
use common\libraries\TableCellRendererActionsColumnSupport;
use common\libraries\RecordTableCellRenderer;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\DatetimeUtilities;
use common\libraries\Utilities;
use common\libraries\Translation;

class PublicationTableCellRenderer extends RecordTableCellRenderer implements TableCellRendererActionsColumnSupport
{

    public function render_cell($column, $publication)
    {
        $content_object = \repository\DataManager :: retrieve_content_object(
            $publication[Publication :: PROPERTY_CONTENT_OBJECT_ID]);
        
        switch ($column->get_name())
        {
            case PublicationTableColumnModel :: COLUMN_STATUS :
                return $content_object->get_icon_image(
                    Theme :: ICON_MINI, 
                    ! (boolean) $publication[Publication :: PROPERTY_HIDDEN]);
                break;
            case ContentObject :: PROPERTY_TITLE :
                $title_short = $content_object->get_title();
                $title_short = Utilities :: truncate_string($title_short, 53, false);
                
                $style = $publication[Publication :: PROPERTY_HIDDEN] ? ' style="color: gray;"' : '';
                
                return '<a' . $style . ' href="' .
                     htmlentities(
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_VIEW, 
                                Manager :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication[Publication :: PROPERTY_ID]))) .
                     '" title="' . $content_object->get_title() . '">' . $title_short . '</a>';
                break;
            case Publication :: PROPERTY_PUBLICATION_DATE :
                $date_format = Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES);
                $data = DatetimeUtilities :: format_locale_date(
                    $date_format, 
                    $publication[Publication :: PROPERTY_PUBLICATION_DATE]);
                break;
            case Publication :: PROPERTY_PUBLISHER_ID :
                $user = \user\DataManager :: retrieve_by_id(
                    \user\User :: class_name(), 
                    (int) $publication[Publication :: PROPERTY_PUBLISHER_ID]);
                if (! $user)
                {
                    $data = '<i>' . Translation :: get('UserUnknown') . '</i>';
                }
                else
                {
                    $data = $user->get_fullname();
                }
                break;
            case PublicationTableColumnModel :: COLUMN_PUBLISHED_FOR :
                if ($publication[Publication :: PROPERTY_EMAIL_SENT])
                {
                    $email_icon = ' - <img src="' . Theme :: get_common_image_path() . 'action_email.png" alt=""
                        style="vertical-align: middle;" title="' .
                         Translation :: get('SentByEmail') . '"/>';
                }
                $data = '<div style="float: left;">' . $this->render_publication_targets($publication) . '</div>' .
                     $email_icon;
                break;
            case ContentObject :: PROPERTY_DESCRIPTION :
                $data = $publication[ContentObject :: PROPERTY_DESCRIPTION];
                $data = Utilities :: truncate_string($data, 100);
        }
        
        if ($data)
        {
            if ($publication[Publication :: PROPERTY_HIDDEN])
            {
                return '<span style="color: gray">' . $data . '</span>';
            }
            else
            {
                return $data;
            }
        }
        
        return parent :: render_cell($column, $publication);
    }

    public function get_actions($publication)
    {
        $toolbar = new Toolbar();
        
        if ($this->get_component()->get_user()->is_platform_admin() ||
             $publication[Publication :: PROPERTY_PUBLISHER_ID] == $this->get_component()->get_user()->get_id())
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Edit', array(), Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . 'action_edit.png', 
                    $this->get_component()->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_EDIT, 
                            Manager :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication[Publication :: PROPERTY_ID])), 
                    ToolbarItem :: DISPLAY_ICON));
            
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', array(), Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . 'action_delete.png', 
                    $this->get_component()->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_DELETE, 
                            Manager :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication[Publication :: PROPERTY_ID])), 
                    ToolbarItem :: DISPLAY_ICON, 
                    true));
            
            if ($publication[Publication :: PROPERTY_HIDDEN])
            {
                $visibility_img = 'action_invisible.png';
            }
            elseif ($publication[Publication :: PROPERTY_FROM_DATE] == 0 &&
                 $publication[Publication :: PROPERTY_TO_DATE] == 0)
            {
                $visibility_img = 'action_visible.png';
            }
            else
            {
                $visibility_img = 'action_period.png';
            }
            
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Hide', array(), Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . $visibility_img, 
                    $this->get_component()->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_HIDE, 
                            Manager :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication[Publication :: PROPERTY_ID])), 
                    ToolbarItem :: DISPLAY_ICON));
        }
        return $toolbar->as_html();
    }

    /**
     * Renders the publication targets
     * 
     * @param Publication $publication
     *
     * @return string
     */
    public function render_publication_targets($publication)
    {
        $target_entities = Rights :: get_instance()->get_target_entities(
            Rights :: VIEW_RIGHT, 
            Manager :: context(), 
            $publication[Publication :: PROPERTY_ID], 
            Rights :: TYPE_PUBLICATION);
        
        $target_list = array();
        
        if (array_key_exists(0, $target_entities[0]))
        {
            $target_list[] = Translation :: get('Everybody', null, Utilities :: COMMON_LIBRARIES);
        }
        else
        {
            $target_list[] = '<select>';
            
            foreach ($target_entities as $entity_type => $entity_ids)
            {
                switch ($entity_type)
                {
                    case NewPlatformGroupEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $group_id)
                        {
                            $group = \group\DataManager :: retrieve_by_id(\group\Group :: class_name(), $group_id);
                            if ($group)
                            {
                                $target_list[] = '<option>' . $group->get_name() . '</option>';
                            }
                        }
                        break;
                    case NewUserEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $user_id)
                        {
                            $user = \user\DataManager :: retrieve_by_id(\user\User :: class_name(), (int) $user_id);
                            if ($user)
                            {
                                $target_list[] = '<option>' . $user->get_fullname() . '</option>';
                            }
                        }
                        break;
                    case 0 :
                        $target_list[] = '<option>Everyone</option>';
                        break;
                }
            }
            $target_list[] = '</select>';
        }
        
        return implode("\n", $target_list);
    }
}
