<?php
namespace core\install;

use common\libraries\SystemCache;
use common\libraries\CoreApplication;
use common\libraries\Path;
use common\libraries\Filesystem;
use common\libraries\Translation;
use common\libraries\PlatformSetting;
use common\libraries\Utilities;
use Exception;

class PlatformInstaller
{

    /**
     *
     * @var InstallerConfig $installer_config
     */
    private $installer_config;

    /**
     *
     * @var string where to write the new config file
     */
    private $config_file_destination;

    /**
     *
     * @var install\InstallerObservers list of the observers to be notified when performing the install
     */
    private $observers;

    /**
     *
     * @var install\DataManagerInterface the storage manager used to perform install
     */
    private $data_manager;

    /**
     *
     * @var multitype:string
     */
    private $packages;

    public function __construct(InstallerConfig $installer_config, $data_manager)
    {
        $this->set_installer_config($installer_config);
        $default_config_file = Path :: get_common_path() . 'configuration/configuration.php';
        $this->set_config_file_destination($default_config_file);

        $this->set_data_manager($data_manager);

        $this->packages = array();
        $this->observers = new InstallerObservers();
    }

    public function set_data_manager($data_manager)
    {
        $this->data_manager = $data_manager;
    }

    public function set_installer_config($installer_config)
    {
        $this->installer_config = $installer_config;
    }

    public function set_config_file_destination($config_file_destination)
    {
        $this->config_file_destination = $config_file_destination;
    }

    public function add_observer(InstallerObserver $observer)
    {
        $this->observers->add_observer($observer);
    }

    /**
     *
     * @return multitype:string
     */
    public function get_packages()
    {
        return $this->packages;
    }

    /**
     *
     * @param multitype:string $packages
     */
    public function set_packages($packages)
    {
        $this->packages = $packages;
    }

    /**
     *
     * @param string $context
     */
    public function add_package($context)
    {
        array_push($this->packages, $context);
    }

    /**
     *
     * @param multitype:string $packages
     */
    public function add_packages($packages)
    {
        foreach ($packages as $package)
        {
            $this->add_package($package);
        }
    }

    /**
     *
     * @return string
     */
    public function get_next_package()
    {
        return array_shift($this->packages);
    }

    public function perform_install()
    {
        $this->observers->before_install();

        $this->initialize_install();
        $this->perform_preprod();
        $this->install_packages();

        $this->observers->after_install();

        SystemCache :: reset_system();
        // not so good but good enough for now
        SystemCache :: reset_client();
    }

    private function initialize_install()
    {
        PlatformSetting :: set_instance(new InstallationPlatformSettings());
        Translation :: get_instance()->set_language($this->installer_config->get_platform_language());
    }

    private function perform_preprod()
    {
        $this->observers->before_preprod();
        $result_config = $this->write_config_file();
        $this->observers->preprod_config_file_written($result_config);

        $this->data_manager->init_storage_access();
        $this->data_manager->init_storage_structure();

        $result = new InstallerStepResult(true, Translation :: get('DatabaseCreated'));
        $this->observers->preprod_db_created($result);

        $this->create_folders();

        $this->observers->after_preprod();
    }

    private function write_config_file()
    {
        $config_template_file = Path :: get_common_path() . 'configuration/configuration.dist.php';
        $template = file_get_contents($config_template_file);

        if ($template === false)
        {
            throw new \Exception(Translation :: get('ConfigWriteFailed'));
        }
        $config_string = $this->template_to_config($template);

        $write_status = Filesystem :: write_to_file($this->config_file_destination, $config_string);

        if ($write_status === false)
        {
            throw new \Exception(Translation :: get('ConfigWriteFailed'));
        }

        return new InstallerStepResult(true, Translation :: get('ConfigWriteSuccess'));
    }

    private function template_to_config($template)
    {
        $config = array();
        $config['{DATABASE_DRIVER}'] = $this->installer_config->get_db_driver();
        $config['{DATABASE_HOST}'] = $this->installer_config->get_db_host();
        $config['{DATABASE_USER}'] = $this->installer_config->get_db_username();
        $config['{DATABASE_PASSWORD}'] = $this->installer_config->get_db_password();
        $config['{DATABASE_NAME}'] = $this->installer_config->get_db_name();
        $config['{ROOT_WEB}'] = $this->installer_config->get_base_url();
        $config['{ROOT_SYS}'] = str_replace('\\', '/', realpath($config['{ROOT_WEB}']) . '/');
        $config['{SECURITY_KEY}'] = md5(uniqid(rand() . time()));
        $config['{URL_APPEND}'] = $this->installer_config->get_url_append();
        $config['{HASHING_ALGORITHM}'] = $this->installer_config->get_crypt_algorithm();
        $config['{INSTALL_DATE}'] = time();

        $result = $template;
        foreach ($config as $key => $value)
        {
            $result = str_replace($key, $value, $result);
        }
        return $result;
    }

    private function install_packages()
    {
        $this->add_package('common\libraries');
        $this->add_packages(CoreApplication :: get_packages_from_filesystem());
        $this->add_packages(Utilities :: get_common_extensions_packages_from_filesystem());
        $this->add_packages($this->installer_config->get_content_objects());
        $this->add_packages($this->installer_config->get_applications());
        $this->add_packages($this->installer_config->get_external_instances());

        $this->observers->before_packages_install();

        while (($package = $this->get_next_package()) != null)
        {
            $this->observers->before_package_install($package);
            $values = $this->installer_config->as_values_array();
            $installer = \common\libraries\package\Installer :: factory($package, $values);

            $success = $installer->run();

            if ($success !== true)
            {
                throw new InstallFailedException($package, $installer->retrieve_message());
            }

            $this->add_packages($installer->get_additional_packages());

            $step_result = new InstallerStepResult($success, $installer->get_message(), $package);
            $this->observers->after_package_install($step_result);
        }

        $this->observers->after_packages_install();
    }

    private function create_folders()
    {
        $this->observers->before_filesystem_prepared();
        $files_path = Path :: get(SYS_FILE_PATH);
        $directories = array('archive', 'cache', 'garbage', 'repository', 'temp', 'userpictures', 'scorm', 'logs',
                'hotpotatoes');

        foreach ($directories as $directory)
        {
            $path = $files_path . $directory;

            if (file_exists($path) && is_dir($path))
            {
                Filesystem :: remove($path);
            }

            if (! Filesystem :: create_dir($path))
            {
                throw new \Exception(Translation :: get('FoldersCreatedFailed'));
            }
        }

        $step_result = new InstallerStepResult(true, Translation :: get('FoldersCreatedSuccess'));
        $this->observers->after_filesystem_prepared($step_result);
    }
}
