<?php
namespace core\lynx\manager;

use common\libraries\Theme;
use common\libraries\Utilities;
use core\lynx\Action;
use common\libraries\CommonDataManager;
use common\libraries\Translation;

set_time_limit(0);

/**
 * Package installation
 *
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 */
class PackageInstaller extends Action
{

    private $additional_packages = array();

    /**
     * Runs the package installer
     *
     * @return boolean
     */
    public function run()
    {
        if ($this->initialize() && $this->process())
        {
            $title = Translation :: get('Finished', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                 'package_action/finished.png';
            return $this->action_successful($title, $image, Translation :: get('PackageCompletelyInstalled'));
        }
        else
        {
            $title = Translation :: get('Failed', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                 'package_action/failed.png';
            return $this->action_failed($title, $image, Translation :: get('PackageInstallFailed'));
        }
    }

    /**
     * Initializes the package installer
     *
     * @return boolean
     */
    public function initialize()
    {
        $title = Translation :: get('Initialization', null, Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
             'package_action/initialization.png';

        if (! $this->get_package() instanceof \common\libraries\package\Package)
        {
            return $this->action_failed($title, $image, Translation :: get('PackageAttributesNotFound'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageAttributesFound'));
        }

        // Check registration
        if ($this->is_package_registered())
        {
            return $this->action_failed($title, $image, Translation :: get('PackageIsAlreadyRegistered'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageNotYetRegistered'));
        }

        return $this->action_successful($title, $image, Translation :: get('PackageInstallInitialized'));
    }

    /**
     * Checks if the package is registered
     *
     * @return boolean
     */
    public function is_package_registered()
    {
        return CommonDataManager :: is_registered($this->get_package()->get_context());
    }

    /**
     * Installs the package
     *
     * @return boolean
     */
    public function process()
    {
        $title = Translation :: get(
            'Installation',
            array('PACKAGE' => Translation :: get('TypeName', null, $this->get_package()->get_context())),
            Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
             'package_action/installation.png';

        $installer = \common\libraries\package\Installer :: factory($this->get_package()->get_context(), array());
        if (! $installer->run())
        {
            $this->add_message($installer->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('InitializationFailed'));
        }
        else
        {
            $this->add_message($installer->retrieve_message());
            $this->action_successful($title, $image);
        }

        $this->add_additional_packages($installer->get_additional_packages());

        $title = Translation :: get('AdditionalPackages', null, Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
             'package_action/additional_packages.png';

        while (($additional_package = $this->get_next_additional_package()) != null)
        {
            if (! $this->install_additional_package($additional_package))
            {
                return $this->action_failed($title, $image, Translation :: get('AdditionalPackagesFailed'));
            }
        }

        return true;
    }

    /**
     * Installs an additional package
     *
     * @param $package_installer \common\libraries\package\Installer
     */
    private function install_additional_package($context)
    {
        $title = Translation :: get(
            'Installation',
            array('PACKAGE' => Translation :: get('TypeName', null, $context)),
            Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path($context) . 'logo/48.png';

        $installer = \common\libraries\package\Installer :: factory($context, array());

        if (! $installer->run())
        {
            $this->add_message($installer->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('InitializationFailed'));
        }
        else
        {
            $this->add_message($installer->retrieve_message());
            $this->action_successful($title, $image);
        }

        $this->add_additional_packages($installer->get_additional_packages());

        return true;
    }

    /**
     * Returns the additional packages
     *
     * @return multitype:string
     */
    public function get_additional_packages()
    {
        return $this->additional_packages;
    }

    /**
     * Sets the additional packages
     *
     * @param multitype:string
     */
    public function set_additional_packages($additional_packages)
    {
        $this->additional_packages = $additional_packages;
    }

    /**
     * Adds an additional package to the list of additional packages
     *
     * @param string
     */
    public function add_additional_package($context)
    {
        array_push($this->additional_packages, $context);
    }

    /**
     * Adds multiple additional packages to the list of additional packages
     *
     * @param multitype:string
     */
    public function add_additional_packages($additional_packages)
    {
        foreach ($additional_packages as $additional_package)
        {
            $this->add_additional_package($additional_package);
        }
    }

    /**
     * Removes and returns the first package from the list of additional packages
     *
     * @return string
     */
    public function get_next_additional_package()
    {
        return array_shift($this->additional_packages);
    }
}
