<?php
namespace core\lynx\manager;

use common\libraries\Theme;
use common\libraries\Utilities;
use core\lynx\Action;
use common\libraries\Translation;

set_time_limit(0);

/**
 * Package installation
 *
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 */
class PackageUpgrader extends Action
{

    /**
     *
     * @var multitype:string $additional_packages
     */
    private $additional_packages = array();

    /**
     *
     * @var boolean
     */
    private $upgrade_additional_packages;

    /**
     *
     * @var \common\libraries\package\Upgrader
     */
    private $upgrader;

    public function __construct($context, $upgrade_additional_packages = true)
    {
        parent :: __construct($context);
        $this->upgrade_additional_packages = $upgrade_additional_packages;
    }

    /**
     * Runs the package remover
     *
     * @return boolean
     */
    public function run()
    {
        if ($this->process())
        {
            $title = Translation :: get('Finished', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                 'package_action/finished.png';
            return $this->action_successful($title, $image, Translation :: get('PackageCompletelyUpgraded'));
        }
        else
        {
            $title = Translation :: get('Failed', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                 'package_action/failed.png';
            return $this->action_failed($title, $image, Translation :: get('PackageUpgradeFailed'));
        }
    }

    /**
     * Installs the package
     *
     * @return boolean
     */
    public function process()
    {
        $title = Translation :: get('Initialization', null, Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
             'package_action/initialization.png';

        if (! $this->get_package() instanceof \common\libraries\package\Package)
        {
            return $this->action_failed($title, $image, Translation :: get('PackageAttributesNotFound'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageAttributesFound'));
        }

        $title = Translation :: get(
            'Upgrade',
            array('PACKAGE' => Translation :: get('TypeName', null, $this->get_package()->get_context())),
            Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) . 'package_action/upgrade.png';

        try
        {
            $this->upgrader = \common\libraries\package\Upgrader :: factory($this->get_package()->get_context());
        }
        catch(\Exception $ex)
        {
            $this->add_message(
                Translation :: get(
                    'UpgraderNotFoundIgnoringPackage', array('CONTEXT' => $this->get_package()->get_context())
                )
            );

            return $this->action_successful($title, $image);
        }

        if (! $this->upgrader->run())
        {
            $this->add_message($this->upgrader->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('UpgradeFailed'));
        }
        else
        {
            $this->add_message($this->upgrader->retrieve_message());
            $this->action_successful($title, $image);
        }

        if ($this->upgrade_additional_packages)
        {
            $this->add_additional_packages($this->upgrader->get_additional_packages());

            $title = Translation :: get('AdditionalPackages', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                'package_action/additional_packages.png';

            while (($additional_package = $this->get_next_additional_package()) != null)
            {
                if (! $this->upgrade_additional_package($additional_package))
                {
                    return $this->action_failed($title, $image, Translation :: get('AdditionalPackagesFailed'));
                }
            }
        }

        return true;
    }

    /**
     * Upgrades an additional package
     *
     * @param string $context
     */
    private function upgrade_additional_package($context)
    {
        $title = Translation :: get(
            'Upgrade',
            array('PACKAGE' => Translation :: get('TypeName', null, $context)),
            Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path($context) . 'logo/48.png';

        $upgrader = \common\libraries\package\Upgrader :: factory($context);

        if (! $upgrader->run())
        {
            $this->add_message($upgrader->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('UpgradeFailed'));
        }
        else
        {
            $this->add_message($upgrader->retrieve_message());
            $this->add_message(Translation :: get('PackageUpgraded'), self :: TYPE_CONFIRM);
            $this->add_additional_packages($upgrader->get_additional_packages());
            return $this->action_successful($title, $image);
        }
    }

    /**
     * Returns the additional packages
     *
     * @return multitype:string
     */
    public function get_additional_packages()
    {
        return $this->additional_packages;
    }

    /**
     * Sets the additional packages
     *
     * @param multitype:string
     */
    public function set_additional_packages($additional_packages)
    {
        $this->additional_packages = $additional_packages;
    }

    /**
     * Adds an additional package to the list of additional packages
     *
     * @param string
     */
    public function add_additional_package($context)
    {
        array_push($this->additional_packages, $context);
    }

    /**
     * Adds multiple additional packages to the list of additional packages
     *
     * @param multitype:string
     */
    public function add_additional_packages($additional_packages)
    {
        foreach ($additional_packages as $additional_package)
        {
            $this->add_additional_package($additional_package);
        }
    }

    /**
     * Removes and returns the first package from the list of additional packages
     *
     * @return string
     */
    public function get_next_additional_package()
    {
        return array_shift($this->additional_packages);
    }

    public function get_upgrader()
    {
        return $this->upgrader;
    }
}
