<?php
namespace core\menu;

use repository\ExternalInstance;
use repository\RepositoryManager;
use common\libraries\Header;
use common\libraries\ArrayResultSet;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\ObjectTableOrder;
use common\libraries\PlatformSetting;
use common\libraries\DataClassRetrievesParameters;
use user\UserManager;

/**
 *
 * @author Hans De Bisschop
 */
abstract class MenuRenderer
{
    const TYPE_MINI_BAR = 'mini_bar';
    const TYPE_SITE_MAP = 'site_map';

    /**
     *
     * @var User null
     */
    private $user;

    /**
     * The layout of the menubar
     *
     * @var String
     */
    protected $html;

    /**
     *
     * @param $user User|null
     */
    public function __construct($user = null)
    {
        $this->user = $user;
    }

    /**
     *
     * @return User null
     */
    public function get_user()
    {
        return $this->user;
    }

    /**
     *
     * @param $type string
     * @param $user User|null
     * @return MenuRenderer
     */
    public static function factory($type, $user)
    {
        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'MenuRenderer';
        $renderer = new $class($user);
        return $renderer;
    }

    /**
     *
     * @param $type string
     * @param $user User|null
     * @return string
     */
    public static function as_html($type, $user)
    {
        return self :: factory($type, $user)->render();
    }

    public function get_menu_items()
    {
        $condition = new EqualityCondition(Item :: PROPERTY_PARENT, 0);
        $order_by = array();
        $order_by[] = new ObjectTableOrder(Item :: PROPERTY_SORT);

        $parameters = new DataClassRetrievesParameters($condition, null, null, $order_by);
        return DataManager :: retrieves(Item :: class_name(), $parameters);
    }

    /**
     * Renders the menu
     *
     * @return string
     */
    public function render()
    {
        $user = $this->get_user();

        if (! $user)
        {
            return;
        }

        $items = $this->get_menu_items();

        $current_section = Header :: get_section();

        $html = array();

        $html[] = $this->display_menu_header($current_section);

        $category_items = array();

        if (PlatformSetting :: get('allow_portal_functionality') || $user->is_platform_admin())
        {
            // home
            $home = new LinkApplicationItem();
            $item_title = new ItemTitle();
            $item_title->set_title(Translation :: get('Home'));
            $item_title->set_isocode(Translation :: get_instance()->get_language());
            $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

            $home->set_titles($item_titles);
            $home->set_url(Redirect :: get_link());
            $home->set_target(LinkItem :: TARGET_SELF);
            $home->set_section('home');
            $html[] = ItemRenderer :: as_html($this, $home);
        }

        while ($item = $items->next_result())
        {
            $html[] = ItemRenderer :: as_html($this, $item);
        }

        if (isset($user))
        {
            if (PlatformSetting :: get('allow_portal_functionality') || $user->is_platform_admin())
            {
                // my account
                $my_account = new LinkApplicationItem();
                $item_title = new ItemTitle();
                $item_title->set_title(Translation :: get('MyAccount'));
                $item_title->set_isocode(Translation :: get_instance()->get_language());
                $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

                $my_account->set_titles($item_titles);
                $my_account->set_url(
                        Redirect :: get_link(UserManager :: APPLICATION_NAME,
                                array(Application :: PARAM_ACTION => UserManager :: ACTION_VIEW_ACCOUNT)));
                $my_account->set_target(LinkItem :: TARGET_SELF);
                $my_account->set_section('my_account');
                $html[] = ItemRenderer :: as_html($this, $my_account);

                // Repository
                $repository = new ApplicationItem();
                $item_title = new ItemTitle();
                $item_title->set_title(Translation :: get('Repository'));
                $item_title->set_isocode(Translation :: get_instance()->get_language());
                $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

                $repository->set_titles($item_titles);
                $repository->set_application(RepositoryManager :: APPLICATION_NAME);
                $html[] = ItemRenderer :: as_html($this, $repository);
            }

            $external_instance_item = $this->get_external_instance_menu_items();
            if (isset($external_instance_item) && (count($external_instance_item) > 0))
            {
                // ExternalRepositories
                $external_repositories = new CategoryItem();
                $item_title = new ItemTitle();
                $item_title->set_title(Translation :: get('ExternalRepositories'));
                $item_title->set_isocode(Translation :: get_instance()->get_language());
                $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

                $external_repositories->set_titles($item_titles);

                $external_repositories->set_children($external_instance_item);
                $html[] = ItemRenderer :: as_html($this, $external_repositories);
            }

            if ($user->is_platform_admin())
            {
                // Administration
                $administration = new ApplicationItem();
                $item_title = new ItemTitle();
                $item_title->set_title(Translation :: get('Administration'));
                $item_title->set_isocode(Translation :: get_instance()->get_language());
                $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

                $administration->set_titles($item_titles);
                $administration->set_application(
                        Utilities :: get_package_name_from_namespace(\core\admin\AdminManager :: context()));
                $html[] = ItemRenderer :: as_html($this, $administration);
            }

            if (! $user->is_anonymous_user())
            {
                // Logout
                $logout = new LinkApplicationItem();
                $item_title = new ItemTitle();
                $item_title->set_title(Translation :: get('Logout'));
                $item_title->set_isocode(Translation :: get_instance()->get_language());
                $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

                $logout->set_titles($item_titles);
                $logout->set_url(
                        Redirect :: get_link(UserManager :: APPLICATION_NAME,
                                array(Application :: PARAM_ACTION => UserManager :: ACTION_LOGOUT)));
                $logout->set_target(LinkItem :: TARGET_SELF);
                $html[] = ItemRenderer :: as_html($this, $logout);
            }
        }

        $html[] = $this->display_menu_footer();

        return implode("\n", $html);
    }

    abstract public function display_menu_header($current_section);

    abstract public function display_menu_footer();

    public function get_external_instance_menu_items()
    {
        $condition = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);

        $parameters = new DataClassRetrievesParameters($condition, null, null,
                new ObjectTableOrder(ExternalInstance :: PROPERTY_TITLE));
        $external_instance_managers = \repository\DataManager :: retrieves(ExternalInstance :: class_name(),
                $parameters);

        while ($external_instance_manager = $external_instance_managers->next_result())
        {
            if ($external_instance_manager instanceof ExternalInstance && $external_instance_manager->is_enabled())
            {
                $external_instance_manager_sub_item = new LinkApplicationItem();
                $item_title = new ItemTitle();
                $item_title->set_title($external_instance_manager->get_title());
                $item_title->set_isocode(Translation :: get_instance()->get_language());
                $item_titles = new ItemTitles(new ArrayResultSet(array($item_title)));

                $external_instance_manager_sub_item->set_titles($item_titles);
                $external_instance_manager_sub_item->set_url(
                        Redirect :: get_link(RepositoryManager :: APPLICATION_NAME,
                                array(
                                        Application :: PARAM_ACTION => RepositoryManager :: ACTION_EXTERNAL_INSTANCE_MANAGER,
                                        RepositoryManager :: PARAM_EXTERNAL_INSTANCE => $external_instance_manager->get_id())));

                $external_instance_manager_sub_item->set_target(LinkItem :: TARGET_SELF);
                $external_instance_manager_sub_item->set_section('external_repository');
                $external_instance_manager_sub_item->set_parent(1);
                $external_instance_manager_items[] = $external_instance_manager_sub_item;
            }
        }

        return $external_instance_manager_items;
    }
}
