<?php
namespace group;

use common\libraries\NotCondition;
use common\libraries\Utilities;
use common\libraries\InequalityCondition;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\NestedSet;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\InCondition;
use core\metadata\GroupMetadataElementValue;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassCountDistinctParameters;
use common\libraries\DataClassJoin;
use common\libraries\Joins;
use common\libraries\OrCondition;

/**
 * $Id: group.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib
 */

/**
 *
 * @author Hans de Bisschop
 * @author Dieter De Neef
 * @author Sven Vanpoucke
 */
class Group extends NestedSet
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_NAME = 'name';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_SORT = 'sort';
    const PROPERTY_CODE = 'code';
    const PROPERTY_DISK_QUOTA = 'disk_quota';
    const PROPERTY_DATABASE_QUOTA = 'database_quota';

    /**
     * Array used to cache users in this group, depending on request (include subgroups, recursive subgroups)
     *
     * @var array
     */
    private $users;

    /**
     * Array used to cache user counts in this group, depending on request (include subgroups, recursive subgroups)
     *
     * @var array
     */
    private $user_count;

    /**
     * Array used to cache subgroups, depending on request (recursive or not)
     *
     * @var array
     */
    private $subgroups;

    /**
     * Array used to cache subgroup counts, depending on request (recursive or not)
     *
     * @var array
     */
    private $subgroup_count;

    /**
     * Cache of the siblings of a group, depending on request (recursive or not)
     *
     * @var array
     */
    private $siblings;

    /**
     * ObjectResultSet used to cache groups this group can use
     *
     * @var ObjectResultSet
     */
    private $allowed_groups;

    /**
     * Get the default properties of all groups.
     *
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
            array(
                self :: PROPERTY_NAME,
                self :: PROPERTY_DESCRIPTION,
                self :: PROPERTY_SORT,
                self :: PROPERTY_CODE,
                self :: PROPERTY_DISK_QUOTA,
                self :: PROPERTY_DATABASE_QUOTA));
    }

    /*
     * Gets the table name for this class
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    /**
     * inherited
     */
    public function get_data_manager()
    {
        return DataManager :: get_instance();
    }

    /**
     * Returns the name of this group.
     *
     * @return String The name
     */
    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    public function get_fully_qualified_name($include_self = true)
    {
        $parents = $this->get_parents($include_self);
        $names = array();

        while ($node = $parents->next_result())
        {
            $names[] = $node->get_name();
        }

        return implode(' <span class="visible">></span> ', array_reverse($names));
    }

    /**
     * Returns the description of this group.
     *
     * @return String The description
     */
    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    public function get_sort()
    {
        return $this->get_default_property(self :: PROPERTY_SORT);
    }

    public function get_code()
    {
        return $this->get_default_property(self :: PROPERTY_CODE);
    }

    /**
     * Returns the disk quota for users in this group.
     *
     * @return Int the disk quota
     */
    public function get_disk_quota()
    {
        return $this->get_default_property(self :: PROPERTY_DISK_QUOTA);
    }

    /**
     * Returns the database quota for users in this group.
     *
     * @return Int the database quota
     */
    public function get_database_quota()
    {
        return $this->get_default_property(self :: PROPERTY_DATABASE_QUOTA);
    }

    /**
     * Sets the name of this group.
     *
     * @param String $name the name.
     */
    public function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    public function set_code($code)
    {
        $this->set_default_property(self :: PROPERTY_CODE, $code);
    }

    public function set_disk_quota($disk_quota)
    {
        $this->set_default_property(self :: PROPERTY_DISK_QUOTA, $disk_quota);
    }

    public function set_database_quota($database_quota)
    {
        $this->set_default_property(self :: PROPERTY_DATABASE_QUOTA, $database_quota);
    }

    public function set_description($description)
    {
        $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    public function set_sort($sort)
    {
        $this->set_default_property(self :: PROPERTY_SORT, $sort);
    }

    /**
     *
     * @deprecated use get_parent_id() instead.
     */
    public function get_parent()
    {
        return $this->get_parent_id();
    }

    /**
     *
     * @deprecated use set_parent_id() || move instead.
     */
    public function set_parent($parent)
    {
        $this->set_parent_id($parent);
    }

    /**
     * Get all of the group's parents
     *
     * @deprecated should use get_ancestors() instead
     */
    public function get_parents($include_self = true)
    {
        return $this->get_ancestors($include_self);
    }

    /**
     *
     * @deprecated should use is_descendant_of
     */
    public function is_child_of($parent)
    {
        return $this->is_descendant_of($parent);
    }

    /**
     *
     * @deprecated should use is_ancestor_of
     */
    public function is_parent_of($child)
    {
        return $this->is_ancestor_of($child);
    }

    /**
     * Get the group's children
     */
    public function get_children($recursive = true)
    {
        return $this->get_subgroups($recursive);
    }

    public function move($new_parent_id, $new_previous_id = 0)
    {
        if ($new_previous_id != 0)
        {
            return parent :: move(self :: AS_NEXT_SIBLING_OF, $new_previous_id);
        }
        else
        {
            return parent :: move(self :: AS_LAST_CHILD_OF, $new_parent_id);
        }
    }

    /**
     * Instructs the DataManager to delete this group.
     *
     * @param in_batch - delete groups in batch and fix nested values later
     * @return boolean True if success, false otherwise.
     * @deprecated should use $this->delete() of self :: deletes( $array ) instead
     */
    public function delete_group($in_batch = false)
    {
        return self :: delete();
    }

    /**
     * Deletes all content related to a group: - Users that are part of a group - Meta-data describing the group -
     * Rights templates tied to a group In principle, these could be deleted implicitly when using foreign keys (the
     * group id) with cascading deletes.
     */
    public function delete_related_content()
    {
        // First, truncate the group so that users are removed.
        $success = $this->truncate();

        // Second, delete the associated meta-data (element values).
        //
        // Since these may have dependent attribute values, we must use
        // the delete method to ensure that the necessary cleanup occurs.
        //
        // Once again, cascading deletes could be a solution.
        if ($success)
        {
            $condition = new EqualityCondition(
                new PropertyConditionVariable(
                    GroupMetadataElementValue :: class_name(),
                    GroupMetadataElementValue :: PROPERTY_GROUP_ID),
                new StaticConditionVariable($this->get_id()));

            // Must use the MetadataDataManager since every data manager has its own prefix
            $metadata_values = \core\metadata\MetadataDataManager :: get_instance()->retrieves(
                GroupMetadataElementValue :: CLASS_NAME,
                new DataClassRetrievesParameters($condition));

            while (($metadata_value = $metadata_values->next_result()) && $success)
            {
                $success = $metadata_value->delete() && $success;
            }
        }

        // Third, delete any links to a rights template
        if ($success)
        {
            $condition = new EqualityCondition(
                new PropertyConditionVariable(
                    GroupRightsTemplate :: class_name(),
                    GroupRightsTemplate :: PROPERTY_GROUP_ID),
                new StaticConditionVariable($this->get_id()));

            $success = DataManager :: deletes(GroupRightsTemplate :: class_name(), $condition) && $success;
        }

        return $success;
    }

    /**
     * Unsubscribes all users from this group
     */
    public function truncate()
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID),
            new StaticConditionVariable($this->get_id()));

        return DataManager :: deletes(GroupRelUser :: CLASS_NAME, $condition);
    }

    public function create($previous_id = 0)
    {
        $parent_id = $this->get_parent_id();

        if ($previous_id)
        {
            return parent :: create(parent :: AS_NEXT_SIBLING_OF, $previous_id);
        }
        else
        {
            return parent :: create(parent :: AS_LAST_CHILD_OF, $parent_id);
        }
    }

    public function get_rights_templates()
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(GroupRightsTemplate :: class_name(), GroupRightsTemplate :: PROPERTY_GROUP_ID),
            new StaticConditionVariable($this->get_id()));

        return DataManager :: retrieves(
            GroupRightsTemplate :: class_name(),
            new DataClassRetrievesParameters($condition));
    }

    public function add_rights_template_link($rights_template_id)
    {
        $template = new GroupRightsTemplate();
        $template->set_group_id($this->get_id());
        $template->set_rights_template_id($rights_template_id);
        return $template->create();
    }

    public function delete_rights_template_link($rights_template_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(GroupRightsTemplate :: class_name(), GroupRightsTemplate :: PROPERTY_GROUP_ID),
            new StaticConditionVariable($this->get_id()));
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                GroupRightsTemplate :: class_name(),
                GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID),
            new StaticConditionVariable($rights_template->get_id()));
        $condition = new AndCondition($conditions);

        return DataManager :: deletes(GroupRightsTemplate :: CLASS_NAME, $condition);
    }

    public function update_rights_template_links($rights_templates)
    {
        // Delete the no longer existing links
        $conditions = array();
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(GroupRightsTemplate :: class_name(), GroupRightsTemplate :: PROPERTY_GROUP_ID),
            new StaticConditionVariable($this->get_id()));
        $conditions[] = new NotCondition(
            new InCondition(
                new PropertyConditionVariable(
                    GroupRightsTemplate :: class_name(),
                    GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID),
                $rights_templates));
        $condition = new AndCondition($conditions);

        $success = DataManager :: deletes(GroupRightsTemplate :: CLASS_NAME, $condition);
        if (! $success)
        {
            return false;
        }

        // Get the group's rights_templates
        $group_rights_templates = $this->get_rights_templates();
        $existing_rights_templates = array();

        while ($group_rights_template = $group_rights_templates->next_result())
        {
            $existing_rights_templates[] = $group_rights_template->get_rights_template_id();
        }

        // Add the new links
        foreach ($rights_templates as $rights_template)
        {
            if (! in_array($rights_template, $existing_rights_templates))
            {
                if (! $this->add_rights_template_link($rights_template))
                {
                    return false;
                }
            }
        }

        return true;
    }

    public function get_users($include_subgroups = false, $recursive_subgroups = false)
    {
        if (! isset($this->users[(int) $include_subgroups][(int) $recursive_subgroups]))
        {
            $condition = $this->get_user_condition($include_subgroups, $recursive_subgroups);
            $group_rel_users = DataManager :: retrieves(GroupRelUser :: class_name(), $condition);
            $users = array();

            while ($group_rel_user = $group_rel_users->next_result())
            {
                $users[$group_rel_user->get_user_id()] = $group_rel_user->get_user_id();
            }

            $this->users[(int) $include_subgroups][(int) $recursive_subgroups] = $users;
        }

        return $this->users[(int) $include_subgroups][(int) $recursive_subgroups];
    }

    private function get_user_condition($include_subgroups = false, $recursive_subgroups = false)
    {
        $groups = array();
        $groups[] = $this->get_id();

        if ($include_subgroups)
        {
            $subgroups = $this->get_subgroups($recursive_subgroups);

            foreach ($subgroups as $subgroup)
            {
                $groups[] = $subgroup->get_id();
            }
        }

        return new InCondition(
            new PropertyConditionVariable(GroupRelUser :: class_name(), GroupReluser :: PROPERTY_GROUP_ID),
            $groups);
    }

    public function count_users($include_subgroups = false, $recursive_subgroups = false)
    {
        if (! isset($this->user_count[(int) $include_subgroups][(int) $recursive_subgroups]))
        {
            if (! $include_subgroups)
            {
                $condition = new EqualityCondition(
                    new PropertyConditionVariable(GroupRelUser :: class_name(), GroupReluser :: PROPERTY_GROUP_ID),
                    new StaticConditionVariable($this->get_id()));
                $parameters = new DataClassCountDistinctParameters($condition, GroupReluser :: PROPERTY_USER_ID);
            }
            elseif ($include_subgroups && $recursive_subgroups)
            {
                $join = new DataClassJoin(
                    Group :: class_name(),
                    new EqualityCondition(
                        new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID),
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID)));
                $joins = new Joins(array($join));

                $conditions = array();
                $conditions[] = new InequalityCondition(

                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE),
                    InequalityCondition :: GREATER_THAN_OR_EQUAL,
                    new StaticConditionVariable($this->get_left_value()));
                $conditions[] = new InequalityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE),
                    InequalityCondition :: LESS_THAN_OR_EQUAL,
                    new StaticConditionVariable($this->get_right_value()));
                $condition = new AndCondition($conditions);

                $parameters = new DataClassCountDistinctParameters($condition, GroupReluser :: PROPERTY_USER_ID, $joins);
            }
            else
            {
                $join = new DataClassJoin(
                    Group :: class_name(),
                    new EqualityCondition(
                        new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID),
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID)));
                $joins = new Joins(array($join));

                $conditions = array();
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID),
                    new StaticConditionVariable($this->get_id()));
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID),
                    new StaticConditionVariable($this->get_id()));
                $condition = new OrCondition($conditions);

                $parameters = new DataClassCountDistinctParameters($condition, GroupReluser :: PROPERTY_USER_ID, $joins);
            }

            $this->user_count[(int) $include_subgroups][(int) $recursive_subgroups] = DataManager :: count_distinct(
                GroupRelUser :: class_name(),
                $parameters);
        }

        return $this->user_count[(int) $include_subgroups][(int) $recursive_subgroups];
    }

    public function get_subgroups($recursive = false)
    {
        if (! isset($this->subgroups[(int) $recursive]))
        {
            if ($recursive)
            {
                $children_conditions = array();
                $children_conditions[] = new InequalityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE),
                    InequalityCondition :: GREATER_THAN,
                    new StaticConditionVariable($this->get_left_value()));
                $children_conditions[] = new InequalityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE),
                    InequalityCondition :: LESS_THAN,
                    new StaticConditionVariable($this->get_right_value()));
                $children_condition = new AndCondition($children_conditions);
            }
            else
            {
                $children_condition = new EqualityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID),
                    new StaticConditionVariable($this->get_id()));
            }

            $groups = DataManager :: retrieves(Group :: class_name(), $children_condition);

            $subgroups = array();

            while ($group = $groups->next_result())
            {
                $subgroups[$group->get_id()] = $group;
            }

            $this->subgroups[(int) $recursive] = $subgroups;
        }

        return $this->subgroups[(int) $recursive];
    }

    public function count_subgroups($recursive = false)
    {
        if (! isset($this->subgroup_count[(int) $recursive]))
        {
            if ($recursive)
            {
                $this->subgroup_count[(int) $recursive] = ($this->get_right_value() - $this->get_left_value() - 1) / 2;
            }
            else
            {
                $children_condition = new EqualityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID),
                    new StaticConditionVariable($this->get_id()));
                $this->subgroup_count[(int) $recursive] = DataManager :: count(
                    Group :: class_name(),
                    $children_condition);
            }
        }

        return $this->subgroup_count[(int) $recursive];
    }

    /**
     *
     * @return array
     */
    public function get_allowed_groups()
    {
        if (! isset($this->allowed_groups))
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(GroupUseGroup :: class_name(), GroupUseGroup :: PROPERTY_REQUEST_GROUP_ID),
                new StaticConditionVariable($this->get_id()));
            $condition = new AndCondition($conditions);

            $groups_result = array();
            $usable_groups = DataManager :: retrieves(GroupUseGroup :: class_name(), $condition);

            while ($usable_group = $usable_groups->next_result())
            {
                $groups_result[$usable_group->get_id()] = $usable_group->get_id();

                $subgroups = $usable_group->get_children();

                while ($subgroup = $subgroups->next_result())
                {
                    $groups_result[$subgroup->get_id()] = $subgroup->get_id();
                }
            }

            $this->allowed_groups = $groups_result;
        }

        return $this->allowed_groups;
    }
}
