<?php
namespace group\upgrade\v4_0_0;

use common\libraries\AndCondition;
use common\libraries\CommonDataManager;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\package\Action;
use common\libraries\package\Package;
use common\libraries\PropertyConditionVariable;
use common\libraries\Registration;
use common\libraries\StaticConditionVariable;
use common\libraries\Utilities;
use group\Group;
use tracking\Event;
use tracking\TrackerRegistration;
use tracking\TrackingDataManager;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\group\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {

                // Type and name seem to be required ... .
                // Length also, if we want a varchar.
                $group_group_column_attributes = array(
                    Group :: PROPERTY_NAME => array(
                        'name' => Group :: PROPERTY_NAME,
                        'type' => 'text',
                        'length' => '255',
                        'notnull' => true,
                        'default' => ''),
                    Group :: PROPERTY_DESCRIPTION => array('name' => Group :: PROPERTY_DESCRIPTION, 'type' => 'text'),
                    Group :: PROPERTY_CODE => array(
                        'name' => Group :: PROPERTY_CODE,
                        'type' => 'text',
                        'length' => '50',
                        'notnull' => true,
                        'default' => ''));

                foreach ($group_group_column_attributes as $column => $attributes)
                {
                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                        Group :: get_table_name(),
                        $column,
                        $attributes);

                    if (! $success)
                    {
                        $upgrader->add_message(
                            Action :: TYPE_ERROR,
                            "Failed to make changes to column '$column' of table " . Group :: get_table_name());
                        return false;
                    }
                    else
                    {
                        $upgrader->add_message(
                            Action :: TYPE_NORMAL,
                            "Made changes to column '$column' of table " . Group :: get_table_name());
                    }
                }

                if (! TrackingDataManager :: get_instance()->rename_storage_unit('group_changes', 'group_change'))
                {
                    return false;
                }

                // New subpackage registrations
                $subpackages = array(
                    'group\integration\tracking',
                    'group\integration\webservice'
                );

                foreach($subpackages as $subpackage)
                {
                    $package_info = Package :: get($subpackage);
                    $application_registration = new Registration();

                    $parent_namespace = Utilities :: get_namespace_parent($subpackage);
                    $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

                    $application_registration->set_name($package_info->get_code());
                    $application_registration->set_category($package_info->get_category());
                    $application_registration->set_version($package_info->get_version());
                    $application_registration->set_status(Registration :: STATUS_ACTIVE);
                    $application_registration->set_context($subpackage);

                    if(!$application_registration->create())
                    {
                        return false;
                    }
                }
                
                // Tracking updates
                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        TrackerRegistration :: class_name(),
                        TrackerRegistration :: PROPERTY_CONTEXT),
                    new StaticConditionVariable('group'));
                $tracker_registrations = TrackingDataManager :: get_instance()->retrieves(
                    TrackerRegistration :: class_name(),
                    new DataClassRetrievesParameters($condition));

                while ($tracker_registration = $tracker_registrations->next_result())
                {
                    $tracker_registration->set_context('group\integration\tracking');
                    $tracker_registration->update();
                }

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(Event :: class_name(), Event :: PROPERTY_CONTEXT),
                    new StaticConditionVariable('group'));
                $event_registrations = TrackingDataManager :: get_instance()->retrieves(
                    Event :: class_name(),
                    new DataClassRetrievesParameters($condition));

                while ($event_registration = $event_registrations->next_result())
                {
                    $event_registration->set_context('group\integration\tracking');
                    $event_registration->update();
                }

                $trackers = array(
                    'group_changes_tracker' => 'change',
                );

                foreach ($trackers as $old_tracker => $new_tracker)
                {
                    $properties = new DataClassProperties();
                    $properties->add(
                        new DataClassProperty(
                            new PropertyConditionVariable(
                                TrackerRegistration :: class_name(),
                                TrackerRegistration :: PROPERTY_TRACKER),
                            new StaticConditionVariable($new_tracker)));

                    $conditions = array();
                    $conditions[] = new EqualityCondition(
                        new PropertyConditionVariable(
                            TrackerRegistration :: class_name(),
                            TrackerRegistration :: PROPERTY_TRACKER),
                        new StaticConditionVariable($old_tracker));
                    $conditions[] = new EqualityCondition(
                        new PropertyConditionVariable(
                            TrackerRegistration :: class_name(),
                            TrackerRegistration :: PROPERTY_CONTEXT),
                        new StaticConditionVariable('group\integration\tracking'));
                    $condition = new AndCondition($conditions);

                    if (! TrackingDataManager :: get_instance()->updates(
                        TrackerRegistration :: class_name(),
                        $properties,
                        $condition))
                    {
                        return false;
                    }
                }

                $package = Package :: get('group');
                $registration = CommonDataManager :: get_registration('group');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
