<?php
namespace repository\content_object\blog;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use common\libraries\ObjectTableOrder;
use repository\ComplexContentObjectItem;
use Exception;

/**
 * Abstract class to define a blog layout so users are able to define new blog layouts and choose between them in the
 * local settings
 * 
 * @author Sven Vanpoucke
 */
abstract class BlogLayout
{

    /**
     * The parent on which this blog layout is rendering
     */
    private $parent;

    /**
     * The blog which needs to be rendered
     * 
     * @var Blog
     */
    private $blog;

    /**
     * Constructor
     * 
     * @param $parent
     * @param $blog Blog
     */
    public function __construct($parent, Blog $blog)
    {
        $this->parent = $parent;
        $this->blog = $blog;
    }

    /**
     * Factory
     * 
     * @param $parent
     * @param $blog Blog
     */
    public function factory($parent, Blog $blog)
    {
        $type = $blog->get_blog_layout();
        
        $file = dirname(__FILE__) . '/blog_layout/' . $type . '_blog_layout.class.php';
        if (! file_exists($file))
        {
            throw new Exception(Translation :: get('BlogLayoutNotExists', array('BLOGLAYOUT' => $type)));
        }
        
        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'BlogLayout';
        return new $class($parent, $blog);
    }
    
    // Getters and setters
    public function get_blog()
    {
        return $this->blog;
    }

    public function set_blog(Blog $blog)
    {
        $this->blog = $blog;
    }

    public function get_parent()
    {
        return $this->parent;
    }

    public function set_parent($parent)
    {
        $this->parent = $parent;
    }
    
    // Render methods
    public function render()
    {
        echo $this->as_html();
    }

    public function as_html()
    {
        $html = array();
        
        $complex_blog_items = $this->retrieve_complex_blog_items();
        while ($complex_blog_item = $complex_blog_items->next_result())
        {
            $html[] = $this->display_blog_item($complex_blog_item);
        }
        
        return implode("\n", $html);
    }

    /**
     * Displays a given blog item
     * 
     * @param $complex_blog_item ComplexBlogItem
     */
    abstract public function display_blog_item(ComplexBlogItem $complex_blog_item);
    
    // Helper methods
    
    /**
     * Returns the actions for the blog item
     * 
     * @param $complex_blog_item ComplexBlogItem
     */
    public function get_blog_item_actions($complex_blog_item)
    {
        $toolbar = new Toolbar();
        if ($this->get_parent()->get_parent()->is_allowed_to_edit_content_object())
        {
            $toolbar->add_item(
                    new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES), 
                            Theme :: get_common_image_path() . 'action_edit.png', 
                            $this->get_parent()->get_complex_content_object_item_update_url($complex_blog_item), 
                            ToolbarItem :: DISPLAY_ICON));
        }
        
        if ($this->get_parent()->get_parent()->is_allowed_to_delete_child())
        {
            $toolbar->add_item(
                    new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES), 
                            Theme :: get_common_image_path() . 'action_delete.png', 
                            $this->get_parent()->get_complex_content_object_item_delete_url($complex_blog_item), 
                            ToolbarItem :: DISPLAY_ICON, true));
        }
        
        return $toolbar->as_html();
    }

    /**
     * Retrieves the children of the current blog by date
     */
    public function retrieve_complex_blog_items()
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $this->get_blog()->get_id(), 
                ComplexContentObjectItem :: get_table_name());
        $parameters = new DataClassRetrievesParameters($condition, null, null, 
                array(new ObjectTableOrder(ComplexContentObjectItem :: PROPERTY_ADD_DATE, SORT_DESC)));
        return \repository\DataManager :: retrieves(ComplexContentObjectItem :: class_name(), $parameters);
    }
}
