<?php
namespace repository;

use common\libraries\StaticConditionVariable;
use common\extensions\category_manager\ProhibitGeneralCategories;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassCountParameters;
use common\libraries\EqualityCondition;
use common\libraries\Session;
use common\libraries\AndCondition;
use common\extensions\category_manager\CategoryManager;
use common\extensions\category_manager\ImpactViewSupport;
use common\libraries\OrCondition;
use common\libraries\NewObjectTableSupport;

/**
 * $Id: repository_category_manager.class.php 205 2009-11-13 12:57:33Z vanpouckesven $
 * 
 * @package repository.lib.category_manager
 */
class RepositoryCategoryManager extends CategoryManager implements ImpactViewSupport, NewObjectTableSupport,
    ProhibitGeneralCategories
{

    /**
     * **************************************************************************************************************
     * Properties *
     * **************************************************************************************************************
     */
    
    /**
     * The type of this category manager
     * 
     * @var int
     */
    private $type;

    /**
     * The impact view table condition
     * 
     * @var Condition
     */
    private $impact_view_table_condition;

    /**
     * **************************************************************************************************************
     * Constructor *
     * **************************************************************************************************************
     */
    public function __construct($parent, $trail, $type = null)
    {
        parent :: __construct($parent, $trail);
        
        if (is_null($type))
        {
            $type = RepositoryCategory :: TYPE_NORMAL;
        }
        
        $this->set_type($type);
    }

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the category object
     * 
     * @return RepositoryCategory
     */
    public function get_category()
    {
        $category = new RepositoryCategory();
        $category->set_type($this->get_type());
        
        return $category;
    }

    /**
     * Counts the categories for a given condition
     * 
     * @param Condition $condition
     *
     * @return int
     */
    public function count_categories($condition)
    {
        if ($condition)
        {
            $conditions[] = $condition;
        }
        
        $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(RepositoryCategory :: class_name(), RepositoryCategory :: PROPERTY_USER_ID), 
                new StaticConditionVariable($this->get_user_id()));
        $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(RepositoryCategory :: class_name(), RepositoryCategory :: PROPERTY_TYPE), 
                new StaticConditionVariable($this->get_type()));
        
        $condition = new AndCondition($conditions);
        $parameters = new DataClassCountParameters($condition);
        
        return DataManager :: count(RepositoryCategory :: class_name(), $parameters);
    }

    /**
     * Retrieves the categories for a given condition
     * 
     * @param Condition $condition
     * @param int $offset
     * @param int $count
     * @param int $order_property
     *
     * @return ResultSet<RepositoryCategory>
     */
    public function retrieve_categories($condition, $offset, $count, $order_property)
    {
        if ($condition)
        {
            $conditions[] = $condition;
        }
        
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_USER_ID, $this->get_user_id());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_TYPE, $this->get_type());
        
        $condition = new AndCondition($conditions);
        
        return DataManager :: retrieve_categories($condition, $offset, $count, $order_property);
    }

    /**
     * Returns the next display order for a given parent
     * 
     * @param int $parent_id
     *
     * @return int
     */
    public function get_next_category_display_order($parent_id)
    {
        return DataManager :: select_next_category_display_order($parent_id, Session :: get_user_id(), 
                $this->get_type());
    }

    /**
     * Returns whether or not the user can delete a category
     * 
     * @param int $category_id
     *
     * @return boolean
     */
    public function allowed_to_delete_category($category_id)
    {
        $object_count = $this->count_category_objects($category_id);
        if ($object_count > 0)
        {
            return false;
        }
        
        return ! $this->contain_subcategories_objects($category_id);
    }

    /**
     * Returns the condition for a table
     * 
     * @param string $class_name
     *
     * @return Condition
     */
    public function get_object_table_condition($class_name)
    {
        return $this->impact_view_table_condition;
    }

    /**
     * Renders the impact view
     * 
     * @param int[] $selected_category_ids - [OPTIONAL] default: array
     * @return string
     */
    public function render_impact_view($selected_category_ids = array())
    {
        $conditions = array();
        foreach ($selected_category_ids as $selected_category_id)
        {
            $this->get_categories_condition($selected_category_id, $conditions);
        }
        
        $condition = new AndCondition(new OrCondition($conditions), 
                new EqualityCondition(ContentObject :: PROPERTY_STATE, ContentObject :: STATE_NORMAL));
        
        $this->impact_view_table_condition = $condition;
        $impact_view_table = new ImpactViewTable($this);
        
        return $impact_view_table->as_html();
    }

    public function has_impact($selected_category_ids = array())
    {
        $conditions = array();
        foreach ($selected_category_ids as $selected_category_id)
        {
            $this->get_categories_condition($selected_category_id, $conditions);
        }
        
        $condition = new AndCondition(new OrCondition($conditions), 
                new EqualityCondition(
                        new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_STATE), 
                        new StaticConditionVariable(ContentObject :: STATE_NORMAL)));
        
        $parameters = new DataClassCountParameters($condition);
        
        return DataManager :: count_active_content_objects(ContentObject :: class_name(), $parameters) > 0;
    }

    /**
     * **************************************************************************************************************
     * Helper Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Helper function that recursivly builds the categories condition
     * 
     * @param int $category_id
     * @param Condition[] $conditions
     */
    private function get_categories_condition($category_id, &$conditions = array())
    {
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_PARENT_ID, $category_id);
        
        // retrieve children
        $retrieve_children_condition = new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $category_id);
        $child_categories = DataManager :: retrieve_categories($retrieve_children_condition);
        
        while ($child_category = $child_categories->next_result())
        {
            $this->get_categories_condition($child_category->get_id(), $conditions);
        }
    }

    /**
     * Returns the url to the content object preview
     * 
     * @param ContentObject $content_object
     *
     * @return string
     */
    public function get_content_object_preview_url($content_object)
    {
        return $this->get_url(
                array(RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_VIEW_CONTENT_OBJECTS, 
                        RepositoryManager :: PARAM_CONTENT_OBJECT_ID => $content_object->get_id(), 
                        RepositoryManager :: PARAM_CATEGORY_ID => $content_object->get_parent_id()));
    }

    /**
     * Count the objects in a category
     * 
     * @param int $category_id
     *
     * @return int
     */
    public function count_category_objects($category_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_PARENT_ID), 
                new StaticConditionVariable($category_id));
        $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_STATE), 
                new StaticConditionVariable(ContentObject :: STATE_NORMAL));
        $condition = new AndCondition($conditions);
        
        $parameters = new DataClassCountParameters($condition);
        return DataManager :: count_active_content_objects(ContentObject :: class_name(), $parameters);
    }

    /**
     * Checks if a category has subcategories with objects in
     * 
     * @param int $category_id
     *
     * @return boolean
     */
    private function contain_subcategories_objects($category_id)
    {
        $condition = new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $category_id);
        $subcategries = DataManager :: retrieve_categories($condition);
        
        while ($cat = $subcategries->next_result())
        {
            $count = $this->count_category_objects($cat->get_id());
            if ($count > 0 || $this->contain_subcategories_objects($cat->get_id()))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * **************************************************************************************************************
     * Getters & Setters Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the type
     * 
     * @return int
     */
    public function get_type()
    {
        return $this->type;
    }

    /**
     * Sets the type
     * 
     * @param int $type
     */
    public function set_type($type)
    {
        $this->type = $type;
    }
}
