<?php
namespace repository;

use common\libraries\GenericTree;
use common\libraries\Request;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Path;
use common\libraries\ObjectTableOrder;
use common\libraries\Translation;

/**
 * This class provides a navigation menu to allow a user to browse through repository categories
 * 
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class RepositoryCategoryTreeMenu extends GenericTree
{
    const ROOT_NODE_CLASS = 'category';
    const CATEGORY_CLASS = 'category';

    private $parent;

    private $additional_items;

    /**
     * Creates a new category navigation menu.
     * 
     * @param $parent - the parent component
     * @param array $additional_items An array of extra tree items, added to the root.
     */
    public function __construct($parent, $additional_items = array())
    {
        $this->parent = $parent;
        $this->additional_items = $additional_items;
        
        parent :: __construct();
    }

    /**
     * Builds the tree and adds additional items
     */
    public function build_tree()
    {
        parent :: build_tree();
        
        foreach ($this->additional_items as $additional_item)
        {
            $this->tree[] = $additional_item;
        }
    }

    /**
     * Returns the url of a node
     * 
     * @param int $node_id
     *
     * @return string
     */
    public function get_node_url($node_id)
    {
        $url_param[RepositoryManager :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS;
        $url_param[RepositoryManager :: PARAM_CATEGORY_ID] = null;
        
        return $this->parent->get_url($url_param) . '&' . RepositoryManager :: PARAM_CATEGORY_ID . '=' . $node_id;
    }

    public function get_current_node_id()
    {
        return intval(Request :: get(RepositoryManager :: PARAM_CATEGORY_ID));
    }

    public function get_node($node_id)
    {
        $condition = new EqualityCondition(RepositoryCategory :: PROPERTY_ID, $node_id);
        $child = DataManager :: retrieve_categories($condition)->next_result();
        return $child;
    }

    public function get_node_children($parent_node_id)
    {
        return DataManager :: retrieve_categories($this->get_retrieve_condition($parent_node_id), null, null, 
                new ObjectTableOrder(RepositoryCategory :: PROPERTY_DISPLAY_ORDER));
    }

    public function node_has_children($parent_node_id)
    {
        return (DataManager :: count(RepositoryCategory :: class_name(), $this->get_retrieve_condition($parent_node_id)) > 0);
    }

    public function get_search_url()
    {
        return Path :: get(WEB_PATH) . 'repository/php/xml_feeds/xml_repository_category_menu_feed.php';
    }

    public function get_url_format()
    {
        return $this->get_node_url('%d');
    }

    public function get_root_node_class()
    {
        return self :: ROOT_NODE_CLASS;
    }

    public function get_node_class($node)
    {
        return self :: CATEGORY_CLASS;
    }

    public function get_root_node_title()
    {
        return Translation :: get('MyRepository');
    }

    public function get_node_title($node)
    {
        return $node->get_name();
    }

    public function get_node_id($node)
    {
        return $node->get_id();
    }

    public function get_node_parent($node)
    {
        return $node->get_parent();
    }

    public function get_breadcrumbs()
    {
        return null;
    }

    public function get_parent()
    {
        return $this->parent;
    }

    /**
     * **************************************************************************************************************
     * Helper functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the retrieve condition
     * 
     * @param int $parent_node_id
     *
     * @return Condition
     */
    protected function get_retrieve_condition($parent_node_id)
    {
        $conditions = array();
        
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $parent_node_id);
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_USER_ID, $this->parent->get_user_id());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_TYPE, $this->get_type());
        
        return new AndCondition($conditions);
    }

    /**
     * Returns the category type
     * 
     * @return int
     */
    protected function get_type()
    {
        return RepositoryCategory :: TYPE_NORMAL;
    }
}
