<?php
namespace repository;

use common\libraries\ClassNotExistException;
use common\libraries\CompositeDataClass;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\StringUtilities;
use common\libraries\DisplayOrderDataClassListener;
use common\libraries\DisplayOrderDataClassListenerSupport;
use common\libraries\PropertyConditionVariable;

/**
 * $Id: complex_content_object_item.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib
 */
/**
 * Instances of this class group generic information about a complex object item
 *
 * @author Sven Vanpoucke
 */
class ComplexContentObjectItem extends CompositeDataClass implements DisplayOrderDataClassListenerSupport
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_REF = 'ref_id';
    const PROPERTY_PARENT = 'parent_id';
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_DISPLAY_ORDER = 'display_order';
    const PROPERTY_ADD_DATE = 'add_date';

    /**
     *
     * @var ContentObject
     */
    private $reference_object;

    public function __construct($default_properties = array(), $additional_properties = null)
    {
        parent :: __construct($default_properties, $additional_properties);
        $this->add_listener(new DisplayOrderDataClassListener($this));
    }

    /**
     * Get the default property names
     *
     * @return array The property names.
     */
    public static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(
            array(
                self :: PROPERTY_REF,
                self :: PROPERTY_PARENT,
                self :: PROPERTY_USER_ID,
                self :: PROPERTY_DISPLAY_ORDER,
                self :: PROPERTY_ADD_DATE));
    }

    public function get_add_date()
    {
        return $this->get_default_property(self :: PROPERTY_ADD_DATE);
    }

    public function set_add_date($add_date)
    {
        $this->set_default_property(self :: PROPERTY_ADD_DATE, $add_date);
    }

    public function get_ref()
    {
        return $this->get_default_property(self :: PROPERTY_REF);
    }

    public function get_ref_object()
    {
        if (! isset($this->reference_object))
        {
            $this->reference_object = \repository\DataManager :: retrieve_content_object($this->get_ref());
        }
        return $this->reference_object;
    }

    public function set_ref($ref)
    {
        $this->set_default_property(self :: PROPERTY_REF, $ref);
    }

    /**
     *
     * @param ContentObject $reference_object
     */
    public function set_ref_object(ContentObject $reference_object)
    {
        $this->reference_object = $reference_object;
    }

    public function get_parent()
    {
        return $this->get_default_property(self :: PROPERTY_PARENT);
    }

    public function get_parent_object()
    {
        return \repository\DataManager :: retrieve_content_object($this->get_parent());
    }

    public function set_parent($parent)
    {
        $this->set_default_property(self :: PROPERTY_PARENT, $parent);
    }

    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    public function get_display_order()
    {
        return $this->get_default_property(self :: PROPERTY_DISPLAY_ORDER);
    }

    public function set_display_order($display_order)
    {
        $this->set_default_property(self :: PROPERTY_DISPLAY_ORDER, $display_order);
    }

    /**
     * Checks this object before saving + adds some default values
     *
     * @return boolean
     */
    public function check_before_save()
    {
        if (! $this->get_add_date())
        {
            $this->set_add_date(time());
        }

        if (! $this->get_display_order())
        {
            $this->set_display_order(DataManager :: select_next_display_order($this->get_parent()));
        }

        if (StringUtilities :: is_null_or_empty($this->get_ref()))
        {
            $this->add_error(Translation :: get('ReferenceObjectShouldNotBeEmpty'));
        }
        else
        {
            $ref_content_object = DataManager :: retrieve_content_object($this->get_ref());
            if (! $ref_content_object)
            {
                $this->add_error(Translation :: get('ReferenceObjectDoesNotExist'));
            }
        }

        if (StringUtilities :: is_null_or_empty($this->get_parent()))
        {
            $this->add_error(Translation :: get('ReferenceObjectShouldNotBeEmpty'));
        }
        else
        {
            $parent_content_object = DataManager :: retrieve_content_object($this->get_parent());
            if (! $parent_content_object)
            {
                $this->add_error(Translation :: get('ParentObjectDoesNotExist'));
            }
        }

        return ! $this->has_errors();
    }

    /**
     * Retrieves the allowed types to add to this complex learning object item
     *
     * @return Array of learning object types
     */
    public function get_allowed_types()
    {
        return array();
    }

    public function is_complex()
    {
        return count($this->get_allowed_types()) > 0;
    }

    public function has_right($right, $user_id = null)
    {
        $ref_object = $this->get_ref_object();
        if (! $ref_object)
        {
            // if we cannot fetch the reference object, surely we have no rights to do anything with it...
            return false;
        }
        return $ref_object->has_right($right, $user_id);
    }

    public static function factory($class, &$record)
    {
        if (is_subclass_of($class, ComplexContentObjectItem :: class_name()))
        {
            return parent :: factory($class, &$record);
        }
        elseif (is_subclass_of($class, ContentObject :: class_name()))
        {
            $class = Utilities :: get_namespace_from_classname($class) . '\Complex' . Utilities :: get_classname_from_namespace(
                $class);

            return parent :: factory($class, &$record);
        }
        else
        {
            throw new ClassNotExistException($class);
        }
    }

    /**
     * Returns the property for the display order
     *
     * @return string
     */
    public function get_display_order_property()
    {
        return new PropertyConditionVariable(ComplexContentObjectItem :: class_name(), self :: PROPERTY_DISPLAY_ORDER);
    }

    /**
     * Returns the properties that define the context for the display order (the properties on which has to be limited)
     *
     * @return Condition
     */
    public function get_display_order_context_properties()
    {
        return array(new PropertyConditionVariable(ComplexContentObjectItem :: class_name(), self :: PROPERTY_PARENT));
    }
}
