<?php
namespace repository;

use common\libraries\ToolbarItem;

use common\libraries\Versionable;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\ObjectTableCellRenderer;
use common\libraries\Theme;
use common\libraries\DatetimeUtilities;

/**
 * $Id: default_content_object_table_cell_renderer.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.content_object_table
 */

/**
 * This is the default cell renderer, used when a ContentObjectTable does not provide its own renderer. The default
 * renderer provides a custom rendering method for the following columns: - The ID of the object Displays the ID. - The
 * type of the object Displays the icon that corresponds to the object type. - The title of the object Displays the
 * title. - The description of the object Strips HTML tags from the description of the object and displays the first 200
 * characters of the resulting string. - The date when the object was created Displays a localized version of the date.
 * - The date when the object was last modified Displays a localized version of the date. Any other column type will
 * result in an empty cell.
 *
 * @see ContentObjectTable
 * @see ContentObjectTableCellRenderer
 * @see DefaultContentObjectTableColumnModel
 * @author Tim De Pauw
 */
class DefaultContentObjectTableCellRenderer extends ObjectTableCellRenderer
{

    /**
     * Constructor
     */
    public function __construct()
    {
    }

    /**
     * Renders a table cell
     *
     * @param ContentObjectTableColumnModel $column The column which should be rendered
     * @param Learning Object $content_object The learning object to render
     * @return string A HTML representation of the rendered table cell
     */
    public function render_cell($column, $content_object)
    {
        switch ($column->get_name())
        {
            case ContentObject :: PROPERTY_ID :
                return $content_object->get_id();
            case ContentObject :: PROPERTY_TYPE :
                return $content_object->get_icon_image(Theme :: ICON_MINI);

            case Theme :: get_common_image('action_category', 'png', Translation :: get('Type'), null,
                    ToolbarItem :: DISPLAY_ICON) :
                return $content_object->get_icon_image(Theme :: ICON_MINI);

            case ContentObject :: PROPERTY_TITLE :
                return Utilities :: truncate_string($content_object->get_title(), 50);
            case ContentObject :: PROPERTY_DESCRIPTION :
                return Utilities :: htmlentities(Utilities :: truncate_string($content_object->get_description(), 50));
            case ContentObject :: PROPERTY_CREATION_DATE :
                return DatetimeUtilities :: format_locale_date(
                        Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES),
                        $content_object->get_creation_date());
            case ContentObject :: PROPERTY_MODIFICATION_DATE :
                return DatetimeUtilities :: format_locale_date(
                        Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES),
                        $content_object->get_modification_date());
            case ContentObject :: get_version_header() :
                if ($content_object instanceof Versionable)
                {
                    if ($content_object->has_versions())
                    {
                        $number = $content_object->get_version_count();
                        return '<img src="' . Theme :: get_image_path() . 'versions_multiple.png" alt="' . Translation :: get(
                                'VersionsAvailable', array('NUMBER' => $number)) . '" title="' . Translation :: get(
                                'VersionsAvailable', array('NUMBER' => $number)) . '" />';
                    }
                    else
                    {
                        return '<img src="' . Theme :: get_image_path() . 'versions_none.png" alt="' . Translation :: get(
                                'NoVersionsAvailable') . '" title="' . Translation :: get('NoVersionsAvailable') . '" />';
                    }
                }
                else
                {
                    return '<img src="' . Theme :: get_image_path() . 'versions_none.png" alt="' . Translation :: get(
                            'NotVersionable') . '" title="' . Translation :: get('NotVersionable') . '" />';
                }
            default :
                return '&nbsp;';
        }
    }

    public function render_id_cell($object)
    {
        return $object->get_id();
    }
}
