<?php
namespace repository\viewer;

use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassCountParameters;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\ObjectTableDataProvider;
use common\libraries\EqualityCondition;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\NotCondition;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\Request;
use repository\ContentObject;
use repository\SharedContentObjectRelCategory;
use repository\RepositoryManager;
use repository\RepositoryRights;
use rights\UserEntity;
use rights\PlatformGroupEntity;

/**
 * This class represents a data provider for a publication candidate table
 */
class ContentObjectTableDataProvider extends ObjectTableDataProvider
{

    /**
     * The user id of the current active user.
     */
    private $owner;

    /**
     * The possible types of learning objects which can be selected.
     */
    private $types;

    /**
     * The search query, or null if none.
     */
    private $query;

    private $parent;

    /**
     * Constructor.
     *
     * @param int $owner The user id of the current active user.
     * @param array $types The possible types of learning objects which can be selected.
     * @param string $query The search query.
     */
    public function __construct($owner, $types, $query = null, $parent)
    {
        $this->set_types($types);
        $this->set_owner($owner);
        $this->set_query($query);
        $this->set_parent($parent);
    }

    /*
     * Inherited
     */
    public function get_objects($offset, $count, $order_property = null)
    {
        $order_property = $this->get_order_property($order_property);

        if (! $this->get_parent()->is_shared_object_browser())
        {
            $parameters = new DataClassRetrievesParameters($this->get_condition(), $count, $offset, $order_property);
            return \repository\DataManager :: retrieve_active_content_objects(
                ContentObject :: class_name(),
                $parameters);
        }
        else
        {
            return \repository\DataManager :: retrieve_shared_content_objects(
                $this->get_condition(),
                $offset,
                $count,
                $order_property);
        }
    }

    /*
     * Inherited
     */
    public function get_object_count()
    {
        if (! $this->get_parent()->is_shared_object_browser())
        {
            $parameters = new DataClassCountParameters($this->get_condition());
            return \repository\DataManager :: count_active_content_objects(ContentObject :: class_name(), $parameters);
        }
        else
        {
            return \repository\DataManager :: count_shared_content_objects($this->get_condition());
        }
    }

    /**
     * Gets the condition by which the learning objects should be selected.
     *
     * @return Condition The condition.
     */
    public function get_condition()
    {
        $owner = $this->get_owner();

        $conditions = array();
        $type_conditions = array();

        $types = $this->get_types();

        foreach ($types as $type)
        {
            $type_conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TYPE),
                new StaticConditionVariable($type));
        }

        $conditions[] = new OrCondition($type_conditions);

        if ($this->get_type_conditions())
        {
            $conditions[] = $this->get_type_conditions();
        }

        $query = $this->get_query();

        if (isset($query) && $query != '')
        {
            $or_conditions[] = new PatternMatchCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TITLE),
                '*' . $query . '*',
                ContentObject :: get_table_name());
            $or_conditions[] = new PatternMatchCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_DESCRIPTION),
                '*' . $query . '*',
                ContentObject :: get_table_name());
            $conditions[] = new OrCondition($or_conditions);
        }

        if (! $this->get_parent()->is_shared_object_browser())
        {
            if (! isset($query) || $query == '')
            {
                $category = Request :: get('category');
                $category = $category ? $category : 0;
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_PARENT_ID),
                    new StaticConditionVariable($category));
            }

            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_OWNER_ID),
                new StaticConditionVariable($owner->get_id()));
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_STATE),
                new StaticConditionVariable(ContentObject :: STATE_NORMAL));

            foreach ($this->get_parent()->get_excluded_objects() as $excluded)
            {
                $conditions[] = new NotCondition(
                    new EqualityCondition(
                        new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
                        new StaticConditionVariable($excluded)));
            }
        }
        else
        {
            $share_category = Request :: get(RepositoryManager :: PARAM_SHARED_CATEGORY_ID);
            if ($share_category)
            {
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        SharedContentObjectRelCategory :: class_name(),
                        SharedContentObjectRelCategory :: PROPERTY_CATEGORY_ID),
                    new StaticConditionVariable($share_category));
            }

            $entities = array();
            $entities[] = new UserEntity();
            $entities[] = new PlatformGroupEntity();

            $retrieve_types = array(RepositoryRights :: TYPE_USER_CONTENT_OBJECT);

            $shared_locations_by_type = RepositoryRights :: get_instance()->get_location_overview_with_rights_granted(
                RepositoryManager :: context(),
                $this->get_owner()->get_id(),
                $entities,
                null,
                $retrieve_types,
                RepositoryRights :: TREE_TYPE_USER);

            if (empty($shared_locations_by_type))
            {
                $shared_locations_by_type[] = - 1;
            }

            $conditions[] = new InCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
                array_unique($shared_locations_by_type[RepositoryRights :: TYPE_USER_CONTENT_OBJECT]));
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_STATE),
                new StaticConditionVariable(ContentObject :: STATE_NORMAL));
        }

        return new AndCondition($conditions);
    }

    protected function set_types($types)
    {
        $this->types = $types;
    }

    protected function set_owner($owner)
    {
        $this->owner = $owner;
    }

    protected function set_query($query)
    {
        $this->query = $query;
    }

    protected function set_parent($parent)
    {
        $this->parent = $parent;
    }

    protected function get_types()
    {
        return $this->types;
    }

    protected function get_owner()
    {
        return $this->owner;
    }

    protected function get_query()
    {
        return $this->query;
    }

    protected function get_parent()
    {
        return $this->parent;
    }

    protected function get_type_conditions()
    {
        return null;
    }
}
