<?php
namespace rights;

use common\libraries\EqualityCondition;
use common\libraries\InEqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Mdb2ConditionTranslator;
use common\libraries\Mdb2Database;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;

/**
 * $Id: database_rights_data_manager.class.php 235 2009-11-16 12:08:00Z scaramanga $
 *
 * @package rights.lib.data_manager
 */

/**
 * This is a data manager that uses a database for storage. It was written for MySQL, but should be compatible with most
 * SQL flavors.
 *
 * @author Tim De Pauw
 * @author Bart Mollet
 * @author Hans De Bisschop
 */
class Mdb2DataManager extends Mdb2Database implements RightsDataManagerInterface
{

    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('rights_');
    }

    // Inherited.
    // NOK: plenty of callers that should use $location->create()
    public function create_location($location)
    {
        $this->set_prefix('rights_');
        return $this->create($location);
    }

    private $granted_rights_cache;

    /**
     * Retrieves the granted rights for a location
     *
     * @param $location <type>
     * @param $enities_condition <type>
     * @return <type>
     */
    public function retrieve_granted_rights_array($location, $entities_condition)
    {
        $hash = (is_object($entities_condition) ? spl_object_hash($entities_condition) : md5($entities_condition));

        if (is_null($this->granted_rights_cache[$location->get_context()][$location->get_id()][$hash]))
        {
            //$this->set_prefix($location->get_context() . '_');

            // first retrieve the parent lowest in the tree before a parent that doesnt inherit.
            // This way we dont check parent from who the location doesn't inherit
            // this is done in a separate query instead of a subquery for performance reasons
            // (a correlated subquery is executed for every row in the outer query)

            $data_manager_class = $location->get_context() . '\DataManager';
            $rights_location_class = $location->get_context() . '\RightsLocation';

            $data_manager = $data_manager_class :: get_instance();

            $rights_location_entity_right_alias = $data_manager->get_alias(RightsLocationEntityRight :: get_table_name());
            $rights_location_alias = $data_manager->get_alias(RightsLocation :: get_table_name());

            $query = 'SELECT ' . $rights_location_entity_right_alias . '.' . RightsLocationEntityRight :: PROPERTY_RIGHT_ID;

            $query .= ' FROM ' . $data_manager->escape_table_name(RightsLocation :: get_table_name()) . ' as ' . $rights_location_alias;
            $query .= ' JOIN ' . $data_manager->escape_table_name(RightsLocationEntityRight :: get_table_name()) . ' as ' . $rights_location_entity_right_alias;
            $query .= ' ON ' . $data_manager->escape_column_name(RightsLocation :: PROPERTY_ID, $rights_location_alias) . '=' . $data_manager->escape_column_name(RightsLocationEntityRight :: PROPERTY_LOCATION_ID, $rights_location_entity_right_alias);

            $condition = new EqualityCondition(
                new PropertyConditionVariable($rights_location_class, RightsLocation :: PROPERTY_ID),
                new StaticConditionVariable($location->get_id())
            );

            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition);

            if (! is_string($entities_condition))
            {
                $entities_condition = Mdb2ConditionTranslator :: render($entities_condition, false);
            }
            $query .= ' AND ' . $entities_condition;

            $result_set = $this->retrieve_result($query, RightsLocation :: get_table_name());

            $granted_rights = array();

            while ($result = $result_set->fetchRow(MDB2_FETCHMODE_ASSOC))
            {
                $granted_rights[] = $result[RightsLocationEntityRight :: PROPERTY_RIGHT_ID];
            }

            if ($location->inherits())
            {
                $parent_location = DataManager :: retrieve_rights_location_by_id($location->get_context(), $location->get_parent_id());
                if ($parent_location)
                {
                    $parent_rights = $this->retrieve_granted_rights_array($parent_location, $entities_condition);
                    $granted_rights = array_merge($granted_rights, $parent_rights);
                }
            }
            $this->granted_rights_cache[$location->get_context()][$location->get_id()][$hash] = $granted_rights;
        }
        return $this->granted_rights_cache[$location->get_context()][$location->get_id()][$hash];
    }
    

    /**
     * @deprecated in principle one should use $location->delete() instead
     */
  
    public function delete_location_nodes($location) {
        $conditions = array();
        $conditions[] = new EqualityCondition(Location :: PROPERTY_APPLICATION, $location->get_application());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_TYPE, $location->get_tree_type());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_IDENTIFIER, $location->get_tree_identifier());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN_OR_EQUAL, $location->get_left_value());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN_OR_EQUAL, $location->get_right_value());
        $condition = new AndCondition($conditions);

        return $this->delete_objects(Location :: get_table_name(), $condition);
    }

    /**
     * @deprecated Only used by the ad hoc implementation of Location which is deprecated in favor of delegating to RightsLocation
     */
    public function delete_nested_values($location) {
        $delta = $location->get_right_value() - $location->get_left_value() + 1;

        // Update all necessary nested-values
        $conditions = array();
        $conditions[] = new EqualityCondition(Location :: PROPERTY_APPLICATION, $location->get_application());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_TYPE, $location->get_tree_type());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_IDENTIFIER, $location->get_tree_identifier());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $location->get_left_value());
        $condition = new AndCondition($conditions);

        $properties = array();
        $properties[Location :: PROPERTY_LEFT_VALUE] = parent :: escape_column_name(Location :: PROPERTY_LEFT_VALUE) . ' - ' . $this->quote($delta);
        $properties[Location :: PROPERTY_RIGHT_VALUE] = parent :: escape_column_name(Location :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta);
        $res = $this->update_objects(Location :: get_table_name(), $properties, $condition);

        if (!$res) {
            return false;
        }

        // Update some more nested-values
        $conditions = array();
        $conditions[] = new EqualityCondition(Location :: PROPERTY_APPLICATION, $location->get_application());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_TYPE, $location->get_tree_type());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_IDENTIFIER, $location->get_tree_identifier());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN, $location->get_left_value());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $location->get_right_value());
        $condition = new AndCondition($conditions);

        $properties = array(
            Location :: PROPERTY_RIGHT_VALUE => parent :: escape_column_name(Location :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta));
        $res = $this->update_objects(Location :: get_table_name(), $properties, $condition);

        if (!$res) {
            return false;
        }

        return true;
    }
    
    
     /**
     * @deprecated Only used by the ad hoc implementation of Location which is deprecated in favor of delegating to RightsLocation
     */
     public function add_nested_values($location, $previous_visited, $number_of_elements = 1) {
        // Update all necessary left-values
        $conditions = array();
        $conditions[] = new EqualityCondition(Location :: PROPERTY_APPLICATION, $location->get_application());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_TYPE, $location->get_tree_type());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_IDENTIFIER, $location->get_tree_identifier());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $previous_visited);
        $condition = new AndCondition($conditions);

        $properties = array(
            Location :: PROPERTY_LEFT_VALUE => parent :: escape_column_name(Location :: PROPERTY_LEFT_VALUE) . ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects(Location :: get_table_name(), $properties, $condition);

        if (!$res) {
            return false;
        }
        // Update all necessary right-values
        $conditions = array();
        $conditions[] = new EqualityCondition(Location :: PROPERTY_APPLICATION, $location->get_application());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_TYPE, $location->get_tree_type());
        $conditions[] = new EqualityCondition(Location :: PROPERTY_TREE_IDENTIFIER, $location->get_tree_identifier());
        $conditions[] = new InequalityCondition(Location :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $previous_visited);
        $condition = new AndCondition($conditions);
        $properties = array(Location :: PROPERTY_RIGHT_VALUE => parent :: escape_column_name(Location :: PROPERTY_RIGHT_VALUE) . ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects(Location :: get_table_name(), $properties, $condition);
        if (!$res) {
            return false;
        }

        return true;
    }
}
