<?php
if (isset($_SERVER['argv']) && isset($_SERVER['argv'][1]))
{
    $package = $_SERVER['argv'][1];
}
else
{
    $package = null;
}

$dependency_scanner = new PackageDependencyScanner($package);
$dependency_scanner->run();

/**
 * Scans a given package and / or all the packages for dependencies
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 * @author Hans De Bisschop - Erasmushogeschool Brussel
 */
class PackageDependencyScanner
{

    /**
     * The package which needs to be scanned
     *
     * @var string
     */
    private $package;

    /**
     * The root directory of Chamilo
     *
     * @var string
     */
    private $root_directory;

    /**
     * Different search modes for files to scan in a package
     */
    const MODE_GENERAL = '/^(?(?!.hg|test).)+\.class\.php$/i';
    const MODE_INSTALL = '/^(?(?!.hg|test).)+\\installer\.class\.php$/i';

    /**
     * Constructor
     *
     * @param string $package
     */
    public function __construct($package = null)
    {
        $this->package = $package;
        $this->root_directory = $root_directory = realpath(
            __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR);
    }

    /**
     * Runs this script
     */
    public function run()
    {
        $packages = $this->get_packages();

        $dependencies_file = __DIR__ . '/files/dependencies.cache';

        if (! file_exists($dependencies_file))
        {
            $used_packages = array();

            foreach ($packages as $package)
            {
                $namespace = str_replace('/', '\\', $package);
                $used_packages[$namespace][self :: MODE_GENERAL] = $this->scan_classes($package, self :: MODE_GENERAL);
                $used_packages[$namespace][self :: MODE_INSTALL] = $this->scan_classes($package, self :: MODE_INSTALL);
            }

            $create_file = fopen($dependencies_file, 'w');
            fwrite($create_file, serialize($used_packages));
            fclose($create_file);
        }
        else
        {
            $used_packages = unserialize(file_get_contents($dependencies_file));
        }

        $this->write_used_packages_to_file($used_packages);
    }

    /**
     * Returns the list of scannable packages
     *
     * @throws Exception
     *
     * @return string[]
     */
    protected function get_packages()
    {
        $packages = array();

        $package_folders_iterator = new \RegexIterator(
            new \RecursiveIteratorIterator(
                new \RecursiveDirectoryIterator($this->root_directory),
                RecursiveIteratorIterator :: SELF_FIRST),
            '/^((?!.hg|test).)+package\.info$/',
            \RegexIterator :: GET_MATCH);

        foreach ($package_folders_iterator as $package_folder)
        {
            $package_folder = $package_folder[0];
            $package_folder = str_replace($this->root_directory . DIRECTORY_SEPARATOR, '', $package_folder);
            $package_folder = str_replace(
                DIRECTORY_SEPARATOR . 'php' . DIRECTORY_SEPARATOR . 'package' . DIRECTORY_SEPARATOR . 'package.info',
                '',
                $package_folder);
            $packages[] = str_replace(DIRECTORY_SEPARATOR, '/', $package_folder);
        }

        return $packages;
    }

    /**
     * Scans all the classes of a given package
     *
     * @param string $package
     *
     * @return string[]
     */
    protected function scan_classes($package, $mode = self :: MODE_GENERAL)
    {
        $classes = new \RegexIterator(
            new \RecursiveIteratorIterator(
                new \RecursiveDirectoryIterator(
                    $this->root_directory . DIRECTORY_SEPARATOR . $package . DIRECTORY_SEPARATOR . 'php' .
                         DIRECTORY_SEPARATOR)),
            $mode,
            \RegexIterator :: GET_MATCH);

        $used_packages = array();

        foreach ($classes as $class)
        {
            $contents = file($class[0]);

            foreach ($contents as $content)
            {
                $this->scan_use_statements($package, $content, $used_packages);
                $this->scan_inline_namespaces($package, $content, $used_packages);
            }
        }

        $unique_values = array_unique($used_packages);
        sort($unique_values);

        return $unique_values;
    }

    /**
     * Scans the content for use statements
     *
     * @param string $content
     * @param string[] $used_packages
     */
    protected function scan_use_statements($package, $content, &$used_packages = array())
    {
        $regex = '/use ([\w\d\\\\]+);/';
        $this->scan_for_used_packages_with_regex($package, $content, $regex, $used_packages);
    }

    /**
     * Scans the content for inline namespaces
     *
     * @param string $content
     * @param string[] $used_packages
     */
    protected function scan_inline_namespaces($package, $content, &$used_packages)
    {
        $regex = '/[\s\(]\\\([\d\w\\\]+\\\[\d\w]+)[\s\),]/';
        $this->scan_for_used_packages_with_regex($package, $content, $regex, $used_packages);
    }

    /**
     * Scans for used packages with a given regex, uses
     *
     * @param string $content
     * @param string $regex
     * @param string[] $used_packages
     */
    protected function scan_for_used_packages_with_regex($package, $content, $regex, &$used_packages)
    {
        $namespace = str_replace('/', '\\', $package);
        // Try to detect whether it's a comment
        $comment_content = trim($content);
        $is_single_line_comment = substr($comment_content, 0, 2) == '//';
        $is_multi_line_comment_start = substr($comment_content, 0, 2) == '/*';
        $is_multi_line_comment_body = substr($comment_content, 0, 2) == '* ';

        if (! $is_single_line_comment && ! $is_multi_line_comment_start && ! $is_multi_line_comment_body)
        {
            $matches = array();

            $result = preg_match_all($regex, $content, $matches);

            if ($result > 0)
            {
                $class_names = $matches[1];
                foreach ($class_names as $class_name)
                {
                    $used_package = $this->get_namespace_from_class_name($class_name);
                    if ($used_package && $used_package != $namespace && $used_package != '\\\\')
                    {
                        if ($used_package == 'common\libraries\package')
                        {
                            $used_package = 'common\libraries';
                        }
                        $used_packages[] = $used_package;
                    }
                }
            }
        }
    }

    /**
     * Returns the namespace from the given class name
     *
     * @param string $class_name
     *
     * @return string
     */
    protected function get_namespace_from_class_name($class_name)
    {
        $namespace_parts = explode('\\', $class_name);
        array_pop($namespace_parts);

        return implode('\\', $namespace_parts);
    }

    /**
     * Writes the used packages to a file
     *
     * @param string[] $used_packages
     */
    protected function write_used_packages_to_file($used_packages)
    {
        $base_folder = __DIR__ . '/files/';
        if (! is_dir($base_folder))
        {
            mkdir($base_folder);
        }

        $this->add_parents($used_packages, self :: MODE_GENERAL);
        $this->add_parents($used_packages, self :: MODE_INSTALL);

        $this->cleanup_used_packages($used_packages, self :: MODE_GENERAL);
        $this->cleanup_used_packages($used_packages, self :: MODE_INSTALL);

        $this->write_csv($base_folder, $used_packages);
        $this->write_graphml($base_folder, $used_packages);
    }

    protected function write_graphml($base_folder, $used_packages)
    {
        $content = array();
        $content[] = '<?xml version="1.0" encoding="UTF-8" standalone="no"?>';
        $content[] = '<graphml xmlns="http://graphml.graphdrawing.org/xmlns" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:y="http://www.yworks.com/xml/graphml" xmlns:yed="http://www.yworks.com/xml/yed/3" xsi:schemaLocation="http://graphml.graphdrawing.org/xmlns http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd">';
        $content[] = '<!--Created by yFiles for Java 2.10-->';
        $content[] = '<key for="node" id="d6" yfiles.type="nodegraphics" />';
        $content[] = '<key for="edge" id="d10" yfiles.type="edgegraphics" />';
        $content[] = '<graph edgedefault="directed" id="G">';
        $node_map = array();
        $node_id = 0;
        foreach ($used_packages as $package => $mode_packages)
        {
            $content[] = '<node id="n' . $node_id . '">';
            $content[] = '<data key="d6"><y:ShapeNode><y:NodeLabel>' . $package . '</y:NodeLabel></y:ShapeNode>';
            $content[] = '</data>';
            $content[] = '</node>';
            $node_map[$package] = $node_id;
            $node_id ++;
        }

        $edge_id = 0;
        foreach ($used_packages as $package => $mode_packages)
        {
            $source_id = $node_map[$package];
            foreach ($mode_packages[self :: MODE_GENERAL] as $mode_package)
            {
                $target_id = $node_map[$mode_package];
                $content[] = '<edge id="e' . $edge_id . '" source="n' . $source_id . '" target="n' . $target_id . '">';
                $content[] = '<data key="d10"><y:PolyLineEdge><y:Arrows source="none" target="standard" /></y:PolyLineEdge></data>';
                $content[] = '</edge>';
                $edge_id ++;
            }
        }
        $content[] = '</graph>';
        $content[] = '</graphml>';
        $create_file = fopen($base_folder . 'dependencies.graphml', 'w');
        fwrite($create_file, implode("\n", $content));
        fclose($create_file);
    }

    protected function write_csv($base_folder, $used_packages)
    {
        $csv_handle = fopen($base_folder . 'dependencies.csv', 'w');
        $csv_fields = array('Package', 'General', 'Install');
        fputcsv($csv_handle, $csv_fields, ';');

        if ($this->package)
        {
            $used_packages = array($this->package => $used_packages[$this->package]);
        }

        foreach ($used_packages as $package => $mode_packages)
        {
            $csv_fields = array(
                $package,
                implode("\n", $mode_packages[self :: MODE_GENERAL]),
                implode("\n", $mode_packages[self :: MODE_INSTALL]));
            fputcsv($csv_handle, $csv_fields, ';');
        }

        fclose($csv_handle);
    }

    protected function add_parents(&$used_packages, $mode = self :: MODE_GENERAL)
    {
        foreach ($used_packages as $package => $mode_packages)
        {
            $mode_packages = $mode_packages[$mode];

            $package_parts = explode('\\', $package);
            while (array_pop($package_parts))
            {
                $new_namespace = implode('\\', $package_parts);

                if (key_exists($new_namespace, $used_packages) && ! in_array($new_namespace, $mode_packages))
                {
                    $mode_packages[] = $new_namespace;
                    break;
                }
            }

            sort($mode_packages);
            $used_packages[$package][$mode] = $mode_packages;
        }
    }

    protected function cleanup_used_packages(&$used_packages, $mode = self :: MODE_GENERAL)
    {
        foreach ($used_packages as $package => $mode_packages)
        {
            $mode_packages = $mode_packages[$mode];
            $new_mode_packages = array();
            foreach ($mode_packages as $mode_package)
            {
                $add = true;
                $package_parts = explode('\\', $package);
                while (array_pop($package_parts))
                {
                    $new_namespace = implode('\\', $package_parts);
                    if (isset($used_packages[$new_namespace]) && isset($used_packages[$new_namespace][$mode]))
                    {
                        $new_namespace_packages = $used_packages[$new_namespace][$mode];
                    }
                    else
                    {
                        $new_namespace_packages = array();
                    }
                    if (in_array($mode_package, $new_namespace_packages))
                    {
                        $add = false;
                        break;
                    }
                }

                if ($add && $mode_package != $package && key_exists($mode_package, $used_packages))
                {
                    $new_mode_packages[] = $mode_package;
                }
            }

            sort($new_mode_packages);
            $used_packages[$package][$mode] = $new_mode_packages;
        }
    }
}
