<?php
namespace tracking;

use common\libraries\Condition;
use common\libraries\Mdb2Database;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;

/**
 * $Id: database_tracking_data_manager.class.php 231 2009-11-16 09:53:00Z vanpouckesven $
 *
 * @package tracking.lib.data_manager
 */
/**
 * This is a data manager that uses a database for storage. It was written for MySQL, but should be compatible with most
 * SQL flavors.
 *
 * @author Sven Vanpoucke
 */

class Mdb2TrackingDataManager extends Mdb2Database implements TrackingDataManagerInterface
{

    // Inherited.
    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('tracking_');
    }

    /**
     * Retrieves the tables of this database
     */
    public function get_tables()
    {
        $this->get_connection()->loadModule('Manager');
        $manager = $this->get_connection()->manager;
        return $manager->listTables();
    }

    /**
     * Creates an event in the database
     *
     * @param $event Event
     */
    public function create_event($event)
    {
        return $this->create($event);
    }

    /**
     * Creates a tracker registration in the database
     *
     * @param $trackerregistration TrackerRegistration
     */
    public function create_tracker_registration($tracker_registration)
    {
        return $this->create($tracker_registration);
    }

    /**
     * Registers a tracker to an event
     *
     * @param $eventtrackerrelation EventTrackerRelation
     */
    public function create_event_tracker_relation($event_tracker_relation)
    {
        return $this->create($event_tracker_relation);
    }

    /**
     * Creates a Tracker Setting in the database
     *
     * @param $trackersetting TrackerSetting
     */
    public function create_tracker_setting($tracker_setting)
    {
        return $this->create($tracker_setting);
    }

    /**
     * Updates an event (needed for change of activity)
     *
     * @param $event Event
     */
    public function update_event($event)
    {
        $condition = new EqualityCondition(Event :: PROPERTY_ID, $event->get_id());
        return $this->update($event, $condition);
    }

    /**
     * Updates an event tracker relation (needed for change of activity)
     *
     * @param $eventtrackerrelation EventTrackerRelation
     */
    public function update_event_tracker_relation($event_tracker_relation)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(EventRelTracker :: PROPERTY_EVENT_ID,
                $eventtrackerrelation->get_event_id());
        $conditions[] = new EqualityCondition(EventRelTracker :: PROPERTY_TRACKER_ID,
                $eventtrackerrelation->get_tracker_id());
        $condition = new AndCondition($conditions);

        return $this->update($event_tracker_relation, $condition);
    }

    /**
     * Updates an tracker setting
     *
     * @param $trackersetting TrackerSetting
     */
    public function update_tracker_setting($tracker_setting)
    {
        $condition = new EqualityCondition(TrackerSetting :: PROPERTY_ID, $tracker_setting->get_id());
        return $this->update($tracker_setting, $condition);
    }

    /**
     * Retrieves the event with the given name
     *
     * @param $name String
     * @return Event event
     */
    public function retrieve_event_by_name($event_name, $block = null)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(Event :: PROPERTY_NAME, $event_name);

        if ($block)
        {
            $conditions[] = new EqualityCondition(Event :: PROPERTY_CONTEXT, $block);
        }

        $condition = new AndCondition($conditions);

        return $this->retrieve_object(Event :: get_table_name(), $condition, array(), Event :: CLASS_NAME);
    }

    /**
     * Retrieve all trackers from an event
     *
     * @param $event_id int
     * @param $active Bool true if only the active ones should be shown (default true)
     * @return array of Tracker Registrations
     */
    public function retrieve_trackers_from_event($event_id, $active = true)
    {
        $tracker_registration_table = $this->get_table_name(TrackerRegistration :: get_table_name());
        $tracker_registration_alias = $this->get_alias(TrackerRegistration :: get_table_name());
        $event_rel_tracker_table = $this->get_table_name(EventRelTracker :: get_table_name());
        $event_rel_tracker_alias = $this->get_alias(EventRelTracker :: get_table_name());

        $query = 'SELECT ' . $tracker_registration_alias . '.* FROM ' . $tracker_registration_table . ' AS ' . $tracker_registration_alias;
        $query .= ' JOIN ' . $event_rel_tracker_table . ' AS ' . $event_rel_tracker_alias . ' ON ';
        $query .= parent :: escape_column_name(TrackerRegistration :: PROPERTY_ID, $tracker_registration_alias) . ' = ';
        $query .= parent :: escape_column_name(EventRelTracker :: PROPERTY_TRACKER_ID, $event_rel_tracker_alias);

        $conditions = array();
        $conditions[] = new EqualityCondition(EventRelTracker :: PROPERTY_EVENT_ID, $event_id,
                EventRelTracker :: get_table_name());
        if ($active)
        {
            $conditions[] = new EqualityCondition(EventRelTracker :: PROPERTY_ACTIVE, 1,
                    EventRelTracker :: get_table_name());
        }

        $condition = new AndCondition($conditions);

        $resultset = $this->retrieve_object_set($query, TrackerRegistration :: get_table_name(), $condition);
        return $resultset->as_array();
    }

    public function retrieve_event_tracker_relations($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(EventRelTracker :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, EventRelTracker :: CLASS_NAME);
    }

    /**
     * Retrieves an event tracker relation by given id's
     *
     * @param $event_id int the event id
     * @param $tracker_id int the tracker id
     * @return EventTrackerRelation that belongs to the given id's
     */
    public function retrieve_event_tracker_relation($event_id, $tracker_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(EventRelTracker :: PROPERTY_TRACKER_ID, $tracker_id);
        $conditions[] = new EqualityCondition(EventRelTracker :: PROPERTY_EVENT_ID, $event_id);
        $condition = new AndCondition($conditions);

        return $this->retrieve_object(EventRelTracker :: get_table_name(), $condition, array(),
                EventRelTracker :: CLASS_NAME);
    }

    /**
     * Retrieves a tracker registration by the given id
     *
     * @param $tracker_id int the tracker id
     * @return Tracker Registration
     */
    public function retrieve_tracker_registration($tracker_id, $active)
    {
        $condition = new EqualityCondition(TrackerRegistration :: PROPERTY_ID, $tracker_id);
        $tracker = $this->retrieve_object(TrackerRegistration :: get_table_name(), $condition, array(),
                TrackerRegistration :: CLASS_NAME);
        $tracker->set_active($active);
        return $tracker;
    }

    /**
     * Retrieves all events
     *
     * @return array of events
     */
    public function retrieve_events($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(Event :: get_table_name(), $condition, $offset, $max_objects, $order_by,
                Event :: CLASS_NAME);
    }

    /**
     * Count events for a given condition
     *
     * @param $condition Condition
     * @return Int event count
     */
    public function count_events($condition = null)
    {
        return $this->count_objects(Event :: get_table_name(), $condition);
    }

    /**
     * Retrieves an event by given id
     *
     * @param $event_id int
     * @return Event $event
     */
    public function retrieve_event($event_id)
    {
        $condition = new EqualityCondition(Event :: PROPERTY_ID, $event_id);
        return $this->retrieve_object(Event :: get_table_name(), $condition, array(), Event :: CLASS_NAME);
    }

    /**
     * Creates a tracker item in the database
     *
     * @param $tablename string the table name where the database has to be written to
     * @param $tracker_item MainTracker a subclass of MainTracker
     * @return true if creation is valid
     */
    public function create_tracker_item($tracker_item)
    {
        return $this->create($tracker_item);
    }

    /**
     *
     * @param $table_name string
     * @param $condition Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by ObjectTableOrder
     * @param $class_name string
     */
    public function retrieve_tracker_items($table_name, $condition = null, $offset = null, $max_objects = null, $order_by = array(),
            $class_name = null)
    {
        return $this->retrieve_objects($table_name, $condition, $offset, $max_objects, $order_by, $class_name);
    }

    /**
     *
     * @param $tablename string
     * @param $condition Condition
     */
    public function count_tracker_items($tablename, $condition)
    {
        return $this->count_objects($tablename, $condition);
    }

    public function count_group_tracker_items($tablename, $group, $count, $condition)
    {
        return $this->count_grouped_object($tablename, $group, $count, $condition);
    }

    /**
     * Retrieves a tracker item from the database
     *
     * @param $tablename string the table name where the database has to be written to
     * @param $id int the id of the tracker item
     * @param $classname string the tracker's class name (needed to create the class when data is retrieved)
     * @return MainTracker $tracker a subclass of MainTracker
     */
    public function retrieve_tracker_item($tablename, $classname, $id)
    {
        $condition = new EqualityCondition('id', $id);
        return $this->retrieve_tracker_item_by_condition($tablename, $condition, array(), $classname);
    }

    public function retrieve_tracker_item_by_condition($table_name, $condition, $order_by, $class_name)
    {
        return $this->retrieve_object($table_name, $condition, $order_by, $class_name);
    }

    /**
     * Updates a tracker item in the database
     *
     * @param $tracker_item MainTracker a subclass of MainTracker
     * @return true if update is valid
     */
    public function update_tracker_item($tracker_item)
    {
        $condition = new EqualityCondition('id', $tracker_item->get_id());
        return $this->update($tracker_item, $condition);
    }

    /**
     * Updates a tracker item in the database with a given condition
     *
     * @param $tracker_item Tracker a subclass of Tracker
     * @param $condition Condition a subclass of Condition
     * @return true if update is valid
     */
    public function update_tracker_item_with_condition(Tracker $tracker_item, Condition $condition)
    {
        return $this->update($tracker_item, $condition);
    }

    /**
     * Deletes tracker items in the database
     *
     * @param Condition conditon which items should be removed
     * @return true if tracker items are removed
     */
    public function remove_tracker_items($table_name, $condition = null)
    {
        return $this->delete_objects($table_name, $condition);
    }

    /**
     * Creates a archive controller item in the database
     *
     * @param ArchiveControllerItem
     * @return true if creation is valid
     */
    public function create_archive_controller_item($archive_controller_item)
    {
        return $this->create($archive_controller_item);
    }

    public function delete_events($condition = null)
    {
        return $this->delete_objects(Event :: get_table_name(), $condition);
    }

    public function delete_tracker_registrations($condition = null)
    {
        return $this->delete_objects(TrackerRegistration :: get_table_name(), $condition);
    }

    public function delete_orphaned_event_rel_tracker()
    {
        $query = 'DELETE FROM ' . $this->escape_table_name(EventRelTracker :: get_table_name()) . ' WHERE ';
        $query .= parent :: escape_column_name(EventRelTracker :: PROPERTY_EVENT_ID) . ' NOT IN (SELECT ' . parent :: escape_column_name(
                Event :: PROPERTY_ID) . ' FROM ' . $this->escape_table_name(Event :: get_table_name()) . ') OR ';
        $query .= parent :: escape_column_name(EventRelTracker :: PROPERTY_TRACKER_ID) . ' NOT IN (SELECT ' . parent :: escape_column_name(
                TrackerRegistration :: PROPERTY_ID) . ' FROM ' . $this->escape_table_name(
                TrackerRegistration :: get_table_name()) . ')';
        return $this->query($query);
    }
}
