<?php
namespace user\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\Setting;
use common\libraries\package\Package;
use common\libraries\Path;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\Registration;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\Translation;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\user\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use ($data_manager, $upgrader)
            {
                $package = Package :: get('user');
                $registration = CommonDataManager :: get_registration('user');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpdateRegistrationFailed', array(), 'core\lynx')
                    );

                    return false;
                }

                $path = Path :: namespace_to_full_path('user') . 'php/package/install/user_login_session.xml';
                $storage_unit_info = $upgrader :: parse_xml_file($path);

                if (! \user\DataManager :: create_storage_unit(
                    $storage_unit_info['name'],
                    $storage_unit_info['properties'],
                    $storage_unit_info['indexes']
                )
                )
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                            'CreateStorageUnitFailed', array('UNIT' => \user\UserLoginSession :: get_table_name()),
                            'core\lynx'
                        )
                    );

                    return false;
                }

                $attributes = array('type' => 'bigint', 'unsigned' => 1, 'notnull' => 1, 'default' => 209715200);
                $success = \user\DataManager :: alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \user\User :: get_table_name(),
                    \user\User :: PROPERTY_DISK_QUOTA,
                    $attributes
                );

                if (! $success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                            'AlterStorageUnitFailed', array(
                                'UNIT' => \user\User :: get_table_name(),
                                'PROPERTY' => \user\User :: PROPERTY_DISK_QUOTA
                            ), 'core\lynx'
                        )
                    );

                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 0, 'default' => 0);
                $success = \user\DataManager :: alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \user\User :: get_table_name(),
                    \user\User :: PROPERTY_TERMS_DATE,
                    $attributes
                );

                if (! $success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                            'AlterStorageUnitFailed', array(
                                'UNIT' => \user\User :: get_table_name(),
                                'PROPERTY' => \user\User :: PROPERTY_TERMS_DATE
                            ), 'core\lynx'
                        )
                    );

                    return false;
                }

                $tracking_storage_unit_renames = array(
                    'user_tracker' => 'user_user',
                    'user_changes_tracker' => 'user_user_change',
                    'visit_tracker' => 'user_visit',
                    'login_logout_tracker' => 'user_login_logout'
                );

                foreach ($tracking_storage_unit_renames as $old_name => $new_name)
                {
                    if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit($old_name, $new_name))
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                                'RenameStorageUnitFailed', array(
                                    'UNIT' => $old_name,
                                    'NEW_UNIT' => $new_name
                                ), 'core\lynx'
                            )
                        );

                        return false;
                    }
                }

                return true;
            }
        );

        if (! $success)
        {
            return false;
        }

        // Settings

        $setting = new Setting();
        $setting->set_context('user');
        $setting->set_user_setting(0);
        $setting->set_value('');
        $setting->set_variable('date_terms_and_conditions_update');

        if(!$setting->create())
        {
            $upgrader->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                    'CreateSettingFailed', array(
                        'Setting' => $setting->get_variable()
                    ), 'core\lynx'
                )
            );

            return false;
        }

        // Home
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \core\home\HomeBlockRegistration :: class_name(),
                \core\home\HomeBlockRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('user'));
        $home_block_registrations = \core\home\DataManager :: retrieves(
            \core\home\HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition)
        );

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            $home_block_registration->set_context('user\integration\core\home');
            $home_block_registration->update();
        }

        $package_info = Package :: get('user\integration\core\home');
        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('user\integration\core\home');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('user\integration\core\home');

        if (! $application_registration->create())
        {
            $upgrader->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                    'CreateRegistrationFailed', array('CONTEXT' => 'user\integration\core\home'), 'core\lynx'
                )
            );

            return false;
        }

        // Reporting
        $package_info = Package :: get('user\integration\reporting');

        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('user\integration\reporting');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('user\integration\reporting');

        if (! $application_registration->create())
        {
            $upgrader->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                    'CreateRegistrationFailed', array('CONTEXT' => 'user\integration\reporting'), 'core\lynx'
                )
            );

            return false;
        }

        // Tracking
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \tracking\TrackerRegistration :: class_name(),
                \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('user'));
        $tracker_registrations = \tracking\TrackingDataManager :: get_instance()->retrieves(
            \tracking\TrackerRegistration :: class_name(),
            new DataClassRetrievesParameters($condition)
        );

        while ($tracker_registration = $tracker_registrations->next_result())
        {
            $tracker_registration->set_context('user\integration\tracking');
            $tracker_registration->update();
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(\tracking\Event :: class_name(), \tracking\Event :: PROPERTY_CONTEXT),
            new StaticConditionVariable('user'));
        $event_registrations = \tracking\TrackingDataManager :: get_instance()->retrieves(
            \tracking\Event :: class_name(),
            new DataClassRetrievesParameters($condition)
        );

        while ($event_registration = $event_registrations->next_result())
        {
            $event_registration->set_context('user\integration\tracking');
            $event_registration->update();
        }

        $package_info = Package :: get('user\integration\tracking');

        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('user\integration\tracking');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('user\integration\tracking');

        if (! $application_registration->create())
        {
            $upgrader->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                    'CreateRegistrationFailed', array('CONTEXT' => 'user\integration\tracking'), 'core\lynx'
                )
            );

            return false;
        }

        $trackers = array(
            'login_logout_tracker' => 'login_logout',
            'browsers_tracker' => 'browser',
            'countries_tracker' => 'country',
            'os_tracker' => 'operating_system',
            'providers_tracker' => 'provider',
            'referrers_tracker' => 'referrer',
            'user_changes_tracker' => 'change',
            'visit_tracker' => 'visit'
        );

        foreach ($trackers as $old_tracker => $new_tracker)
        {

            $properties = new DataClassProperties();
            $properties->add(
                new DataClassProperty(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                    new StaticConditionVariable($new_tracker))
            );

            $conditions = array();
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                new StaticConditionVariable($old_tracker));
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
                new StaticConditionVariable('user\integration\tracking'));
            $condition = new AndCondition($conditions);

            if (! \tracking\TrackingDataManager :: get_instance()->updates(
                \tracking\TrackerRegistration :: class_name(),
                $properties,
                $condition
            )
            )
            {
                $upgrader->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                        'UpdateTrackerRegistrationFailed', array('TRACKER' => $old_tracker), 'core\lynx'
                    )
                );

                return false;
            }
        }

        // Webservices
        $package_info = Package :: get('user\integration\webservice');

        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('user\integration\webservice');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('user\integration\webservice');

        if (! $application_registration->create())
        {
            $upgrader->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get(
                    'CreateRegistrationFailed', array('CONTEXT' => 'user\integration\webservice'), 'core\lynx'
                )
            );

            return false;
        }

        return true;
    }
}
